( function( window, undefined ) {
	'use strict';

	/**
	 * Handles managing all events for whatever you plug it into. Priorities for hooks are based on lowest to highest in
	 * that, lowest priority hooks are fired first.
	 */
	var EventManager = function() {
		var slice = Array.prototype.slice;

		/**
		 * Maintain a reference to the object scope so our public methods never get confusing.
		 */
		var MethodsAvailable = {
			removeFilter : removeFilter,
			applyFilters : applyFilters,
			addFilter : addFilter,
			removeAction : removeAction,
			doAction : doAction,
			addAction : addAction
		};

		/**
		 * Contains the hooks that get registered with this EventManager. The array for storage utilizes a "flat"
		 * object literal such that looking up the hook utilizes the native object literal hash.
		 */
		var STORAGE = {
			actions : {},
			filters : {}
		};

		/**
		 * Adds an action to the event manager.
		 *
		 * @param action Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addAction( action, callback, priority, context ) {
			if( typeof action === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'actions', action, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs an action if it exists. You can pass as many arguments as you want to this function; the only rule is
		 * that the first argument must always be the action.
		 */
		function doAction( /* action, arg1, arg2, ... */ ) {
			var args = slice.call( arguments );
			var action = args.shift();

			if( typeof action === 'string' ) {
				_runHook( 'actions', action, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified action if it contains a namespace.identifier & exists.
		 *
		 * @param action The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeAction( action, callback ) {
			if( typeof action === 'string' ) {
				_removeHook( 'actions', action, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Adds a filter to the event manager.
		 *
		 * @param filter Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addFilter( filter, callback, priority, context ) {
			if( typeof filter === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'filters', filter, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs a filter if it exists. You should only ever pass 1 argument to be filtered. The only rule is that
		 * the first argument must always be the filter.
		 */
		function applyFilters( /* filter, filtered arg, arg2, ... */ ) {
			var args = slice.call( arguments );
			var filter = args.shift();

			if( typeof filter === 'string' ) {
				return _runHook( 'filters', filter, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified filter if it contains a namespace.identifier & exists.
		 *
		 * @param filter The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeFilter( filter, callback ) {
			if( typeof filter === 'string') {
				_removeHook( 'filters', filter, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified hook by resetting the value of it.
		 *
		 * @param type Type of hook, either 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to remove
		 * @private
		 */
		function _removeHook( type, hook, callback, context ) {
			var handlers, handler, i;

			if ( !STORAGE[ type ][ hook ] ) {
				return;
			}
			if ( !callback ) {
				STORAGE[ type ][ hook ] = [];
			} else {
				handlers = STORAGE[ type ][ hook ];
				if ( !context ) {
					for ( i = handlers.length; i--; ) {
						if ( handlers[i].callback === callback ) {
							handlers.splice( i, 1 );
						}
					}
				}
				else {
					for ( i = handlers.length; i--; ) {
						handler = handlers[i];
						if ( handler.callback === callback && handler.context === context) {
							handlers.splice( i, 1 );
						}
					}
				}
			}
		}

		/**
		 * Adds the hook to the appropriate storage container
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to add to our event manager
		 * @param callback The function that will be called when the hook is executed.
		 * @param priority The priority of this hook. Must be an integer.
		 * @param [context] A value to be used for this
		 * @private
		 */
		function _addHook( type, hook, callback, priority, context ) {
			var hookObject = {
				callback : callback,
				priority : priority,
				context : context
			};

			// Utilize 'prop itself' : http://jsperf.com/hasownproperty-vs-in-vs-undefined/19
			var hooks = STORAGE[ type ][ hook ];
			if( hooks ) {
				hooks.push( hookObject );
				hooks = _hookInsertSort( hooks );
			}
			else {
				hooks = [ hookObject ];
			}

			STORAGE[ type ][ hook ] = hooks;
		}

		/**
		 * Use an insert sort for keeping our hooks organized based on priority. This function is ridiculously faster
		 * than bubble sort, etc: http://jsperf.com/javascript-sort
		 *
		 * @param hooks The custom array containing all of the appropriate hooks to perform an insert sort on.
		 * @private
		 */
		function _hookInsertSort( hooks ) {
			var tmpHook, j, prevHook;
			for( var i = 1, len = hooks.length; i < len; i++ ) {
				tmpHook = hooks[ i ];
				j = i;
				while( ( prevHook = hooks[ j - 1 ] ) &&  prevHook.priority > tmpHook.priority ) {
					hooks[ j ] = hooks[ j - 1 ];
					--j;
				}
				hooks[ j ] = tmpHook;
			}

			return hooks;
		}

		/**
		 * Runs the specified hook. If it is an action, the value is not modified but if it is a filter, it is.
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook ( namespace.identifier ) to be ran.
		 * @param args Arguments to pass to the action/filter. If it's a filter, args is actually a single parameter.
		 * @private
		 */
		function _runHook( type, hook, args ) {
			var handlers = STORAGE[ type ][ hook ], i, len;

			if ( !handlers ) {
				return (type === 'filters') ? args[0] : false;
			}

			len = handlers.length;
			if ( type === 'filters' ) {
				for ( i = 0; i < len; i++ ) {
					args[ 0 ] = handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			} else {
				for ( i = 0; i < len; i++ ) {
					handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			}

			return ( type === 'filters' ) ? args[ 0 ] : true;
		}

		// return all of the publicly available methods
		return MethodsAvailable;

	};

	window.wp = window.wp || {};
	window.wp.hooks = new EventManager();

} )( window );
/**
 * Mixins used in the app.
 *
 * @package     ProjectHuddle
 * @subpackage  js
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.2.0
 */

/**
 * Base model for our models to extend
 */
ph.api.Model = {

  /**
   * Custom sync method for PH models
   * Set nonce header before every Backbone sync.
   * Check for user registration and trigger identify event
   *
   * @param {string} method.
   * @param {Backbone.Model} model.
   * @param {{beforeSend}, *} options.
   * @returns {*}.
   */
  sync: function (method, model, options) {
    var beforeSend;

    options = options || {};

    // Remove date_gmt if null.
    if (_.isNull(model.get('date_gmt'))) {
      model.unset('date_gmt');
    }

    // Remove slug if empty.
    if (_.isEmpty(model.get('slug'))) {
      model.unset('slug');
    }

    if ( _.isFunction( model.nonce ) && ! _.isEmpty( model.nonce() ) ) {
      beforeSend = options.beforeSend;

      // @todo enable option for jsonp endpoints
      // options.dataType = 'jsonp';

      // Include the nonce with requests.
      options.beforeSend = function (xhr) {
        xhr.setRequestHeader('X-WP-Nonce', model.nonce());

        if (beforeSend) {
          return beforeSend.apply(this, arguments);
        }
      };

      // Update the nonce when a new nonce is returned with the response.
      options.complete = function( xhr ) {
        var returnedNonce = xhr.getResponseHeader( 'X-WP-Nonce' );

        if ( returnedNonce && _.isFunction( model.nonce ) && model.nonce() !== returnedNonce ) {
          model.endpointModel.set( 'nonce', returnedNonce );
        }
      };
    }

    // Add '?force=true' to use delete method when required.
    if (this.requireForceForDelete && 'delete' === method) {
      model.url = model.url() + '?force=true';
    }

    // Post data as FormData object on create to allow file upload
    if(options.formData){
      var formData = new FormData();

      // Loop over model attributes and append to formData
      _.each(model.attributes, function(value, key){
        formData.append(key, value);
      });

      _.defaults(options || (options = {}), {
        data: formData,
        method     : 'POST',
        contentType: false,
        processData: false,
        xhr: function(){
          // get the native XmlHttpRequest object
          var xhr = jQuery.ajaxSettings.xhr();
          // set the onprogress event handler
          xhr.upload.onprogress = function(event) {
            if (event.lengthComputable) {
              // console.log('%d%', (event.loaded / event.total) * 100);
              // Trigger progress event on model for view updates
              model.trigger('progress', (event.loaded / event.total) * 100);
            }
          };
          // set the onload event handler
          xhr.upload.onload = function(){
            // console.log('complete');
            model.trigger('progress', 100);
          };
          // return the customized object
          return xhr;
        }
      });
    }
    return Backbone.sync(method, model, options);
  },

  /**
   * Build a helper function to retrieve related model.
   *
   * @param  {int}    modelId          The model ID if the object to request
   * @param  {string} modelName        The model name to use when constructing the model.
   * @param  {string} embedSourcePoint Where to check the embedds object for _embed data.
   * @param  {string} embedCheckField  Which model field to check to see if the model has data.
   *
   * @return {Deferred.promise}        A promise which resolves to the constructed model.
   */
  getter: function( modelId, modelName, embedSourcePoint, embedCheckField ) {
    var getModel, embeddeds, attributes, deferred;

    deferred  = jQuery.Deferred();
    embeddeds = this.get( '_embedded' ) || {};

    // Verify that we have a valid object id.
    if ( ! _.isNumber( modelId ) || 0 === modelId ) {
      deferred.reject();
      return deferred;
    }

    // If we have embedded object data, use that when constructing the getModel.
    if ( embeddeds[ embedSourcePoint ] ) {
      attributes = _.findWhere( embeddeds[ embedSourcePoint ], { id: modelId } );
    }

    // Otherwise use the modelId.
    if ( ! attributes ) {
      attributes = { id: modelId };
    }

    // Create the new getModel model.
    getModel = new ph.api.models[ modelName ]( attributes );

    if ( ! getModel.get( embedCheckField ) && ! embeddeds[ embedSourcePoint ]) {
      getModel.fetch( {
        success: function( getModel ) {
          deferred.resolve( getModel );
        },
        error: function( getModel, response ) {
          deferred.reject( response );
        }
      } );
    } else {
      // Resolve with the embedded model.
      deferred.resolve( getModel );
    }

    // Return a promise.
    return deferred.promise();
  }
};

ph.api.Collection = {
  save: function(key, val, options) {
    // extend options
    options = _.extend(
        {
          url: wpApiSettings.root + wpApiSettings.versionString + "batch"
        },
        options
    );

    // success/error
    var success = options.success;
    var error = options.error;

    // prepare bulk request
    var request = this.prepareBulkRequest();

    // run ajax call
    return Backbone.ajax({
      url: options.url,
      method: "PUT",
      contentType: "application/json",
      data: JSON.stringify({ requests: request }),

      beforeSend: function(xhr) {
        xhr.setRequestHeader("X-WP-Nonce", wpApiSettings.nonce);
      },

      success: _.bind(function(resp, model) {
        if (success) success.call(options.context, this, resp, options);
      }, this),

      error: function(xhr, textStatus, errorThrown) {
        if (xhr && xhr.responseJSON && xhr.responseJSON.message) {
          alert(xhr.responseJSON.message);
        } else if (textStatus !== "parsererror") {
          alert(PH_Settings.translations.generic_error);
        }
        console.error(arguments);

        if (error) error.call(options.context, xhr, textStatus, errorThrown);
      }
    });
  },

  prepareBulkRequest: function() {
    var requests = [];

    this.forEach(function(model) {
      console.log(model.changedAttributes());
      requests.push({
        method: model.isNew() ? "POST" : "PATCH",
        route: model.url(),
        params: model.changedAttributes()
      });
    });

    return requests;
  }
};


/**
 * A model for all the syncable models to extend
 *
 * Adds toast errors if they exist
 *
 * @since 1.0.0
 */
var phModel = Backbone.Model.extend({
    // log errors
    parse: function (response) {
        if (response.success == false) {
            console.error(response);
        }
        return response;
    },
    // add nonce
    sync: function (method, model, options) {
        options.beforeSend = function(xhr) {
            xhr.setRequestHeader('X-WP-Nonce', PH.nonce);
        };
        return Backbone.sync.apply(this, arguments);
    }
});

/**
 * A collection for all the syncable collections to extend
 *
 * @since 1.0.0
 */
var phCollection = Backbone.Collection.extend({
    // log errors
    parse: function (response) {
        if (response.success == false) {
            console.error(response);
        }
        return response;
    },
    // add nonce
    sync: function (method, model, options) {
        options.beforeSend = function(xhr) {
            xhr.setRequestHeader('X-WP-Nonce', PH.nonce);
        };
        return Backbone.sync.apply(this, arguments);
    }
});

/**
 * A view for all wp views to extend
 *
 * Extends wp.Backbone.View with .prepare() and .inject()
 *
 * @since 1.2.0
 */
ph.api.View = wp.Backbone.View.extend({
    inject: function( selector ) {
        this.render();
        jQuery(selector).html( this.el );
        this.views.ready();
    },

    prepare: function() {
        if ( ! _.isUndefined( this.model ) && _.isFunction( this.model.toJSON ) ) {
            this._parseTriggers();
            return this.model.toJSON();
        } else {
            return {};
        }
    },

    _parseTriggers: function () {
        if ( this.triggers ) {
            _.each( this.triggers || {}, function ( fn, event ) {

                // objects
                if ( typeof fn == 'object' ) {
                    _.each(fn, function (fn) {
                        var oHandler = this[fn];
                        if ( _.isFunction( oHandler ) ) {
                            this.stopListening( this.model, event, oHandler );
                            this.listenTo( this.model, event, oHandler );
                        }
                    }, this);
                    return false;
                }

                // regular
                var handler = this[fn];
                if ( _.isFunction( handler ) ) {
                    this.stopListening( this.model, event, handler );
                    this.listenTo( this.model, event, handler );
                }
            }, this );
        }
    }
});

/**
 * Underscore Mixins
 *
 * @since 1.0.0
 */
_.mixin( {
    /**
     * Convert /n newline to <br>
     *
     * @param str String to convert
     * @param is_xhtml Use xhtml tag
     * @returns {string} Output with <br> tag
     *
     * @since 1.0.0
     */
    nl2br: function ( str, is_xhtml ) {
        var breakTag = (is_xhtml || typeof is_xhtml === 'undefined') ? '<br />' : '<br>';
        return (str + '').replace( /([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + breakTag + '$2' );
    },

	/**
	 * Mixin to let new models extend initialize function
	 *
	 * @param oldmodel
	 * @param callback
	 */
	extendInitialize: function (oldmodel, callback) {
		// cach old init
		var cachedInit = oldmodel.prototype.initialize;

		/**
		 * Extend model prototype functions with our own
		 */
		_.extend(oldmodel.prototype, {
			initialize: function (attributes, options) {
				// do cached init
				cachedInit.call(this, attributes, options);
				// do new callback
				callback.call(this, attributes, options);
			}
		});
	},

	/**
	 * Mixin to let new models extend initialize function
	 *
	 * @param original
	 * @param extended
	 */
	phExtend: function (original, extended) {
	    var combined = {};

	    // if initialize is added, cache so we can do both
		if ( extended.initialize ) {
			var cachedOriginal = original.prototype.initialize;
			var cachedExtended = extended.initialize;
			combined = {
				initialize: function (attributes, options) {
					// do original init
					cachedOriginal.call(this, attributes, options);
					// do extended init
					cachedExtended.call(this, attributes, options);
				}
			};
        }

		// extend while combining initialize
		_.extend(original.prototype, extended, combined);
	},

    /**
     * Human readable time
     *
     * @param time
     * @returns {*}
     */
    timeSince: function (time) {
        switch (typeof time) {
            case 'number':
                break;
            case 'string':
                time = +new Date(time);
                break;
            case 'object':
                if (time.constructor === Date) time = time.getTime();
                break;
            default:
                time = +new Date();
        }

        var time_formats = [
            [60, 's', 1], // 60
            [120, '1m', '1m'], // 60*2
            [3600, 'm', 60], // 60*60, 60
            [7200, '1h', '1h'], // 60*60*2
            [86400, 'h', 3600], // 60*60*24, 60*60
            [172800, '1d', '1d'], // 60*60*24*2
            [604800, 'd', 86400], // 60*60*24*7, 60*60*24
            [1209600, '1w', '1w'], // 60*60*24*7*4*2
            [2419200, 'w', 604800], // 60*60*24*7*4, 60*60*24*7
            [4838400, '1mth', '1mth'], // 60*60*24*7*4*2
            [29030400, 'mths', 2419200], // 60*60*24*7*4*12, 60*60*24*7*4
            [58060800, 'y', '1y'], // 60*60*24*7*4*12*2
            [2903040000, 'y', 29030400] // 60*60*24*7*4*12*100, 60*60*24*7*4*12
        ];

        // use gmt
        var now     = new Date();
        var now_utc = new Date(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate(), now.getUTCHours(), now.getUTCMinutes(), now.getUTCSeconds());

        var seconds     = (+now_utc.getTime() - time) / 1000,
            token       = 'ago',
            list_choice = 1;

        if (seconds == 0) {
            return 'Now'
        }
        if (seconds < 0) {
            seconds     = Math.abs(seconds);
            token       = 'from now';
            list_choice = 2;
        }
        var i = 0,
            format;
        while (format = time_formats[i++]) {
            if (seconds < format[0]) {
                if (typeof format[2] === 'string')
                    return format[list_choice];
                else
                    return Math.floor(seconds / format[2]) + format[1];
            }
        }
        return time;
    }
} );

ph.api.setGlobal = function(data) {

    ph.api.me   = new wp.api.models.UsersMe(data.me || {});

    // global project users collection
    ph.api.users = data.users;

    ph.api.project_id = data.id;

    ph.api.comment_scroll = data.comment_scroll;

    // editor options
    ph.api.editor = {};
    ph.api.editor.editorOptions = ['bold', 'italic', 'blockquote', 'code-block', 'link', { 'list': 'ordered'}, { 'list': 'bullet' }];
    ph.api.editor.editorFormats = ['bold', 'italic', 'blockquote', 'code-block', 'link', 'list', 'code', 'image'];

    ph.api.editor.setLinkPlaceholder = function (quill) {
        var input          = quill.theme.tooltip.root.querySelector("input[data-link]");
        input.dataset.link = 'http://';
    };

    ph.api.editor.isWhitespace = function (ch) {
        var whiteSpace = false;
        if ((ch == ' ') || (ch == '\t') || (ch == '\n')) {
            whiteSpace = true;
        }
        return whiteSpace;
    };

    // Autolink URLs when typing or pasting
    ph.api.editor.autoLinkHighlight = function(quill) {
        quill.on('text-change', _.bind( function(delta, oldDelta, source) {
            var regex = /https?:\/\/[^\s]+$/;
            if(delta.ops.length === 2 && delta.ops[0].retain && ph.api.editor.isWhitespace(delta.ops[1].insert)) {
                var endRetain = delta.ops[0].retain;
                var text = quill.getText().substr(0, endRetain);
                var match = text.match(regex);

                if(match !== null) {
                    var url = match[0];

                    var ops = [];
                    if(endRetain > url.length) {
                        ops.push({ retain: endRetain - url.length });
                    }

                    ops = ops.concat([
                        { delete: url.length },
                        { insert: url, attributes: { link: url } }
                    ]);

                    quill.updateContents({
                        ops: ops
                    });
                }
            }
        }, this ));

        quill.clipboard.addMatcher(Node.TEXT_NODE, function(node, delta) {
            var regex = /https?:\/\/[^\s]+/g;
            if(typeof(node.data) !== 'string') return;
            var matches = node.data.match(regex);

            if(matches && matches.length > 0) {
                var ops = [];
                var str = node.data;
                matches.forEach(function(match) {
                    var split = str.split(match);
                    var beforeLink = split.shift();
                    ops.push({ insert: beforeLink });
                    ops.push({ insert: match, attributes: { link: match } });
                    str = split.join(match);
                });
                ops.push({ insert: str });
                delta.ops = ops;
            }

            return delta;
        });
    }
};
/**
 * App.js
 * @since 1.0
 */
'use strict';

/**
 * Start App
 * @since 3.0.0
 */
ph.api.start = function (data) {
	// store me
	ph.api.me = new ph.api.models.UserMe(data.me || {});

  // store project id
  ph.api.project_id = data.id;

	// editor options
	ph.api.editor = {
		editorOptions : ['bold', 'italic', 'blockquote', 'code-block', 'link', {'list': 'ordered'}, {'list': 'bullet'}],
		editorFormats : ['bold', 'italic', 'blockquote', 'code-block', 'link', 'list', 'code', 'image'],
	};

	// create website container view and inject
	ph.api.container = new ph.api.views.Website({
		model: new ph.api.models.Website(data)
	});
	ph.api.container.inject('#project_pages_container');
};

ph.api.url = wpApiSettings.root + 'projecthuddle/v1/';

ph.api.setLinkPlaceholder = function (quill) {
	var input          = quill.theme.tooltip.root.querySelector("input[data-link]");
	input.dataset.link = 'http://';
};

ph.api.isWhitespace = function (ch) {
	var whiteSpace = false;
	if ((ch == ' ') || (ch == '\t') || (ch == '\n')) {
		whiteSpace = true;
	}
	return whiteSpace;
};

// Autolink URLs when typing or pasting
ph.api.autoLinkHighlight = function (quill) {
	quill.on('text-change', _.bind(function (delta, oldDelta, source) {
		var regex = /https?:\/\/[^\s]+$/;
		if (delta.ops.length === 2 && delta.ops[0].retain && this.isWhitespace(delta.ops[1].insert)) {
			var endRetain = delta.ops[0].retain;
			var text      = quill.getText().substr(0, endRetain);
			var match     = text.match(regex);

			if (match !== null) {
				var url = match[0];

				var ops = [];
				if (endRetain > url.length) {
					ops.push({retain: endRetain - url.length});
				}

				ops = ops.concat([
					{delete: url.length},
					{insert: url, attributes: {link: url}}
				]);

				quill.updateContents({
					ops: ops
				});
			}
		}
	}, this));

	quill.clipboard.addMatcher(Node.TEXT_NODE, function (node, delta) {
		var regex = /https?:\/\/[^\s]+/g;
		if (typeof(node.data) !== 'string') return;
		var matches = node.data.match(regex);

		if (matches && matches.length > 0) {
			var ops = [];
			var str = node.data;
			matches.forEach(function (match) {
				var split      = str.split(match);
				var beforeLink = split.shift();
				ops.push({insert: beforeLink});
				ops.push({insert: match, attributes: {link: match}});
				str = split.join(match);
			});
			ops.push({insert: str});
			delta.ops = ops;
		}

		return delta;
	});
};

/**
 * Declare global events in namespace
 */
ph.api.Event = _.extend({}, Backbone.Events);
/**
 * Model to store our comment filter data
 *
 * @since 2.5.0
 */
ph.api.models.CommentFilter = Backbone.Model.extend({
    defaults: {
        'name': '',
        'active': false
    }
});
/**
 * Model to store our individual comment item
 *
 * @since 2.0.0
 */
ph.api.models.Comment = wp.api.models.Comment.extend(_.defaults({
    // extend defaults
	defaults: _.defaults({}, wp.api.models.Comment.prototype.defaults, {
        'edit'    : false,
        'moderate': false
    }),
    initialize: function (attributes, options) {
		  wp.api.models.Comment.prototype.initialize.call(this, attributes, options);

        this.set({
            'humanDate': this.get('date_gmt') ? _.timeSince(this.get('date_gmt')) : 'Now',
            'moderate': this.get('author') === ph.api.me.get('id')
        });

		// hook into init
		wp.hooks.doAction('ph.api.models.Comment.initialize', this);
    }
}, ph.api.Model));
/**
 * Model to store our individual comment item
 *
 * @since 2.0.0
 */
ph.api.models.User = wp.api.models.User.extend(_.defaults({
	noRegister: true,
	can: function(cap) {
		var caps = this.get('capabilities');

		if (_.isEmpty(caps)) {
			return false;
		}

		return caps[cap];
	}
}, ph.api.Model));

ph.api.models.UserMe = wp.api.models.UsersMe.extend(_.defaults({
	noRegister: true,
	can: function(cap) {
		var caps = this.get('capabilities');

		if (_.isEmpty(caps)) {
			return false;
		}

		return caps[cap];
	}
}, ph.api.Model));
/**
 * Website Thread
 */
ph.api.models.WebsitePage = wp.api.models.WebsitePage.extend({
  // extend defaults
  defaults: _.extend({}, wp.api.models.WebsitePage.prototype.defaults, {
    'comment_count'   : 0,
    'resolved_count'  : 0,
    'filter'          : 'all',
    'unresolved_count': 0,
    'parent_id'       : 0,
    'controls'        : '',
    'commentsEnabled' : false,
  }),

  threads: {},

  filters: {},

  initialize: function(attributes, options) {
    // init parent
    wp.api.models.WebsitePage.prototype.initialize.call(this, attributes,
        options);
    this.requireForceForDelete = false; // don't force delete

    // store threads outside attributes
    this.set('threads',
        new ph.api.collections.WebsiteThread(this.get('threads') || []));

    this.listenTo(this, 'change:filter', this.counts);

    // when threads are added to the collection, set the order in the model automatically
    this.listenTo(this.get('threads'), 'change:resolved add remove',
        this.counts);

    this.counts();

    this.listenTo(this.get('threads'), 'sorted', this.updateOrder);
  },

  // model counts
  counts: function() {
    this.set('comment_count', this.get('threads').length);
    this.set('resolved_count', this.resolvedThreads().length);
    this.set('unresolved_count', this.unresolvedThreads().length);
  },

  unresolvedThreads: function() {
    return this.get('threads')['filter_' + this.get('filter')]().
        where({resolved: false});
  },

  resolvedThreads: function() {
    return this.get('threads')['filter_' + this.get('filter')]().
        where({resolved: true});
  },

  filteredThreads: function() {
    return this.get('threads')['filter_' + this.get('filter')]();
  },

  // selected filter model
  filter: function(model) {
    this.set('filter', model.attributes);
  },

  // Update the order attribute for all comments
  updateOrder: function(model, position) {
    this.get('threads').remove(model);

    this.get('threads').each(function(model, index) {
      var ordinal = index;
      if (index >= position) {
        ordinal += 1;
      }
      model.unset('menu_order', {silent:true}).set('menu_order', ordinal);
    });

    model.unset('menu_order', {silent:true}).set('menu_order', position);
    this.get('threads').add(model, {at: position});
    this.get('threads').save();
  },
});
/**
 * Website Thread
 */
ph.api.models.WebsiteThread = wp.api.models.WebsiteThread.extend(_.defaults({
  // extend defaults
  defaults: _.defaults({},
      wp.api.models.WebsiteThread.prototype.defaults, {
        title             : '',
        author_avatar_urls: {},
        page_url          : '',
        comments_pages    : 0,
        formContent       : '',
        author_name       : '',
        website_id        : 0,
        controls          : false,
        focus             : false,
        highlight         : false,
        show              : false,
        diff              : 0,
        assigned          : 0,
        assigned_model    : {},
        humanDate         : '',
        order             : 0,
      }),

  comparator: function(model) {
    return model.get('menu_order');
  },

  users: {},

  initialize: function(attributes, options) {
    // init parent
    wp.api.models.WebsiteThread.prototype.initialize.call(
        this,
        attributes,
        options,
    );

    this.requireForceForDelete = false; // don't force delete

    this.set('assigned_model', new ph.api.models.User());
    this.set('users', new ph.api.collections.Users());
    this.get('users').comparator = function(model) {
      return model.get('assigned') ? 0 : 1;
    };
    this.get('users').sort();

    // set listeners
    this.listeners();

    // set website ID in model
    this.setWebsiteID();

    // set comments in model
    this.setComments();

    this.getAuthorUser().done(
        _.bind(function(user) {
          this.set({
            author_name       : user.get('name'),
            author_avatar_urls: user.get('avatar_urls'),
          });
        }, this),
    );

    this.set({humanDate: _.timeSince(this.get('date'))});

    this.hasMoreComments();
    this.listenTo(this.get('comments'), 'sync', this.hasMoreComments);

    // hook into init
    wp.hooks.doAction('ph.api.models.WebsiteThread.initialize', this);
  },

  hasMoreComments: function() {
    this.set('hasMore', Boolean(this.get('comments').hasMore()));
  },

  // model listeners
  listeners: function() {
    // update assignments
    this.listenTo(this.get('users'), 'assign', this.assign);
    this.listenTo(this.get('users'), 'unAssign', this.unAssign);
    this.listenTo(
        this.get('users'),
        'update assign unAssign',
        this.selectedUser,
    );

    // listen to submit
    this.listenTo(this, 'submit', this.saveThread);
    this.listenTo(this, 'submit', this.createComment);

    this.listenTo(this, 'change:show', this.maybeDestroy);
  },

  // update comments subcollection on parse
  parse: function(response, options) {
    // rule out patch requests that don't contain comments
    if (options.patch === true) {
      if (typeof options.attrs.comments === 'undefined') {
        response.comments = this.get('comments');
        return response;
      }
    }

    // handle comments sync
    if (response.comments && this.get('comments')) {
      // set models comments to response
      this.get('comments').set(response.comments);
      response.comments = this.get('comments');
    }
    return response;
  },

  // maybe destroy thread
  maybeDestroy: function() {
    if (this.get('show')) return; // only if we're hiding

    if (
        this.get('comment_content') &&
        this.get('comment_content') !== '<p><br></p>'
    ) {
      this.confirmDelete(); // confirm delete if form content
    } else if (!this.get('comments').length) {
      this.destroyThread(); // else if there are no comments, destroy automatically
    }
  },

  confirmDelete: function() {
    vex.dialog.confirm({
      message : PH_Settings.translations.discard,
      callback: function(value) {
        if (value) {
          if (this.isNew()) {
            this.destroyThread();
          } else {
            this.set({
              comment_content: '',
              controls       : false,
              show           : false,
            }).trigger('resetTextArea');
          }
        }
      }.bind(this),
    });
  },

  // destroy thread model
  destroyThread: function() {
    // if (ph.api.hooks.applyFilters('ph_destroy_thread', true, this)) {
    this.destroy();
    // }
  },

  /**
   * Store website_id in model
   */
  setWebsiteID: function() {
    this.set('website_id', ph.api.project_id);
  },

  /**
   * Set initial comment load state
   */
  initialState: function() {
    // has been initially fetched
    if (this.get('comments').state &&
        this.get('comments').state.totalPages) {
      return;
    }

    // set currents from load
    this.get('comments').state = {
      totalObjects: this.get('comments_count'),
      currentPage : 1,
      totalPages  : Math.ceil(this.get('comments_count') /
          this.get('comments').length),
    };
  },

  /**
   * Set comments in model
   */
  setComments: function() {
    if (!this.get('comments')) {
      this.set('comments', new ph.api.collections.Comments([]));
      return;
    }

    this.set(
        'comments',
        new ph.api.collections.Comments(this.get('comments') || []),
    );

    this.initialState();

    // update count
    this.listenTo(
        this.get('comments').state.totalObjects,
        'change',
        this.updateCount,
    );
  },

  /**
   * Set the assigned user in the model
   */
  getAssigned: function() {
    this.getter(
        this.get('assigned'),
        'User',
        'assigned',
        'assigned_model',
    ).done(
        _.bind(function(user) {
          this.set('assigned_model', user);
        }, this),
    );
  },

  /**
   * Save thread
   */
  saveThread: function() {
    if (!this.isNew()) {
      return;
    }

    var content = this.get('formContent') ?
        this.get('formContent') :
        this.get('title');

    // add comment to response
    this.get('comments').add(
        wp.hooks.applyFilters('ph_new_comment_data', {
          author_name       : ph.api.me.get('name'),
          content           : content,
          author            : ph.api.me.get('id'),
          author_avatar_urls: ph.api.me.get('avatar_urls'),
        }, this),
        {validate: true},
    );

    // get page url minus hash
    var page_url = window.parent.location.href.toString().
        replace(window.parent.location.hash, '');

    // save thread with comment
    this.save(wp.hooks.applyFilters('ph_new_thread_data', {
          page_url  : page_url,
          page_title: parent.document.title,
          content   : this.get('comments').first().get('content'),
          slug      : this.get('id'),
          website_id: ph.api.project_id,
          status    : 'publish',
          title     : this.get('comments').first().get('content'),
          author    : ph.api.me ? ph.api.me.get('id') : 0,
        }, this), {
          success: _.bind(function(model) {
            this.set('formContent', '');
            PH_Website_Settings.debug && console.log('New Thread:', model); // debug
          }, this),
          error  : this.trigger('error'),
        },
    );

    ph.api.me.get('id') && PH_Website_Settings.options.global.auto_close &&
    this.autoHide();
  },

  /**
   * Create/Add Comment
   */
  createComment: function() {
    if (this.isNew()) {
      return;
    }

    // create comment
    this.get('comments').create(
        wp.hooks.applyFilters('ph_new_comment_data', {
          author_name       : ph.api.me.get('name'),
          content           : this.get('formContent'),
          post              : this.get('id'),
          author            : ph.api.me.get('id'),
          author_avatar_urls: ph.api.me.get('avatar_urls'),
        }, this),
        {validate: true},
    );
  },

  /**
   * Update item count in model
   */
  updateCount: function() {
    this.set('comments_count', this.get('comments').state.totalObjects);
  },

  /**
   * Set/save assignment
   * @param model User model
   */
  assign: function(model) {
    var method = this.isNew() ? 'set' : 'save';
    this[method](wp.hooks.applyFilters('ph_assign_thread_data', {
          assigned: model.get('id'),
        }),
        {patch: true},
    );

    this.set({assigned_model: new ph.api.models.User(model.toJSON())});
  },

  /**
   * Set/save unassignment
   */
  unAssign: function() {
    var method = this.isNew() ? 'set' : 'save';
    this[method](
        wp.hooks.applyFilters('ph_unassign_thread_data', {
          assigned: 0,
        }),
        {patch: true},
    );

    this.set({assigned_model: new ph.api.models.User()});
  },

  /**
   * Set assigned user in sub collection
   */
  selectedUser: function() {
    this.get('users').invoke('set', {assigned: false}); // set all as unassigned
    this.assigned = this.get('users').get(this.get('assigned'));
    this.assigned && this.assigned.set({assigned: true});
    this.get('users').sort(); // sort by assigned first
  },

  /**
   * Auto hide comment after 1.5 seconds
   */
  autoHide: function() {
    this.hiding = _.delay(
        _.bind(function() {
          this.set({show: false});
        }, this),
        1500,
    ); // wait a second and a half
  },

  /**
   * Kill autohide timer
   */
  cancelAutoHide: function() {
    clearTimeout(this.hiding); //Kill the timer
  },
}, ph.api.Model));

/**
 * Website model
 * Stores entire website project
 */
ph.api.models.Website = wp.api.models.Website.extend({
  // extend defaults
  defaults: _.extend({}, wp.api.models.Website.prototype.defaults, {
    filter: 'all',
  }),

  pages: {},

  initialize: function(attributes, options) {
    wp.api.models.Website.prototype.initialize.call(this, attributes, options);
    this.requireForceForDelete = false; // don't force delete

    // all pages
    this.set('pages',
        new ph.api.collections.WebsitePage(this.get('pages') || []));

    this.listenTo(this.get('pages'), 'sorted', this.updateOrder);

    this.listenTo(this, 'change:filter', this.updatePages );
  },

  updatePages: function (model) {
    this.get('pages').invoke('set', {'filter': model.get('filter')});
  },

  // Update the order attribute for all comments
  updateOrder: function(model, position) {
    this.get('pages').remove(model);

    this.get('pages').each(function(model, index) {
      var ordinal = index;
      if (index >= position) {
        ordinal += 1;
      }
      model.unset('menu_order', {silent:true}).set('menu_order', ordinal);
    });

    model.unset('menu_order', {silent:true}).set('menu_order', position);
    this.get('pages').add(model, {at: position});
    this.get('pages').save();
  },
});
ph.api.collections.CommentFilters = Backbone.Collection.extend({
    model: ph.api.models.CommentFilter
});
ph.api.collections.Users = wp.api.collections.Users.extend({
    model: ph.api.models.User
});
ph.api.collections.Comments = wp.api.collections.Comments.extend({
	model: ph.api.models.Comment,

	comparator: function (model) {
		return model.get('date');
	},

	initialize: function( attributes, options ) {
		// init parent
		wp.api.collections.Comments.prototype.initialize.call(this, attributes, options);
		this.listenTo(this, 'add update remove', this.sort);
	},
});
ph.api.collections.WebsitePage = wp.api.collections.WebsitePage.extend(
    _.defaults({
      model: ph.api.models.WebsitePage,

      comparator: function(model) {
        return model.get('menu_order');
      },

      initialize: function(models, options) {
        wp.api.collections.WebsitePage.prototype.initialize.apply(models,
            options);
        this.listenTo(this, 'add update remove', this.sort);
      },
    }, ph.api.Collection),
);
/**
 * Website Threads
 */
ph.api.collections.WebsiteThread = wp.api.collections.WebsiteThread.extend(
    _.defaults({
      model: ph.api.models.WebsiteThread,

      comparator: function(model) {
        return model.get('menu_order');
      },

      initialize: function(attributes, options) {
        wp.api.collections.WebsiteThread.prototype.initialize.call(this,
            attributes,
            options);
        this.listenTo(this, 'add update remove', this.sort);
      },

      // filter
      filter_all: function() {
        return this;
      },

      filter_mine: function() {
        if (!ph.api.me) {
          return this;
        }
        return new ph.api.collections.WebsiteThread(
            _.filter(this.models, function(thread) {
              return parseInt(thread.get('assigned')) ===
                  parseInt(ph.api.me.get('id'));
            }));
      },

      filter_unassigned: function() {
        return new ph.api.collections.WebsiteThread(
            _.filter(this.models, function(thread) {
              return !thread.get('assigned');
            }));
      },
    }, ph.api.Collection),
);
/**
 * View to show our individual comment item
 *
 * @since 1.0
 */
ph.api.views.MinimalCommentItem = ph.api.View.extend({
	template : wp.template('project-minimal-comment-item'),
	className: 'ph-comment-item minimal-comment',

	events: {
		'click .ph-remove': 'removeComment',
	},

	triggers: {
		'change:moderate': 'commentControls'
	},

	initialize: function () {
		// comment item text view
		this.views.set('.ph-comment-item-text', new ph.api.views.CommentItemText({
			model: this.model
		}));
	},

	/**
	 * Remove comment click handler - let the collection handle itself
	 */
	removeComment: function (e) {
		e.preventDefault();
		confirm(PH_Website_Settings.translations.are_you_sure) && this.destroyComment();
	},

	/**
	 * Animate removal
	 */
	destroyComment: function () {
		this.$el.slideToggle(250, _.bind(function () {
			this.remove();
			this.unbind();
			this.model.destroy();
		}, this));
	}
});
/**
 * Assign View Option Item
 */
ph.api.views.AssignOption = ph.api.View.extend({
    className: 'ph-single-user-option-wrap',
    template : wp.template('ph-assign-dropdown-option'),

    events: {
        'click': 'assign',
        'click .ph-close': 'unAssign'
    },

    triggers: {
        'change:assigned': 'render'
    },

    assign: function () {
        this.model.trigger('assign', this.model);
        return false;
    },

    unAssign: function() {
        this.model.trigger('unAssign', this.model);
        return false;
    }
});
/**
 * Panel Assign View Dropdown
 */
ph.api.views.AssignOptions = ph.api.View.extend({
  className: 'ph-assign-options-select',
  template : wp.template('ph-loading'),

  initialize: function() {
    this.setOptions();
    this.listenTo(this.model.get('users'), 'update', this.setOptions);
  },

  setOptions: function() {
    var views = [];

    if (this.model.get('users').length) {
      this.model.get('users').forEach(function(user) {
        views.push(new ph.api.views.AssignOption({
          model: user,
        }));
      }, this);
      this.views.set(views);
    }
  },
});
/**
 * Assign View Dropdown
 */
ph.api.views.Assign = ph.api.View.extend({
  template : wp.template('assign'),
  className: 'ph-comment-assigned',

  events: {
    'click': 'open',
  },

  triggers: {
    'change:assigned_model': 'render',
    'change:assigned': ['render', 'close']
  },

  initialize: function() {
    this.views.add('.ph-assign-options', new ph.api.views.AssignOptions({
      model: this.model,
    }));
  },

  open: function(e) {
    this.$el.toggleClass('ph-open');
    jQuery('.ph-open').not(this.$el).removeClass('ph-open');

    this.model.get('users').fetch({
      data: {
        project : ph.api.project_id,
        per_page: 100,
      },
    });

    e.stopPropagation();
  },

  close: function() {
    this.$el.removeClass('ph-open');
  },
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.Avatar = ph.api.View.extend({
    template : wp.template('avatar'),
    className: 'ph-avatar'
});
/**
 * Comment Filter View
 *
 * @since 1.0
 */
ph.api.views.CommentFilter = ph.api.View.extend({
  template : wp.template('comment-filter'),
  className: 'comment-switcher-dropdown-wrap',

  triggers: {
    'change:filter': ['render', 'hide'],
  },

  events: {
    'click .ph-show-all-comments'       : 'all',
    'click .ph-show-my-comments'        : 'mine',
    'click .ph-show-unassigned-comments': 'unassigned',
    'mouseenter'                        : 'show',
    'mouseleave'                        : 'hide',
  },

    all: function () {
        this.model.set('filter', 'all');
    },

    mine: function () {
        this.model.set('filter', 'mine');
    },

    unassigned: function () {
        this.model.set('filter', 'unassigned');
    },

  show: function() {
    this.$el.addClass('ph-show');
  },

  hide: function() {
    this.$el.removeClass('ph-show');
  },
});
/**
 * View to show our comment form
 *
 * @since 1.0
 */
ph.api.views.CommentForm = ph.api.View.extend({
    className: 'comment-textarea',
    tagName: 'div',

    events: {
        "keydown .ql-editor": "keyDownHandler",
        "focus .ql-editor": "textAreaFocus"
    },

    /**
     * Model trigger handlers
     */
    triggers: {
        'resetTextArea': 'resetTextArea',
        'focusTextArea': [
            'enableTextArea',
            'resetTextArea',
            'focusTextArea'
        ],
        'change:controls': 'focusClass',
        'change:focus': 'focusHandler'
    },

    initialize: function () {
        this.listenTo(this.model.get('comments'), 'add', this.resetTextArea);
    },

    ready: function () {
        this.setQuill();
    },

    setQuill: function () {
        this.quill = new Quill(this.$el.get(0), {
            modules: {toolbar: ph.api.editor.editorOptions},
            formats: this.editorFormats,
            placeholder:  PH.translations.comment_placeholder || 'Leave a Comment',
            theme  : 'bubble',
            bounds : this.el
        });
        ph.api.autoLinkHighlight(this.quill);
        ph.api.setLinkPlaceholder(this.quill);

        // on text-change
        this.quill.on('text-change', _.bind(this.contentChangedHandler, this));
    },

    textAreaFocus: function () {
        this.model.set('controls', 'true');
    },

    focusClass: function () {
        this.$el.toggleClass('focus', this.model.get('controls'));
    },

    disableTextArea: function () {
        this.quill.disable();
    },

    enableTextArea: function () {
        this.quill.enable();
    },

    focusTextArea: function () {
        this.quill.focus();
    },

    resetTextArea: function () {
        this.quill.setContents([{insert: '\n'}]);
        this.model.set('comment_content', this.$('.ql-editor').html());
        this.$('.ql-tooltip').addClass('ql-hidden'); // manually hide
    },

    keyDownHandler: function (e) {
        if (e.shiftKey && e.keyCode == 13) { // shift + enter
            e.preventDefault();
            this.model.trigger('submitComment');
        }
    },

    contentChangedHandler: function (delta, oldDelta, source) {
        this.model.set('comment_content', this.$('.ql-editor').html());
    },

    focusHandler: function () {
        this.model.get('focus') && this.quill.focus();
    }
});
/**
 * View to show our comment button
 *
 * @since 1.0
 */
ph.api.views.CommentSubmitButton = ph.api.View.extend({
    template : wp.template('submit-new-comment'),
    className: 'button button-primary button-large',
    tagName  : 'button',

    events: {
        'click': 'buttonClickHandler'
    },

    /**
     * Model trigger handlers
     */
    triggers: {
        'change:controls': 'showHide'
    },

    /**
     * Handle Button Click
     */
    buttonClickHandler: function (e) {
        e.preventDefault();
        if (!this.model.get('comment_content') || this.model.get('comment_content') === '<p><br></p>') {
            return false;
        }
        this.createItem();
        return false;
    },

    createItem: function () {
        this.model.get('comments').create({
            'author_name'       : ph.api.me.get('name'),
            'content'           : this.model.get('comment_content'),
            'post'              : this.model.get('id'),
            'author'            : ph.api.me.get('id'),
            'author_avatar_urls': ph.api.me.get('avatar_urls')
        });
        this.model.set('comment_content', '');
    },

    showHide: function () {
        if (!this.model.get('controls')) {
            this.$el.hide();
        } else {
            this.$el.show();
        }
    }
});

/**
 * View to show our individual comment item
 *
 * @since 1.0
 */
ph.api.views.Comment = ph.api.View.extend({
    template : wp.template('info-comment-item'),
    className: 'ph-comment',

    events: {
        'click .ph-remove': 'removeComment',
        'click .ph-edit'  : 'editComment'
    },

    initialize: function () {
        // comment item text view
        this.views.set('.ph-comment-item-text', new ph.api.views.CommentItemText({
            model: this.model
        }));

        // edit comment form
        this.views.set('.ph-edit-textarea', new ph.api.views.CommentEditForm({
            model: this.model
        }));

        // comment item text view
        if (this.model.get('author') === ph.api.me.get('id')) {
            this.views.set('.ph-item-controls', new ph.api.views.CommentItemControls({
                model: this.model
            }));
        }
    },

    /**
     * Remove comment click handler - let the collection handle itself
     */
    removeComment: function (e) {
        e.preventDefault();
        confirm("Are you sure?") && this.destroyComment();
    },

    /**
     * Shows the edit textbox, focuses and places cursor at end
     */
    editComment: function () {
        this.model.set('edit', true);
    },

    /**
     * Animate removal
     */
    destroyComment: function () {
        this.$el.slideToggle(250, _.bind(function () {
            this.remove();
            this.unbind();
            this.model.destroy();
        }, this));
    }
});

/**
 * View to show our individual comment item text
 *
 * @since 1.0
 */
ph.api.views.CommentItemText = ph.api.View.extend({
    template : wp.template('comment-item-text'),
    className: 'comment-item-text',

    triggers: {
        'change:content': 'render',
        'change:edit'   : ['showHide', 'render']
    },

    showHide: function () {
        this.$el.toggle(!this.model.get('edit'));
    }
});

/**
 * View to show our individual comment item edit box
 *
 * @since 1.0
 */
ph.api.views.CommentEditForm = ph.api.View.extend({
    template : wp.template('edit-comment-form'),
    className: 'ph-edit-form',

    events: {
        'click .ph-edit-cancel'      : 'cancelEdit',
        'click .ph-edit-save:enabled': 'updateComment'
    },

    triggers: {
        'change:edit': ['toggleEdit', 'setQuill'],
        'change:content': ['render']
    },

    ready: function () {
        this.toggleEdit();
    },

    setQuill: function () {
        this.quill = new Quill(this.$('.ph-edit-content').get(0), {
            modules: {toolbar: ph.api.editor.editorOptions},
            formats: this.editorFormats,
            theme  : 'bubble',
            bounds : this.el
        });
        ph.api.autoLinkHighlight(this.quill);
        ph.api.setLinkPlaceholder(this.quill);
    },

    toggleEdit: function () {
        this.$el.toggle(this.model.get('edit'));
    },

    cancelEdit: function () {
        this.model.set('edit', false);
        return false;
    },

    updateComment: function () {
        this.model.save({
            'content': this.$('.ql-editor').html(),
            'edit'           : false // done editing
        });
        return false;
    }
});

/**
 * View to show our individual comment item text
 *
 * @since 1.0
 */
ph.api.views.CommentItemControls = ph.api.View.extend({
    template : wp.template('comment-item-controls'),
    className: 'comment-item-controls-inner',

    initialize: function () {
        this.listenTo(this.model, 'change:edit', this.showHide);
    },

    showHide: function () {
        this.$el.toggle(!this.model.get('edit'));
    }
});
ph.api.views.CommentsContainer = ph.api.View.extend({
	className: 'ph-comment-text',

	triggers: {
		'change:show': 'scrollBottom', // when showInfo/hide scroll to bottom
	},

	// element ui defaults
	savedScrollHeight  : 0,
	savedScrollPosition: 0,
	loading            : false,

	initialize: function () {
		// comment items
		this.setComments();

		// comments added, defer
		this.listenTo(this.model.get('comments'), 'update', this.setComments);

		// scroll to bottom on add update sync
		this.listenTo(this.model.get('comments'), 'add update sync', this.scrollBottom);

		// bind this
		_.bindAll(this, 'pagination', 'scrollBottom', 'saveScrollPosition', 'restoreScrollPosition');
	},

	ready: function () {
		// scroll pagination events
		this.$el.on('scroll', _.debounce(this.pagination, 350));
	},

	// set comment subviews
	setComments: function () {
		var views = [];
		this.model.get('comments').forEach(function (item) {
			if (item.get('type') && item.get('type') === 'ph_approval') {
				views.push(new ph.api.views.MinimalCommentItem({
					model : item,
					iframe: this.iframe
				}));
			} else {
				views.push(new ph.api.views.Comment({
					model : item,
					iframe: this.iframe
				}));
			}
		}, this);
		this.views.set(views);
	},

	// scroll to bottom of list
	scrollBottom: function (collection, response, options) {
		options || (options = {});
		options.scroll = _.isUndefined(options.scroll) ? true : options.scroll;
		if (options.scroll) {
			typeof this.$el !== 'undefined' && this.$el.scrollTop(this.$el[0].scrollHeight);
			this.$el.focus();
		}
	},

	// do pagination
	pagination: function (e) {
		// must be showInfoing to do pagination
		if (!this.model.get('show')) {
			return;
		}

		if (!this.model.get('comments').hasMore()) {
			return;
		}

		// if we've reached the top and we're not loading
		if (this.$el.scrollTop() < 250 && !this.loading) {
			this.loading = true;

			// save and restore scroll position
			this.listenToOnce(this.model.get('comments'), 'add', this.saveScrollPosition);
			this.listenToOnce(this.model.get('comments'), 'update', this.restoreScrollPosition);

			this.model.get('comments').more({
				data   : {
					per_page: this.model.get('comments').length/this.model.get('comments').state.currentPage,
					post    : this.model.get('id'),
					page    : this.model.get('comments').hasMore() === null ? 2 : undefined
				},
				remove : false,
				scroll : false,
				loading: true,
				success: _.bind(function () {
					this.loading = false;
				}, this)
			});
		}
	},

	// save current scroll position
	saveScrollPosition: function (collection, response, options) {
		options || (options = {});
		options.scroll = _.isUndefined(options.scroll) ? true : options.scroll;

		if (!options.scroll) {
			this.savedScrollHeight   = this.$el.prop("scrollHeight");
			this.savedScrollPosition = this.$el.scrollTop();
		}
	},

	// restore saved scroll position
	restoreScrollPosition: function (collection, options) {
		options || (options = {});
		options.scroll = _.isUndefined(options.scroll) ? true : options.scroll;
		if (!options.scroll) {
			this.$el.scrollTop(this.$el.prop("scrollHeight") - this.savedScrollHeight + this.savedScrollPosition);
		}
	}
});
/**
 * Members view
 *
 * @since 2.6.0
 */
ph.api.views.Members = ph.api.View.extend({
  events: {
    'select2:select'  : 'members',
    'select2:unselect': 'members',
  },

  initialize: function() {
    this.setElement(this.el);
    this.render();
    _.bindAll(this, 'members');
  },

  /**
   * Save new member on select
   *
   * @param e
   */
  members: function(e) {
    var data = _.isUndefined(e.params.data)? e.params : e.params.data;
    var newList = data.selected ? _.union(this.model.get('project_members'), [parseInt(data.id)]) : _.without(this.model.get('project_members'),parseInt(data.id))

    this.model.save({
      'project_members': newList,
    }, {patch: true});
  },
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.Notifications = ph.api.View.extend({
    events: {
        'change #ph_project_email_notifications_disable_all': 'toggle'
    },

    initialize: function () {
        this.setElement(this.el);
        this.render();
        _.bindAll(this, 'email');
    },

    toggle: function(e) {
        this.email(jQuery(e.target).prop('checked'));
    },

    /**
     * Ajax email notifications
     */
    email: function (checked) {
        // update via ajax
        jQuery.ajax({
            type      : checked ? "PUT" : "DELETE", // post if selected, delete if unselected
            beforeSend: function (xhr) {
                xhr.setRequestHeader('X-WP-Nonce', wpApiSettings.nonce);
            },
            url       : ph.api.url + 'suppression/project/' + this.model.get('id')
        });
    }
});
/**
 * Members view
 *
 * @since 2.6.0
 */
ph.api.views.Options = ph.api.View.extend({
	events: {
		'change .ph_project_access input'  : 'updateAccess',
		'change .ph_toolbar_location input': 'updateToolbar'
	},

	initialize: function () {
		this.setElement(this.el);
		this.render();
	},

	updateAccess: function (e) {
		this.model.save({
			'project_access': e.target.value,
		}, {
			patch: true
		});
	},

	updateToolbar: function(e) {
		this.model.save({
			'toolbar_location': e.target.value,
		}, {
			patch: true
		});
	}
});

/**
 * Website list
 *
 * Subview of the panel view that shows an individual comment in a list
 *
 * @since 1.2.0
 */
ph.api.views.ResolveBox = ph.api.View.extend({
  template : wp.template('ph-resolve-check'),
  className: 'ph-resolve',
  tagName  : 'div',

  events: {
    'click': 'toggleResolve',
  },

  triggers: {
    'change:resolved': 'render',
  },

  toggleResolve: function(e) {
    this.model.save({
      'resolved': !this.model.get('resolved'),
    }, {
      patch: true,
      success: _.bind(function() {
        this.model.get('comments').fetch({
          data: {
            post: this.model.get('id'),
            per_page: 25,
          },
        });
      }, this)
    });
    return false;
  },
});
/**
 * Resolved tab view
 *
 * @since 1.0
 */
ph.api.views.ResolvedTab = ph.api.View.extend({
    template : wp.template('website-tab'),
    className: 'ph-webpage-completed-tab',

    triggers: {
        'change:resolved_count': 'render'
    }
});
/**
 * Tasks View (unresolved)
 */
ph.api.views.ResolvedThreads = ph.api.View.extend({
    template : wp.template('resolved-tasks'),
    className: 'ph-webpage-completed-tasks',

    events: {
        'click .ph-resolved-tab': 'toggleResolved'
    },

    triggers: {
        'change:filter': 'setThreads',
    },

    ready: function () {
        if (this.model.resolvedThreads().length > 1) {
            this.$('.ph-resolved-tasks').sortable({
                handle: ".ph-sort-handle",
                delay : 150,
                helper: 'clone',
                // consider using update instead of stop
                start : function (event, ui) {
                    ui.item.trigger('start', ui.item.index());
                },
                stop  : function (event, ui) {
                    ui.item.trigger('drop', ui.item.index());
                }
            });
        }
    },

    initialize: function () {
        this.views.add('.ph-resolved-tab', new ph.api.views.ResolvedTab({
            model: this.model
        }));

        this.listenTo(this.model.get('threads'), 'change:resolved', this.setThreads);

        this.setThreads();
    },

    setThreads: function () {
        var views = [];
        this.model.resolvedThreads().forEach(function (thread) {
            views.push(new ph.api.views.Thread({
                model: thread
            }));
        }, this);

        this.views.set('.ph-resolved-tasks', views);
    },

    toggleResolved: function () {
        this.$el.toggleClass('show');
    }
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.ShareButton = ph.api.View.extend({
	events: {
		'click': 'linkDialog'
	},

	initialize: function () {
		this.setElement(this.el);
		this.render();
		_.bindAll(this, 'linkDialog');
	},

	linkDialog: function () {
		window.prompt('Share this access link:', this.model.get('link'));
	}
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.ThreadControls = ph.api.View.extend({
  template : wp.template('ph-thread-controls'),
  className: 'ph-webpage-controls',

  triggers: {
    'change:assigned'     : 'render',
    'change:comment_count': 'render',
    'change:assigned_model': 'render'
  },

  initialize: function() {
    this.views.set('.ph-thread-assignment', new ph.api.views.Assign({
      model: this.model
    }));
  },

  ready: function() {
    this.model.getAssigned();
  },
});
/**
 * Webpage View
 *
 * @since 1.2.0
 */
ph.api.views.ThreadInfoPanel = ph.api.View.extend({
  className: 'ph-info-inner',
  template : wp.template('thread-info-panel'),

  initialize: function() {
    this.views.set('.info-panel-comments-list',
        new ph.api.views.CommentsContainer({
          model: this.model,
        }));

    this.views.add('.info-form', new ph.api.views.Avatar({
      model: this.model,
    }));

    this.views.add('.info-form', new ph.api.views.CommentForm({
      model: this.model,
    }));

    this.views.add('.info-button', new ph.api.views.CommentSubmitButton({
      model: this.model,
    }));

    this.views.add('.select-users', new ph.api.views.Assign({
      model: this.model
    }));

    this.listenTo(this.model.get('comments'), 'request', this.addLoading);
    this.listenTo(this.model.get('comments'), 'sync', this.removeLoading);
    this.listenTo(this.model.get('comments'), 'sync', this.moreClass);
    this.moreClass();
  },

  moreClass: function() {
    this.$el.toggleClass('ph-has-more', this.model.get('comments').hasMore());
  },

  addLoading: function(collection, xhr, options) {
    options || (options = {});
    options.loading && this.$el.addClass('ph-comments-loading');
  },

  removeLoading: function() {
    this.$el.removeClass('ph-comments-loading');
  },
});
/**
 * Website list
 *
 * Subview of the panel view that shows an individual comment in a list
 *
 * @since 1.2.0
 */
ph.api.views.ThreadText = ph.api.View.extend({
    template : wp.template('ph-website-thread-text'),
    className: 'ph-comment-content-inner',

    triggers: {
        'change:textOverflow': 'showHideLinks', // show more link?
        'change:showFull'    : 'showHideContent' // show or hide content?
    },

    initialize: function () {
        _.bindAll(this, 'heightHandler');
    },

    ready: function () {
        this.heightHandler();
        jQuery(window).resize(this.heightHandler);
    },

    heightHandler: function () {
        // if the scrollheight is bigger than the height
        if (this.$el.height() + 5 < this.el.scrollHeight) {
            this.model.set('textOverflow', true);
        } else {
            this.model.set('textOverflow', false);
        }
    }
});
/**
 * Top level view that orchestrates the front-end experience.
 *
 * Extends Huddle.View mixin
 *
 * this.collection holds the entire images collection and all subdata
 *
 * @since 1.0
 */
ph.api.views.Thread = ph.api.View.extend({
    template: wp.template('website-thread'),
    className: 'ph-thread',

    events: {
        'click .ph-trash-thread': 'trash',
        'touchstart .ph-trash-thread': 'trash',

        'drop': 'drop',

        'click .ph-show-info': 'triggerThreadPanel',
        'click .ph-more': 'showText',
        'click .ph-less': 'hideText',

        'touchend .ph-show-info': 'triggerThreadPanel',
        'touchend .ph-more': 'showText',
        'touchend .ph-less': 'hideText'
    },

    triggers: {
        'change:show': 'showThreadPanel',
        'change:textOverflow': 'showHideLinks',
        'change:display': 'showHideContent',
        'change:display': 'showHide'
    },

    initialize: function () {
        // checkbox
        this.views.set('.checkbox-wrap', new ph.api.views.ResolveBox({
            model: this.model
        }));

        // info panel
        this.views.set('.ph-task-info', new ph.api.views.ThreadInfoPanel({
            model: this.model
        }));

        // controls
        this.views.set('.ph-webpage-controls-wrap', new ph.api.views.ThreadControls({
            model: this.model
        }));

        // thread text
        this.views.set('.ph-comment-content', new ph.api.views.ThreadText({
            model: this.model
        }));
    },

    drop: function(event, index) {
        this.model.trigger('sorted scrollBottom', this.model, index);
    },

    trash: function (e) {
        e.preventDefault();
        confirm(PH.translations.are_you_sure) && this.destroyThread();
    },

    destroyThread: function() {
        this.$el.slideToggle(250, _.bind(function () {
            this.remove();
            this.unbind();
            this.model.destroy();
        }, this));
    },

    showText: function(e) {
        e.preventDefault();
        this.model.set('show', true);
    },

    hideText: function(e) {
        e.preventDefault();
        this.model.set('show', false);
    },

    triggerThreadPanel: function(e) {
        e.preventDefault();

        // cache model attribute
        var show = this.model.get('show');

        // if hidden, show, otherwise hide
        if (show) {
            this.model.set('show', false);
        } else {
            this.model.set('show', true);
        }
    },

    showThreadPanel: function() {
        if ( this.model.get('show') ) {
            this.$('.ph-task-info').show(0, _.bind(function(){
                this.model.trigger('scrollBottom');
            }, this));
        } else {
            this.$('.ph-task-info').hide();
        }
    },

    showHide: function() {
        this.model.get('show') ? this.$el.show() : this.$el.hide();
    },

    showHideLinks: function() {
        this.$('.ph-more').toggleClass('ph-hidden', !this.model.get('textOverflow'));
        this.$('.ph-less').toggleClass('ph-hidden', !this.model.get('textOverflow'));
    },

    showHideContent: function() {
        this.$el.toggleClass('ph-show', this.model.get('show'));
    }
});
/**
 * Threads View (unresolved)
 */
ph.api.views.Threads = ph.api.View.extend({
  className: 'ph-webpage-tasks',

  triggers: {
    'change:filter': 'setThreads',
  },

    initialize: function () {
        this.listenTo(this.model.get('threads'), 'change:resolved', this.setThreads);

        this.setThreads();
    },

    ready: function () {
        if (this.model.unresolvedThreads().length > 1) {
            this.$el.sortable({
                helper: 'clone',
                handle: ".ph-sort-handle",
                delay : 150,
                // consider using update instead of stop
                stop  : function (event, ui) {
                    ui.item.trigger('drop', ui.item.index());
                }
            });
        }
    },

    setThreads: function () {
        var views = [];
        this.model.unresolvedThreads().forEach(function (thread) {
            views.push(new ph.api.views.Thread({
                model: thread
            }));
        }, this);

        if (!this.model.filteredThreads()) {
            views.push(new ph.api.views.NoThreads({
                model: this.model
            }));
        }

        this.views.set(views);
    }
});

ph.api.views.NoThreads = ph.api.View.extend({
    template: wp.template('no-comments')
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.Title = ph.api.View.extend({
    events    : {
        'blur': 'saveTitle'
    },
    initialize: function () {
        this.setElement(this.el);
        this.render();
    },
    saveTitle : function () {
        this.model.save({
            title: this.$el.val()
        }, {
            patch  : true,
            success: function (response) {
                window.onbeforeunload = null;
                jQuery(window).off('beforeunload.edit-post unload');
            }
        });
        window.onbeforeunload = null;
        jQuery(window).off('beforeunload.edit-post unload');
    }
});
ph.api.views.WebpageTitleEdit = ph.api.View.extend({
    template: wp.template('edit-webpage-title'),
    className: 'ph-title-edit',

    events: {
        'click .ph-edit-save:enabled': 'save'
    },

    save: function(e){
        this.$('.ph-edit-save').attr('disabled', 'disabled');
        this.model.save({
            'title': this.$('.ph-edit-content').val()
        }, {
            patch: true,
            wait: true
        });

        return false;
    }
});
ph.api.views.WebpageTitle = ph.api.View.extend({
    template: wp.template('webpage-title'),
    className: 'ph-title-edit',

    events: {
        'click .ph-edit-title': 'editTitle',
        'click .ph-edit-cancel': 'removeEdit'
    },

    triggers: {
        'change:title': ['removeEdit', 'render']
    },

    editTitle: function() {
        this.$el.addClass('ph-editing');
        this.views.set('.ph-website-title-edit', new ph.api.views.WebpageTitleEdit({
            model: this.model
        }));

        return false;
    },

    removeEdit: function() {
        this.$el.removeClass('ph-editing');
        this.views.remove('.ph-website-title-edit');

        return false;
    }
});
/**
 * Webpage View
 *
 * @since 1.2.0
 */
ph.api.views.Webpage = ph.api.View.extend({
    template: wp.template('website-page'),
    className: 'ph-webpage',

    triggers: {
        'change:empty': 'render'
    },

    events: {
        'click .ph-toggle-page': 'toggleHeight',
        'dropped'              : 'drop',
        'click .ph-trash-page' : 'trash'
    },

    initialize: function () {
        // title
        this.views.add('.ph-webpage-title', new ph.api.views.WebpageTitle({
            model: this.model
        }));

        // threads
        this.views.add('.ph-webpage-tasks-container', new ph.api.views.Threads({
            model: this.model
        }));
        this.views.add('.ph-webpage-tasks-container', new ph.api.views.ResolvedThreads({
            model: this.model
        }));

        this.listenTo(this.model.get('threads'), 'remove', this.removeIfEmpty);
    },

    drop: function (event, index) {
        this.model.trigger('sorted', this.model, index);
    },

    maybeNone: function () {
        if (this.model.get('threads').where({show: true}).length) {
            this.model.set('empty', false);
        } else {
            this.model.set('empty', true);
        }
    },

    removeIfEmpty: function () {
        if ( this.model.get('threads').length == 0 ) {
            this.remove();
        }
    },

    toggleHeight: function(e) {
        this.$el.toggleClass('closed');
        e.preventDefault();
    },

    trash: function() {
        var confirm = window.confirm(PH.translations.are_you_sure);

        if (confirm === true) {
            this.model.destroy();
            this.remove();
        }
    }
});
/**
 * Avatar View
 *
 * @since 1.0
 */
ph.api.views.WebsiteTitle = ph.api.View.extend({
    events: {
        'blur': 'saveTitle'
    },
    initialize: function () {
        this.setElement(this.el);
        this.render();
    },
    saveTitle: function () {
        this.model.save({
            title: this.$el.val()
        }, {
            patch: true,
            success: function(response) {
                window.onbeforeunload = null;
                jQuery(window).off( 'beforeunload.edit-post unload' );
            }
        });
        window.onbeforeunload = null;
        jQuery(window).off( 'beforeunload.edit-post unload' );
    }
});
/**
 * Top level view that orchestrates the front-end experience.
 *
 * Extends Huddle.View mixin
 *
 * this.collection holds the entire images collection and all subdata
 *
 * @since 1.0
 */
ph.api.views.Website = ph.api.View.extend({
    template: wp.template('website-container'),
    className: "website-inner",

    initialize: function () {
        // views
        this.model.get('pages').forEach(function (item) {
            this.views.add('.ph-webpages', new ph.api.views.Webpage({
                model: item
            }));
        }, this);

        this.views.add('.comment-switcher-dropdown-container', new ph.api.views.CommentFilter({
            model: this.model
        }));

        // add titlebox
        this.titleBox = new ph.api.views.WebsiteTitle({
            el:  jQuery("#titlewrap #title"),
            model: this.model
        });

        // add members
        this.members = new ph.api.views.Members({
            el:  jQuery(".ph-select2.ph-get-users"),
            model: this.model
        });

        // project options
        this.options = new ph.api.views.Options({
            el:  jQuery("#ph-website-project-options"),
            model: this.model
        });

        // add notifications
        this.notifications = new ph.api.views.Notifications({
            el:  jQuery("#project_emails_notifications_container"),
            model: this.model
        });
    },

    /**
     * Make sortable
     */
    ready: function () {
        this.$('.ph-webpages').sortable({
            helper : 'clone',
            delay: 150,
            handle: ".ph-webpage-title",
            // consider using update instead of stop
            stop: function (event, ui) {
                ui.item.trigger('dropped', ui.item.index());
            }
        });
    }
});
/**
 * Setup page events and collections
 *
 * @package     ProjectHuddle
 * @subpackage  js
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0
 */

(function($){

    // use strict mode
    'use strict';

    /**
     * Setup Page
     *
     * @since 1.0
     */
    $(document).ready(function(){
        // tabs
		$('.nav-tab-wrapper a').click(function(e){
			var $this = $(this),
				current = $this.data('value');

			// change input
			$this.parent().find('input').val($this.data('value'));

			// class change
			$('.nav-tab-active').removeClass('nav-tab-active');
			$this.addClass('nav-tab-active');

			$('.ph-tab-content').hide();
			$('.show-' + current ).show();

			e.preventDefault();
		});

		// find active tab
		var current = $('.nav-tab-active').data('value');

		$('.ph-tab-content').hide();
		$('.show-' + current ).show();
	});

}(jQuery));