<?php
/**
 * Class to output website scripts and styles to the
 * website toolbar frontend.
 *
 * We use this instead of wp_enqueue_scripts as to not
 * interfere with theme functions or output unnecessary
 * styles an scripts.
 *
 * @package ProjectHuddle
 * @since 2.5.0
 */

/**
 * Website Scripts class
 */
class PH_Website_Scripts {
	/**
	 * Store javascript directory
	 *
	 * @var string
	 */
	public $js_dir = '';

	/**
	 * Store CSS directory
	 *
	 * @var string
	 */
	public $css_dir = '';

	/**
	 * Whether to use min script
	 *
	 * @var string
	 */
	public $suffix = '';

	/**
	 * Get things going.
	 */
	public function __construct() {
		$this->suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? $this->suffix : '.min';

		$this->css_dir = PH_WEBSITE_PLUGIN_URL . 'assets/css/';
		$this->js_dir  = PH_WEBSITE_PLUGIN_URL . 'assets/js/';

		// include script enqueues.
		add_action( 'ph_website_header', 'wp_enqueue_scripts' );

		// include customizer css.
		add_action( 'ph_website_header', 'wp_custom_css_cb', 101 );

		// add scripts and styles.
		add_action( 'ph_website_header', array( $this, 'header_styles' ) );
		add_action( 'ph_website_header', array( $this, 'header_scripts' ) );
		add_action( 'ph_website_footer', array( $this, 'footer_scripts' ), 20 );

		// register.
		add_action( 'wp_enqueue_scripts', array( $this, 'register_scripts' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'register_styles' ) );

		// add url attribute.
		add_filter( 'script_loader_tag', array( $this, 'add_url_attribute' ), 10, 2 );
	}

	/**
	 * Register our PH styles
	 */
	public function register_styles() {
		// return for other pages.
		if ( ! is_singular( 'ph-website' ) && ! is_singular( 'phw_comment_loc' ) ) {
			return;
		}

		// need to use CDN due to servers not likely having CORS policy.
		wp_register_style( 'project-huddle-fonts', '//fonts.googleapis.com/css?family=Montserrat:400,700', array(), PH_VERSION );
		wp_register_style(
			'ph-website-comments', $this->css_dir . 'ph-website-comments' . $this->suffix . '.css', array(
				'project-huddle-fonts',
				'ph-ionicons',
			), PH_VERSION
		);

		wp_add_inline_style( 'ph-website-comments', apply_filters( 'ph_inline_styles', ph_website_style_options() ) );
	}

	/**
	 * Register our PH Scripts
	 */
	public function register_scripts() {
		// return for other pages.
		if ( ! is_singular( 'ph-website' ) && ! is_singular( 'phw_comment_loc' ) ) {
			return;
		}

		wp_register_script( 'x-domain', $this->js_dir . 'includes/xdomain.min.js', array(), '1.7', false );
		wp_register_script( 'ph-url-parse', $this->js_dir . 'includes/urlparse.js', array(), '1.1.9', false );

		if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) {
			wp_deregister_script( 'backbone' );
			wp_deregister_script( 'underscore' );
			wp_register_script( 'backbone', PH_PLUGIN_URL . 'assets/js/includes/backbone.js' );
			wp_register_script( 'underscore', 'https://cdnjs.cloudflare.com/ajax/libs/underscore.js/1.8.3/underscore.js' );
		}

		wp_register_script(
			'ph-website-comments', PH_PLUGIN_URL . 'assets/js/dist/' . 'ph-website-comments' . $this->suffix . '.js', array(
				'x-domain',
				'ph-url-parse',
				'jquery',
				'jquery-ui-core',
				'jquery-ui-draggable',
				'jquery-ui-droppable',
				'underscore',
				'backbone',
				'wp-api',
				'wp-backbone',
			), PH_VERSION, true
		);

		// localize our origin.
		wp_localize_script(
			'ph-website-comments', 'PH_Website_Settings', array(
				'debug'             => defined( 'SCRIPT_DEBUG' ) ? SCRIPT_DEBUG : false,
				'comments_per_page' => apply_filters( 'ph_comments_per_page', 25 ),
				'options'           => PHW()->website->get_project_options( get_the_ID() ),
				'edit_link'         => admin_url( sprintf( get_post_type_object( 'ph-website' )->_edit_link . '&action=edit', get_the_ID() ) ),
				'translations'      => ph_website_translation_strings(),
			)
		);

		global $wp_rest_server;

		// Ensure the rest server is intiialized.
		if ( empty( $wp_rest_server ) ) {
			/** This filter is documented in wp-includes/rest-api.php */
			$wp_rest_server_class = apply_filters( 'wp_rest_server_class', 'WP_REST_Server' );
			$wp_rest_server       = new $wp_rest_server_class();
			/** This filter is documented in wp-includes/rest-api.php */
			do_action( 'rest_api_init', $wp_rest_server );
		}

		// Load the schema.
		$schema_request  = new WP_REST_Request( 'GET', '/projecthuddle/v2' );
		$schema_response = $wp_rest_server->dispatch( $schema_request );
		$schema          = null;
		if ( ! $schema_response->is_error() ) {
			$schema = $schema_response->get_data();
		}

		// Localize the plugin settings and schema.
		$settings = array(
			'root'          => esc_url_raw( get_rest_url() ),
			'nonce'         => wp_create_nonce( 'wp_rest' ),
			'versionString' => 'projecthuddle/v2/',
			'schema'        => $schema,
			'cacheSchema'   => true,
		);

		/**
		 * Filter the JavaScript Client settings before localizing.
		 *
		 * Enables modifying the config values sent to the JS client.
		 *
		 * @param array  $settings The JS Client settings.
		 */
		$settings = apply_filters( 'rest_js_client_settings', $settings );
		wp_localize_script( 'wp-api', 'wpApiSettings', $settings );
	}

	/**
	 * Add url slave attribute to xdomain script
	 *
	 * @param string $tag URL attribute.
	 * @param string $handle Script handle.
	 *
	 * @return mixed
	 */
	public function add_url_attribute( $tag, $handle ) {
		global $post;
		if ( 'x-domain' !== $handle ) {
			return $tag;
		}
		$debug = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? 'debug="true"' : false;

		return str_replace( ' src', ' slave="' . esc_url( home_url() . '/?ph_handler=' . (int) $post->ID ) . '" ' . $debug . ' src', $tag );
	}

	/**
	 * Header scripts
	 */
	public function header_scripts() {
		global $wp_scripts;

		$scripts = array(
			'x-domain',
			'jquery',
		);

		$wp_scripts->do_items( apply_filters( 'ph_website_header_scripts', $scripts ) );
	}

	/**
	 * Header styles
	 */
	public function header_styles() {
		global $wp_styles;

		$styles = array();

		$allowed = apply_filters( 'ph_allowed_website_styles', array() );
		if ( ! empty( $allowed ) ) {
			foreach ( $allowed as $style ) {
				$styles[] = $style;
			}
		}

		$wp_styles->do_items( apply_filters( 'ph_website_footer_scripts', $styles ) );
	}

	/**
	 * Footer scripts
	 */
	public function footer_scripts() {
		global $wp_scripts;

		$ph_scripts = array(
			'ph-website-comments',
		);

		$allowed = apply_filters( 'ph_allowed_website_scripts', array() );
		if ( ! empty( $allowed ) ) {
			foreach ( $allowed as $script ) {
				$ph_scripts[] = $script;
			}
		}

		$wp_scripts->do_items( apply_filters( 'ph_website_footer_scripts', $ph_scripts ) );
	}
}
