<?php
/**
 * Comment Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) { exit; }

/**
 * Website Comment Class
 *
 * This class handles all comment data
 *
 * @since 2.6.0
 */
class PH_Website_Comment extends PH_Rest_Object {
	/**
	 * It's a comment
	 * @var string
	 */
	public $post_type = 'comment';

	/**
	 * User comment meta
	 * @var string
	 */
	public $meta_type = 'comment';

	/**
	 * For website commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'website';

	/**
	 * Parent collection post type
	 * @var string
	 */
	public $parent_post_type = 'phw_comment_loc';

	/**
	 * Slug for actions
	 * @var string
	 */
	public $action_slug = 'comment';


	public function __construct() {
		// add schema
		$this->schema = $this->schema();

		// register fields
		$this->register_fields_from_schema();

		// register comment route
		add_action( 'rest_api_init', array( $this, 'register_route' ) );

		// create rest fields from schema
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// allow our comment types for specific queries
//		add_filter( "rest_comment_query", array( $this, 'allow_comment_types' ), 10, 2);

		// force our comment type if parent is correct
//		add_filter( "rest_pre_insert_comment", array( $this, 'force_comment_type_save' ), 10, 2 );
//		add_filter( "rest_comment_query", array( $this, 'force_comment_type_query' ), 10, 2 );

		// force approval for our type
		add_filter( "pre_comment_approved", array( $this, 'force_comment_approval' ), 10, 2 );

		// disable flood filter
		add_filter( 'comment_flood_filter', array( $this, 'disable_flood_filter' ), 10, 3 );

		// allow duplicate comments for our comment type
		add_filter( 'duplicate_comment_id', array( $this, 'allow_duplicates' ), 10, 2 );

		// permissions
//		add_filter( 'map_meta_cap', array( $this, 'edit_own_comment' ), 10, 4 );

		// standardize actions
		add_action( 'wp_insert_comment', array( $this, 'new_comment_action' ), 10, 2 );
		add_action( 'edit_comment', array( $this, 'edit_comment_action' ), 10, 2 );
		add_action( 'trashed_comment', array( $this, 'delete_comment_action' ), 10, 2 );

		// make sure we always get avatars
		add_filter( "rest_prepare_{$this->post_type}", array( $this, 'force_avatars' ), 9, 3);

		// standardize REST actions
		add_filter( "rest_prepare_{$this->post_type}", array( $this, 'handle_rest_prepare' ), 10, 3 );
		add_action( "rest_insert_{$this->post_type}", array( $this, 'edit_rest_comment_action' ), 10, 3 );
		add_action( "rest_delete_{$this->post_type}", array( $this, 'delete_rest_comment_action' ), 10, 3 );
	}

	// Function to register our new routes from the controller.
	public function register_route() {
		$controller = new PH_REST_Comments_Controller(array(
			'rest_base' => 'website/comment'
		));
		$controller->register_routes();
	}

	/**
	 * Schema for post meta
	 * @return array
	 */
	public function schema() {
		return apply_filters( "{$this->comment_type}_meta", array(
			'action' => array(
				'description' => esc_html__( 'Type of comment action. Options include resolve. For ProjectHuddle only.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'sanitize_text_field',
					'validate_callback' => 'esc_html'
				),
				'default'     => ''
			)
		) );
	}

	/**
	 * Allow querying of our comment only if it matches our
	 * parent post type
	 *
	 * @link https://developer.wordpress.org/reference/classes/wp_comment_query/
	 *
	 * @param array           $prepared_args Array of arguments for WP_Comment_Query.
	 * @param WP_REST_Request $request       The current request.
	 *
	 * @return mixed
	 */
	public function allow_comment_types( $prepared_args, $request ) {
		if ( ! empty($prepared_args['post__in']) && get_post_type($prepared_args['post__in'][0]) === $this->parent_post_type ) {
			$prepared_args['ph_force_show'] = true; // force show our comment type
		}

		return $prepared_args;
	}

	/**
	 * Filter prepared comment before save to make sure it's always our
	 * comment type so public comments are not added by accident.
	 *
	 * @param $prepared_comment array Prepared comment
	 * @param $request          object Request object
	 *
	 * @return array
	 */
	public function force_comment_type_save($prepared_comment, $request) {
		if ( get_post_type($prepared_comment['comment_post_ID']) === $this->parent_post_type ) {
			$prepared_comment['comment_type'] = $this->comment_type;
		}
		return $prepared_comment;
	}

	/**
	 * Filter prepared comment before save to make sure it's always approved
	 *
	 * @param bool|string $approved    The approval status. Accepts 1, 0, or 'spam'.
	 * @param array       $commentdata Comment data.
	 *
	 * @return boolean
	 */
	public function force_comment_approval( $approved, $commentdata ) {
		// ignore spam or trash
		if ( $approved == 'trash' || $approved == 'spam' ) {
			return $approved;
		}

		// if it matches our post type, approve
		if ( get_post_type( $commentdata['comment_post_ID'] ) == $this->parent_post_type ) {
			return 1;
		}

		// approve
		return $approved;
	}


	/**
	 * Force queried website comments to our type
	 *
	 * @param $args    array Query args
	 * @param $request object Request object
	 *
	 * @return array
	 */
	public function force_comment_type_query( $args, $request ) {
		if ( isset( $args['post_id'] ) && get_post_type( $args['post_id'] ) === $this->parent_post_type ) {
			$args['type'] = $this->comment_type;
		}
		if ( isset( $args['post__in']) ) {
			foreach( $args['post__in'] as $post ) {
				if ( get_post_type($post) === $this->parent_post_type ) {
					$args['type'] = $this->comment_type;
				}
			}
		}

		return $args;
	}

	/**
	 * Allow duplicate comments for comments on our website pages
	 *
	 * @param $dupe_id     integer Duplicate comment id
	 * @param $commentdata array Comment data
	 *
	 * @return bool
	 */
	public function allow_duplicates($dupe_id, $commentdata) {
		if ( get_post_type($commentdata['comment_post_ID']) === $this->parent_post_type ) {
			return false;
		}
		return $dupe_id;
	}

	/**
	 * Filters a comment returned from the API.
	 *
	 * Allows modification of the comment right before it is returned.
	 *
	 * @since 4.7.0
	 *
	 * @param WP_REST_Response  $response The response object.
	 * @param WP_Comment        $comment  The original comment object.
	 * @param WP_REST_Request   $request  Request used to generate the response.
	 *
	 * @return WP_REST_Response
	 */
	public function force_avatars( $response, $comment, $request ) {
		$response->data['author_avatar_urls'] = rest_get_avatar_urls( $comment->comment_author_email );
		return $response;
	}
	/**
	 * Filter on the users capabilities.
	 * This function is used to explicitly allow users to edit their own comments
	 * Regardless of their capabilities or roles.
	 *
	 * Comment must be a ph-comment type.
	 *
	 * @param array  $caps    Returns the user's actual capabilities.
	 * @param string $cap     Capability name.
	 * @param int    $user_id The user ID.
	 * @param array  $args    Adds the context to the cap. Typically the object ID.
	 *
	 * @return array
	 */
	public function edit_own_comment( $caps, $cap, $user_id, $args ) {
		// Bail out if we're not asking about a comment
		if ( 'edit_comment' !== $cap ) {
			return $caps;
		}

		// Load the comment data
		$comment = get_comment( $args[0] );

		// bail if comment doesn't exist
		if ( ! $comment ) {
			return $caps;
		}

		// let them edit their own comment, if it's our comment type
		if ( $this->comment_type === $comment->comment_type ) {
			// No cap required to edit your own comment
			if ( (int) $user_id === (int) $comment->user_id ) {
				return array();
			}
		}

		// let them edit if they can moderate_comments
		if ( current_user_can( 'moderate_comments' ) ) {
			return array();
		}

		return $caps;
	}

	/**
	 * Disable comment flood filter if current user can edit threads
	 *
	 * @param boolean $block
	 * @param string $time_lastcomment
	 * @param string $time_newcomment
	 *
	 * @return bool
	 */
	public function disable_flood_filter( $block, $time_lastcomment, $time_newcomment ) {
		if ( current_user_can( "edit_{$this->parent_post_type}s" ) ) {
			return false;
		}

		return $block;
	}

	/**
	 * New comment action
	 *
	 * @param int        $id      The comment ID.
	 * @param WP_Comment $comment Comment object.
	 */
	public function new_comment_action( $id, $comment ) {
		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		// custom action
		do_action( 'ph_website_publish_comment', $id, $comment  );
	}

	/**
	 * Edit comment filter
	 *
	 * @param int   $id The comment ID.
	 * @param array $data       Comment data.
	 */
	public function edit_comment_action( $id, $data ) {
		$comment = get_comment( $id );

		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		if ( ! $comment ) {
			return;
		}

		do_action( 'ph_website_edit_comment', $id, $comment );
	}

	/**
	 * Edit comment filter
	 *
	 * @param int $id The comment ID.
	 */
	public function delete_comment_action( $id ) {
		$comment = get_comment( $id );

		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		do_action( 'ph_website_delete_comment', $id, $comment );
	}

	/**
	 * Fires after a single post object is created via the REST API.
	 *
	 * @param WP_Comment      $comment  Inserted or updated comment object.
	 * @param WP_REST_Request $request  Request object.
	 * @param bool            $creating True when creating a comment, false when updating.
	 */
	public function new_rest_comment_action( $comment, $request, $creating ) {
		// must be new
		if ( ! $creating ) {
			return;
		}

		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		do_action( "ph_website_rest_publish_comment", $comment, $request );
	}

	/**
	 * Fires after a single post object is created via the REST API.
	 *
	 * @param WP_Comment      $comment  Inserted or updated comment object.
	 * @param WP_REST_Request $request  Request object.
	 * @param bool            $creating True when creating a comment, false when updating.
	 */
	public function edit_rest_comment_action( $comment, $request, $creating ) {
		// must not be new
		if ( $creating ) {
			return;
		}

		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		do_action( "ph_website_rest_edit_comment", $comment, $request );
	}

	/**
	 * Fires after a single post object is created via the REST API.
	 *
	 * @param WP_Comment       $comment  Inserted or updated comment object.
	 * @param WP_REST_Response $response The response data.
	 * @param WP_REST_Request  $request  The request sent to the API.
	 */
	public function delete_rest_comment_action( $comment, $response, $request ) {
		// must be our comment type
		if ( $comment->comment_type !== $this->comment_type ) {
			return;
		}

		do_action( "ph_website_rest_delete_comment", $comment, $request );
	}
}