<?php
/**
 * Website Project Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PH_Project Class
 *
 * This class handles the project data
 *
 * @since 2.6.0
 */
class PH_Website_Project extends PH_Rest_Object {
	/**
	 * Post type
	 *
	 * @var string
	 */
	public $post_type = 'ph-website';

	/**
	 * Slug for actions
	 *
	 * @var string
	 */
	public $action_slug = 'project';

	/**
	 * For website commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'website';

	/**
	 * Register actions
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// add schema.
		$this->schema = $this->schema();

		// add fields.
		$this->register_fields_from_schema();

		// standardize actions.
		$this->add_actions();
		$this->add_filters();

		// check for trailing slash in website_url query.
		add_filter( 'ph_website_url_collection_query_args', array( $this, 'smart_match_url' ), 10, 3 );

		// add token field.
		add_action( 'rest_api_init', array( $this, 'add_token_field' ) );

		// comment scroll field.
		add_action( 'rest_api_init', array( $this, 'comment_scroll_field' ) );
		add_action( 'ph_website_script_loaded', array( $this, 'clear_scroll' ) );

		// register fields.
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );
	}

	/**
	 * Get post type route
	 *
	 * @return string
	 */
	public function route() {
		return $this->route_base . '/' . $this->endpoint_type;
	}

	/**
	 * Schema for post meta
	 *
	 * @return array
	 */
	public function schema() {
		return apply_filters(
			'ph_website_meta', array(
				'website_url'      => array(
					'description' => esc_html__( 'URL for the website.', 'project-huddle' ),
					'type'        => 'url',
					'default'     => '',
					'required'    => true,
				),
				'pages'            => array(
					'description' => esc_html__( 'An array of page objects belonging to project.', 'project-huddle' ),
					'type'        => 'array',
					'default'     => array(),
					'items'       => array(
						'description' => esc_html__( 'Page object.', 'project-huddle' ),
						'type'        => 'object',
						'readonly'    => true,
					),
					'readonly'    => true,
				),
				'project_access'   => array(
					'description' => esc_html__( 'Access options for the project', 'project-huddle' ),
					'type'        => 'string',
					'default'     => 'login',
				),
				'toolbar_location' => array(
					'description' => esc_html__( 'Toolbar location for project', 'project-huddle' ),
					'type'        => 'string',
					'default'     => 'bottom-right',
				),
				'project_members'  => array(
					'description' => esc_html__( 'A list of user IDs that have access to the project.', 'project-huddle' ),
					'type'        => 'array',
					'default'     => array(),
					'items'       => array(
						'description' => esc_html__( 'User id.', 'project-huddle' ),
						'type'        => 'number',
					),
				),
			)
		);
	}

	/**
	 * Collection query params
	 *
	 * Allows querying collections via the following meta data.
	 *
	 * @return array
	 */
	public function collection_params() {
		return apply_filters(
			'ph_website_collection_parameters', array(
				'website_url' => array(
					'description' => esc_html__( 'URL for the website.', 'project-huddle' ),
					'type'        => 'url',
					'meta'        => true,
				),
			)
		);
	}

	/**
	 * Also check for trailing slash in website_url attribute query
	 * instead of exact query math
	 *
	 * @param array           $args Arguments.
	 * @param string          $name Parameter name.
	 * @param WP_REST_Request $request Request.
	 *
	 * @return array
	 */
	public function smart_match_url( $args, $name, $request ) {
		if ( ! $request[ $name ] ) {
			return $args;
		}
		// get https and http variants.
		$url   = $request[ $name ];
		$parse = parse_url( $url );

		if ( 'http' === $parse['scheme'] ) {
			$url_https = preg_replace( '/^http:/i', 'https:', $url );
		} else {
			$url_https = $url;
			$url       = preg_replace( '/^https:/i', 'http:', $url );
		}

		return array(
			'relation' => 'OR',
			array(
				'key'     => 'ph_' . $name,
				'value'   => array(
					untrailingslashit( $url ),
					trailingslashit( $url ),
					untrailingslashit( $url_https ),
					trailingslashit( $url_https ),
				),
				'compare' => 'IN',
			),
			array(
				'key'     => $name,
				'value'   => array(
					untrailingslashit( $url ),
					trailingslashit( $url ),
					untrailingslashit( $url_https ),
					trailingslashit( $url_https ),
				),
				'compare' => 'IN',
			),
		);
	}

	/**
	 * Gets options used for project
	 *
	 * @since 1.0
	 *
	 * @param int $post_id ID of project.
	 *
	 * @return array
	 */
	public function get_project_options( $post_id = 0 ) {
		global $post;
		$post_id = $post_id ? $post_id : $post->ID;

		$options = array(
			// global options.
			'global'  => (array) $this->get_global_project_options(),
			// project options.
			'project' => array(
				'project_access'      => get_post_meta( $post_id, 'ph_project_access', true ),
				'allow_anon_comments' => get_post_meta( $post_id, 'ph_project_comments', true ),
				'query_strings'       => apply_filters( 'ph_website_ignore_query_strings', false, $post_id ),
				'retina_images'       => get_post_meta( $post_id, 'ph_retina', true ),
				'sharing'             => get_post_meta( $post_id, 'ph_project_sharing', true ),
				'tooltip'             => apply_filters( 'ph_comment_tooltip', true, get_post( $post_id ) ),
			),
		);

		return $options;
	}

	/**
	 * Get global project options.
	 *
	 * @return array
	 */
	public function get_global_project_options() {
		$options = array(
			'avatar_default'            => get_option( 'avatar_default' ),
			'auto_close'                => get_option( 'ph_auto_close', 'on' ) == 'on' ? true : false,
			'ph_image_background_color' => get_option( 'ph_image_bg' ) ? get_option( 'ph_image_bg' ) : false,
		);

		return apply_filters( 'ph_global_website_options', $options );
	}

	/**
	 * Access Token Field
	 */
	public function add_token_field() {
		register_rest_field(
			$this->post_type, 'access_token', array(
				'get_callback'   => 'ph_get_access_token',
				'project_access' => array(
					'description' => esc_html__( 'Access token for the project', 'project-huddle' ),
					'type'        => 'string',
					'readonly'    => true,
				),
			)
		);
	}

	/**
	 * Comment Scroll Field
	 */
	public function comment_scroll_field() {
		register_rest_field(
			$this->post_type, 'comment_scroll', array(
				'get_callback'   => function () {
					return PH()->session->get( 'ph_comment_id' );
				},
				'comment_scroll' => array(
					'description' => esc_html__( 'Comment scroll information', 'project-huddle' ),
					'type'        => 'string',
					'readonly'    => true,
				),
			)
		);
	}

	/**
	 * Clear Comment Scroll
	 */
	public function clear_scroll() {
		PH()->session->clear( 'ph_comment_id' );
	}
	/**
	 * Gets all stored project data
	 *
	 * @var $post_id int post id
	 *
	 * @since 1.0
	 * @return array
	 */
	public function get_project_info( $post_id = 0 ) {
		global $post;

		if ( ! $post_id ) {
			$post_id = $post->ID;
		}

		$user = wp_get_current_user();

		$project_output = array(
			'id'             => $post_id,
			'permalink'      => get_permalink( $post_id ),
			'title'          => get_the_title( $post_id ),
			'site_url'       => get_site_url(),
			'website_id'     => $post_id,
			'user'           => PH()->user->get_current_user_info(),
			'comments'       => PHW()->thread->get_website_threads(),
			'options'        => $this->get_project_options(),
			'pages'          => $this->get_pages( $post_id ),
			'translations'   => ph_website_translation_strings(),
			'users'          => $this->get_users(),
			'assign_filters' => $this->get_assign_filters(),
			'comment_scroll' => PH()->session->get( 'ph_comment_id' ),
		);

		// return array of project data
		return apply_filters( 'ph_website_project_data', $project_output );
	}

	/**
	 * Gets all commented pages from a specific site id
	 *
	 * @param bool $site_id ID of site.
	 *
	 * @return array Pages and comments
	 */
	public function get_pages( $site_id = false ) {
		global $post;

		$site_id = $site_id ? $site_id : 0;

		if ( is_object( $post ) ) {
			$site_id = $post->ID;
		}

		if ( ! $site_id ) {
			return;
		}

		$pages = get_post_meta( $site_id, 'ph_webpages', true );

		$page_data = array();
		if ( isset( $pages ) && ! empty( $pages ) ) {
			foreach ( $pages as $page_id => $page_url ) {
				// validate existing page id
				if ( ! $page_id || get_post_status( $page_id ) != 'publish' ) {
					continue;
				}
				if ( $data = PHW()->page->data( $site_id, $page_id, $page_url ) ) {
					$page_data[ $page_id ] = $data;
				}
			}
		}

		return array_values( $page_data );
	}

	/**
	 * Add page to website meta
	 */
	public function add_page( $args ) {
		// Parse incoming $args into an array and merge it with $defaults
		$args = wp_parse_args(
			$args, array(
				'website_id' => 0,
				'webpage_id' => 0,
				'location'   => 0,
			)
		);

		if ( ! $args['webpage_id'] || ! $args['location'] ) {
			return false;
		}

		// get current pages
		$pages = (array) get_post_meta( $args['website_id'], 'ph_webpages', true );

		// if it already exists and not published, remove
		if ( $existing_id = array_search( esc_html( $args['location'] ), $pages ) ) {
			// if not published, remove
			if ( get_post_status( $existing_id ) != 'publish' ) {
				unset( $pages[ $existing_id ] );
			}
		}

		// add our page in form of i.e. 89 => '/contact'
		$pages[ (int) $args['webpage_id'] ] = esc_html( $args['location'] );

		// update meta
		update_post_meta( $args['website_id'], 'ph_webpages', $pages );

		return true;
	}

	/**
	 * Updates page order in database
	 *
	 * @param int   $id
	 * @param array $page_ids
	 *
	 * @return bool Success/Failure
	 */
	public function set_page_order( $id = 0, $page_ids = array() ) {
		// get current pages
		$pages = (array) get_post_meta( $id, 'ph_webpages', true );

		// return if empty
		if ( empty( $pages ) ) {
			return false;
		}

		// order by ids
		$pages_ordered = array_replace( array_flip( array_map( 'intval', $page_ids ) ), $pages );

		// update order
		update_post_meta( $id, 'ph_webpages', $pages_ordered );

		return true;
	}

	/**
	 * Does the page exist in the site?
	 *
	 * @param int $site_id
	 * @param int $page_id
	 *
	 * @return mixed
	 */
	public function page_exists( $site_id = 0, $page_id = 0 ) {
		// does it exist in array?
		return array_search( $page_id, get_post_meta( $site_id, 'ph_webpages', true ) );
	}

	/**
	 * Checks to see if webpage already exists before adding
	 */
	public function maybe_add_webpage( $website_id = 0, $webpage_id = 0, $title = 'webpage' ) {
		// get existing webpages
		$existing = get_post_meta( $website_id, 'ph_webpages', true );

		// does it exist in array?
		$exists = array_search( $webpage_id, $existing );

		// If it doesn't exist, add it, then update meta
		if ( $exists == null ) {
			$webpage_id              = $this->add_webpage( $existing );
			$existing[ $webpage_id ] = $this->data['location'];
		} else {
			// make sure it hasn't been permanently deleted
			if ( false === get_post_status( $exists ) ) {
				unset( $existing[ $exists ] );
				update_post_meta( $website_id, 'ph_webpages', $existing );
				$this->add_webpage( $existing, $title );
			} else {
				// otherwise, store ID in class
				$this->webpage_id = $exists;
			}
		}
	}

	private function add_webpage( $existing, $title ) {
		PHW()->page->new_page( $title );
		$existing[ $this->webpage_id ] = $this->data['location'];
	}

	public function get_users( $post_id = 0 ) {
		global $post;

		if ( ! $post_id ) {
			$post_id = $post->ID;
		}

		if ( is_object( $post_id ) && isset( $post_id->ID ) ) {
			$post_id = $post_id->ID;
		}

		$users = ph_get_project_members( $post_id );

		return $users;
	}

	public function get_assign_filters() {
		return apply_filters(
			'ph_assign_filters', array(
				array(
					'name' => __( 'All Comments', 'project-huddle' ),
					'slug' => 'all',
				),
				array(
					'name' => __( 'Assigned To Me', 'project-huddle' ),
					'slug' => 'mine',
				),
				array(
					'name' => __( 'Unassigned', 'project-huddle' ),
					'slug' => 'unassigned',
				),
			), get_the_ID()
		);
	}
}
