<?php
/**
 * Comment Thread Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2017, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Website Thread Class
 * @since 2.6.0
 */
class PH_Website_Thread extends PH_Rest_Object {
	/**
	 * Post type
	 *
	 * @var string
	 */
	public $post_type = 'phw_comment_loc';

	/**
	 * Parent collection post type
	 *
	 * @var string
	 */
	public $parent_post_type = 'ph-webpage';

	/**
	 * Collection name
	 *
	 * @var string
	 */
	public $collection_name = 'threads';

	/**
	 * Slug for actions
	 *
	 * @var string
	 */
	public $action_slug = 'thread';

	/**
	 * For website commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'website';

	/**
	 * Add object meta and filters
	 */
	public function __construct() {
		// add schema.
		$this->schema = $this->schema();

		// add fields.
		$this->register_fields_from_schema();

		// register fields.
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// auto add comment data.
		add_action( 'rest_api_init', array( $this, 'add_comment_data' ) );

		// dynamic thread title.
		add_filter( 'the_content', array( $this, 'dynamic_thread_content' ) );

		// comments count field.
		add_filter( 'rest_api_init', array( $this, 'comments_count' ) );

		// add filters.
		$this->add_filters();

		// standardize actions.
		$this->add_actions();

		// permissions.
		add_filter( 'user_has_cap', array( $this, 'caps' ), 10, 4 );

		// maybe create page if it doesn't exist.
		add_action( "rest_prepare_{$this->post_type}", array( $this, 'maybe_create_page' ), 10, 3 );

		// force comments open for this post type.
		add_filter( 'comments_open', array( $this, 'force_comments_open' ), 10, 2 );

		// store approval comment when approval is set.
		add_action( "ph_website_rest_update_{$this->action_slug}_attribute", array(
			$this,
			'store_approval_comment'
		), 10, 3 );

		// dynamic comment text for approval comments to make sure user display name and translations are up to date.
		add_filter( 'comment_text', array( $this, 'approval_comment_text' ), 8, 2 );

		// sort threads by menu order
		add_filter( "ph_{$this->collection_name}_collection_data", array( $this, 'sort_by_menu_order' ) );
	}

	/**
	 * Schema for post meta
	 * @return array
	 */
	public function schema() {
		return apply_filters( 'ph_website_thread_meta', array(
			'parent_id'      => array(
				'description' => esc_html__( 'ID of parent webpage.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0,
			),
			'path'           => array(
				'description' => esc_html__( 'Path to the clicked element.', 'project-huddle' ),
				'type'        => 'string',
				'default'     => ''
			),
			'relativeX'      => array(
				'description' => esc_html__( 'Relative horizontal click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'relativeY'      => array(
				'description' => esc_html__( 'Relative vertical click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'html'           => array(
				'description' => esc_html__( 'Inner html of the clicked element.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'wp_kses_post',
				),
				'default'     => ''
			),
			'resX'           => array(
				'description' => esc_html__( 'Horizontal resolution of user who reported issue.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => ''
			),
			'resY'           => array(
				'description' => esc_html__( 'Vertical resolution of user who reported issue.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => ''
			),
			'resolved'       => array(
				'description' => esc_html__( 'Issue resolve status.', 'project-huddle' ),
				'type'        => 'boolean',
				'default'     => false
			),
			'browser'        => array(
				'description' => esc_html__( 'Browser information.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'sanitize_text_field',
				),
				'default'     => ''
			),
			'browserVersion' => array(
				'description' => esc_html__( 'Browser version information.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => '',
			),
			'browserOS'      => array(
				'description' => esc_html__( 'Operating system information.', 'project-huddle' ),
				'type'        => 'string',
				'arg_options' => array(
					'sanitize_callback' => 'sanitize_text_field',
				),
				'default'     => ''
			),
			'page_url'       => array(
				'description' => esc_html__( 'Url for the page.', 'project-huddle' ),
				'type'        => 'url',
			),
			'page_title'     => array(
				'description' => esc_html__( 'Title for the new page.', 'project-huddle' ),
				'type'        => 'string'
			),
			'website_id'     => array(
				'description' => esc_html__( 'Website project id for the thread.', 'project-huddle' ),
				'type'        => 'integer',
				'required'    => true,
			),
			'total_comments' => array(
				'description' => esc_html__( 'Total comments on the thread.', 'project-huddle' ),
				'type'        => 'integer',
				'readonly'    => true,
			),
			'assigned'       => array(
				'description' => esc_html__( 'ID of user who is assigned.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0
			),
		) );
	}

	/**
	 * Collection query params
	 *
	 * Allows querying collections via the following meta data.
	 *
	 * @return array
	 */
	public function collection_params() {
		return apply_filters( 'ph_website_thread_collection_parameters', array(
			'resolved' => array(
				'description' => esc_html__( 'Limit results by resolved status.', 'project-huddle' ),
				'type'        => 'boolean',
				'meta'        => true
			),
			'assigned' => array(
				'description' => esc_html__( 'Limit by ID of user who is assigned.', 'project-huddle' ),
				'type'        => 'integer',
				'meta'        => true
			)
		) );
	}

	/**
	 * Add comment data as rest field
	 */
	public function add_comment_data() {
		// comments
		register_rest_field( $this->post_type, 'comments',
			array(
				'update_callback' => function ( $comments, $post, $attr, $request, $object_type ) {
					if ( empty( $comments ) ) {
						return new WP_Error( 'rest_missing_comment', __( 'You must provide at least one comment to create a thread.', 'project-huddle' ), array( 'status' => '400' ) );
					}

					// save comments to post
					foreach ( $comments as $comment ) {
						$comment['post'] = $post->ID; // set the post id

						// new item
						$request = new WP_REST_Request( 'POST', '/' . PH()->comment->namespace . '/' . PH()->comment->rest_base );

						// set new comment params
						$request->set_body_params( apply_filters( 'ph_new_website_thread_comment', $comment, $post, $request ) );

						// do request
						$created = rest_do_request( $request );
					}

					if ( is_wp_error( $created ) ) {
						return $created;
					}

					return $comments;
				},
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$url = '/' . PH()->comment->namespace . '/' . PH()->comment->rest_base;

					$request = new WP_REST_Request( 'GET', $url );

					$request->set_query_params(
						array(
							'post'     => array( $post['id'] ),
							'per_page' => apply_filters( 'ph_comments_per_page', 25 ),
							'order'    => 'desc'
						)
					);
					$response = rest_do_request( $request );

					return $response->get_data();
				},
				'schema'          => array(
					'description' => esc_html__( 'Array of comment objects in thread.', 'project-huddle' ),
					'type'        => 'array',
					'items'       => array(
						'description' => esc_html__( 'Comment object.', 'project-huddle' ),
						'type'        => 'object',
					),
				)
			)
		);
	}

	function comments_count() {
		register_rest_field( $this->post_type, 'comments_count',
			array(
				'update_callback' => null,
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$comments_count = ph_count_comments( $post['id'] );
					return (int) $comments_count->approved;
				},
				'schema'          => array(
					'description' => esc_html__( 'Array of comment objects in thread.', 'project-huddle' ),
					'type'        => 'integer',
					'readonly'    => true,
				)
			)
		);
	}

	function dynamic_thread_content( $content ) {
		global $post;

		if ( $post->post_type !== $this->post_type ) {
			return $content;
		}

		$comments = ph_get_comments( array(
			'post_id'  => $post->ID,
			'order_by' => 'comment_date',
			'order'    => 'asc',
			'type__in' => array(
				'ph_website_comment',
				'ph_comment',
			),
			'number'   => 1,
		) );

		if ( ! empty( $comments ) ) {
			return $comments[0]->comment_content;
		} else {
			$comments = ph_get_comments( array(
				'post_id'  => $post->ID,
				'order_by' => 'comment_date',
				'order'    => 'desc',
				'type__in' => array(
					'ph_approval',
				),
				'number'   => 1,
			) );

			if ( ! empty( $comments ) ) {
				return $comments[0]->comment_content;
			} else {
				return __( 'No comment text', 'project-huddle' );
			}
		}
	}

	/**
	 * Filter on the current_user_can() function.
	 * This function is used to explicitly allow users to edit their own comments
	 * Regardless of their capabilities or roles.
	 *
	 * @param array  $allcaps All the capabilities of the user
	 * @param array  $cap     [0] Required capability
	 * @param array  $args    [0] Requested capability
	 *                        [1] User ID
	 *                        [2] Associated object ID
	 * @param object $user    User object
	 *
	 * @return array
	 */
	public function caps( $allcaps, $cap, $args, $user ) {
		// Bail out if we're not asking about a post:
		if ( 'edit_post' != $args[0] ) {
			return $allcaps;
		}

		// Load the post data:
		$post = get_post( $args[2] );

		if ( ! $post ) {
			return $allcaps;
		}

		// bail if not our comment type
		if ( $this->post_type !== $post->post_type ) {
			return $allcaps;
		}

		// Bail out if the user is the post author:
		if ( (int) $user->ID !== (int) $post->post_author ) {
			return $allcaps;
		}

		// add required caps
		$allcaps['edit_comment']         = true;
		$allcaps['edit_published_posts'] = true;
		$allcaps[ $cap[0] ]              = true;

		return $allcaps;
	}

	/**
	 * Maybe create a page post when a comment thread is added
	 *
	 * @since 4.7.0
	 *
	 * @param WP_REST_Response $response The response object.
	 * @param WP_Post          $post     Post object.
	 * @param WP_REST_Request  $request  Request object.
	 *
	 * @return WP_REST_Response|WP_Error
	 */
	public function maybe_create_page( $response, $post, $request ) {
		// if we're posting a new thread
		if ( $request->get_method() !== 'POST' ) {
			return $response;
		}

		// Can't create comment if didn't supply parent_id or page_url
		if ( ! isset( $request['parent_id'] ) || ! isset( $request['page_url'] ) ) {
			return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a page id or page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
		}

		// if we already have the parent id, don't do anything
		if ( $request['parent_id'] ) {
			if ( $website = get_post_meta( $request['parent_id'], 'parent_id', true ) ) {
				return $response;
			}
		}

		// can't create comment if blank parent_id or page_url
		if ( ! $request['page_url'] ) {
			return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a page id or page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
		} else {
			// find existing page
			$items = PHW()->page->get_items( array(
				'page_url'  => $request['page_url'],
				'parent_id' => $request['website_id']
			) );

			// If page already exists and website project is published.
			if ( isset( $items[0] ) && get_post_status( $items[0]['parent_id'] ) === 'publish' ) {
				update_post_meta( $post->ID, 'parent_id', $items[0]['id'] );
				// Update the parent id.
				$response->data['parent_id'] = $items[0]['id'];
			} else {
				// get website id provided by api
				$website_id = isset( $request['website_id'] ) ? (int) $request['website_id'] : 0;

				// if website id is not provided, query by url as backup
				if ( ! $website_id ) {
					// find website with same hostname
					$parsed  = parse_url( $request['page_url'] );
					$origin  = $parsed['scheme'] . '://' . $parsed['host'];
					$origin  .= isset( $parsed['port'] ) ? $parsed['port'] : '';
					$website = PHW()->website->get_items( array(
						'website_url' => $origin,
					) );

					$website_id = ! empty( $website[0] ) || isset( $website[0]['id'] ) ? $website[0]['id'] : 0;
				}

				// otherwise create parent page in website project
				// if no website, do nothing
				if ( ! $website_id ) {
					return new WP_Error( 'rest_cannot_create', __( 'Sorry, you need to provide a website id or valid page url for the thread.' ), array( 'status' => rest_authorization_required_code() ) );
				}

				// double-check permissions
				if ( ! current_user_can( 'read_ph-website', $website_id ) ) {
					return new WP_Error( 'rest_cannot_create', __( 'Sorry, you don\'t have access to this website project.' ), array( 'status' => rest_authorization_required_code() ) );
				}

				// create page
				$page = PHW()->page->create_item( array(
					'title'     => isset( $request['page_title'] ) ? $request['page_title'] : $request['page_url'],
					'page_url'  => $request['page_url'],
					'parent_id' => $website_id
				) );

				// update thread parent
				update_post_meta( $post->ID, 'parent_id', $page['id'] );
				$response->data['parent_id'] = $page['id'];
			}
		}

		return $response;
	}

	/**
	 * Check for existing webpage
	 *
	 * @param $prepared_post object Prepared post object
	 * @param $request       object Request object
	 *
	 * @return object Request object
	 */
	public function always_publish( $prepared_post, $request ) {
		if ( $prepared_post->post_type == $this->post_type ) {
			$prepared_post->post_status = 'publish';
		}

		return $prepared_post;
	}

	/**
	 * Finds existing webpage post based on referring url
	 * @return mixed
	 */
	public function find_post( $post_id = false ) {
		global $post;

		// get referring URL to find correct webpage post
		$referring_url = parse_url( $_SERVER['HTTP_REFERER'] );

		// get existing pages on website
		$existing = (array) get_post_meta( $post->ID, 'ph_webpages', false );

		if ( empty( $existing ) || ! isset( $existing[0] ) || ! is_array( $existing[0] ) ) {
			return false;
		}

		// return webpage id if it exists
		return array_search( $referring_url['path'], $existing[0] );
	}

	/**
	 * Make sure comments are always open on our parent post
	 * type before comment is saved
	 *     *
	 * @return array
	 */
	public function force_comments_open( $open, $post_id ) {
		if ( get_post_type( $post_id ) === $this->post_type ) {
			if ( ! $open ) {
				$open = true;
			}
		}

		return $open;
	}

	/**
	 * Store approval date and user data when image is approved
	 *
	 * @param string  $attr  Updated attribute
	 * @param mixed   $value Attribute value
	 * @param WP_Post $post  Post object
	 */
	public function store_approval_comment( $attr, $value, $post ) {
		// if not our meta info or post type, bail
		if ( $attr !== 'resolved' || get_post_type( $post->ID ) !== $this->post_type ) {
			return;
		}

		// don't add if there are no comments yet
		if ( ! get_comments_number( $post->ID ) ) {
			return;
		}

		// get current user
		$user = wp_get_current_user();

		// Insert new comment and get the comment ID
		wp_insert_comment( array(
			'comment_post_ID'      => $post->ID,
			'comment_author'       => $user->display_name,
			'comment_author_email' => $user->user_email,
			'user_id'              => $user->ID,
			'comment_content'      => $value ? __( 'Approved', 'project-huddle' ) : __( 'Unapproved', 'project-huddle' ),
			'comment_type'         => 'ph_approval',
			'comment_approved'     => 1, // force approval
			'comment_meta'         => array(
				'approval' => (bool) $value,
			)
		) );
	}

	/**
	 * Make approval comment text dynamic
	 *
	 * @param string $comment_text
	 * @param null   $comment
	 *
	 * @return string Comment text
	 */
	function approval_comment_text( $comment_text = '', $comment = null ) {
		if ( $comment->comment_type !== 'ph_approval' || get_post_type( $comment->comment_post_ID ) !== $this->post_type ) {
			return $comment_text;
		}

		// get dynamic author and approval meta
		$user = get_userdata( $comment->user_id );
		if ( $user && $user->display_name ) {
			$author = $user->display_name;
		} else {
			$author = $comment->comment_author;
		}

		$approval = get_comment_meta( $comment->comment_ID, 'approval', true ) ? __( 'resolved', 'project-huddle' ) : __( 'unresolved', 'project-huddle' );

		// create comment text
		$comment_text = apply_filters( 'ph_approval_comment_text', sprintf( __( '%1$s %2$s this conversation.' ), esc_html( $author ), esc_html( $approval ) ), $comment );

		return $comment_text;
	}

	/**
	 * Sort images by menu order to keep order correct.
	 * Uses date as a fallback
	 *
	 * @param array $args Arguments for get request.
	 *
	 * @return array
	 */
	public function sort_by_menu_order( $args ) {
		$args['order_by'] = 'menu_order date';
		return $args;
	}
}