<?php
/**
 * Email functions
 */

/**
 * Email subscribed people about new comments
 *
 * @param integer    $id      Inserted or updated comment id.
 * @param WP_Comment $comment Inserted or updated comment object.
 *
 * @return void
 * @throws Exception
 */
function ph_website_new_comment_email( $id, $comment ) {
	// get ids of parent posts
	$parents = ph_get_parents_ids( $id, 'comment' );

	// get website
	$page = get_post( $parents['website-page'] );

	// get all emails
	if ( ! $to_emails = ph_get_project_emails( $parents['website'], get_user_by( 'email', $comment->comment_author_email ) ) ) {
		return;
	}

	// filter
	$to_emails = (array) apply_filters( 'ph_website_create_comment_email_to', $to_emails, $comment );

	// send each email individually
	foreach ( $to_emails as $email ) {
		$email = ( new PH_Mail() )
			->template(
				ph_locate_template( 'email/new-comment-email.php' ), [
					'commenter'    => sanitize_text_field( $comment->comment_author ),
					'avatar'       => get_avatar(
						get_current_user_id(), 42, '', '', array(
							'force_display' => true,
							'extra_attr'    => 'style="border-radius: 42px"',
						)
					),
					'project_name' => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['website'] ) ) ),
					'item_name'    => sanitize_text_field( html_entity_decode( $page->post_title ) ),
					'content'      => wpautop( $comment->comment_content ),
					'link'         => ph_email_link( get_permalink( (int) $comment->comment_post_ID ), __( 'View Comment', 'project-huddle' ) ),
				]
			)
			->to( $email )
			->subject( apply_filters( 'ph_website_new_comment_email_subject', sprintf( __( '%1$1s made a new comment on %2$2s', 'project-huddle' ), '{{commenter}}', '{{project_name}}' ), $id, $comment ) )
			->send();
	}
}

add_action( 'ph_website_publish_comment', 'ph_website_new_comment_email', 10, 2 );

/**
 * Email subscribed people about new comments
 *
 * @param boolean $value  Meta value data
 * @param WP_Post $thread Post object
 *
 * @return void
 * @throws Exception
 */
function ph_website_assigned_thread_email( $attr, $value, $thread ) {
	// must be resolved attribute and have value
	if ( 'assigned' !== $attr ||  ! isset( $value ) ) {
		return;
	}

	$user         = get_userdata( (int) $value );
	$current_user = wp_get_current_user();

	// need a user
	if ( ! $user ) {
		return;
	}

	// don't notify yourself, silly!
	if ( $user->ID === $current_user->ID ) {
		return;
	}

	// get assigned user email
	if ( ! $to_emails = array( sanitize_email( $user->user_email ) ) ) {
		return;
	}

	// filter
	$to_emails = (array) apply_filters( 'ph_website_assigned_thread_email_to', $to_emails, $thread, $value );

	// get parent ids
	$parents = ph_get_parents_ids( $thread->ID );

	// get original comment
	$first_comment = ph_get_comments(
		array(
			'post_id'       => $thread->ID,
			'ph_force_show' => true,
			'number'        => 1,
			'order'         => 'ASC',
		)
	);

	$content = ! empty( $first_comment ) ? $first_comment[0]->comment_content : $thread->post_content;

	// send each email individually
	foreach ( $to_emails as $email ) {
		$email = ( new PH_Mail() )
			->template(
				ph_locate_template( 'email/assigned-thread-email.php' ), [
					'commenter'    => sanitize_text_field( $current_user->display_name ),
					'content'      => wpautop( wp_kses_post( $content ) ),
					'avatar'       => get_avatar(
						$current_user->ID, 42, '', '', array(
							'force_display' => true,
							'extra_attr'    => 'style="border-radius: 42px"',
						)
					),
					'item_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['website-page'] ) ) ),
					'page_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['website-page'] ) ) ),
					'project_name' => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['website'] ) ) ),
					'link'         => ph_email_link( get_permalink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) ),
				]
			)
			->subject( apply_filters( 'ph_website_assigned_thread_email_subject', sprintf( __( 'You have been assigned to an issue on %1s.', 'project-huddle' ), '{{project_name}}' ), $value, $thread ) )
			->to( $email )
			->send();
	}
}

add_action( 'ph_website_rest_update_thread_attribute', 'ph_website_assigned_thread_email', 10, 3 );

/**
 * Email subscribed people about new comments
 *
 * @param boolean $value  Meta value data
 * @param WP_Post $thread Post object
 *
 * @return void
 *
 * @throws Exception
 */
function ph_resolved_thread_email( $attr, $value, $thread ) {
	global $is_ph_batch;

	// don't notify for batch requests
	if ( $is_ph_batch ) {
		return;
	}

	// must be resolved attribute and have value
	if ( 'resolved' !== $attr ||  ! isset( $value ) ) {
		return;
	}

	// get current user
	$current_user = wp_get_current_user();

	// get ids of parent posts
	$parents = ph_get_parents_ids( $thread->ID );

	// get original comment
	$first_comment = ph_get_comments(
		array(
			'post_id'       => $thread->ID,
			'ph_force_show' => true,
			'number'        => 1,
			'order'         => 'ASC',
		)
	);

	if ( ! $first_comment ) {
		return;
	}

	// get emails
	if ( ! $to_emails = ph_get_project_emails( $parents['project'], get_userdata( $current_user->user_email ) ) ) {
		return;
	}

	// backwards compat
	if ( 'ph-website' === get_post_type( $parents['project'] ) ) {
		$to_emails = (array) apply_filters( 'ph_website_resolved_thread_email_to', $to_emails, $thread, $value );
	}

	// @since 3.0.0
	$to_emails = (array) apply_filters( 'ph_resolved_thread_email_to', $to_emails, $thread, $value );

	if ( $value ) {
		$subject = sprintf( __( '%1$1s marked an issue on %2$2s unresolved.', 'project-huddle' ), '{{commenter}}', '{{project_name}}' );

		// backwards compat
		if ( 'ph-website' === get_post_type( $parents['project'] ) ) {
			$subject = apply_filters( 'ph_website_resolved_thread_email_subject', $subject, $value, $thread );
		}

		// send each email individually
		foreach ( $to_emails as $email ) {
			$email = ( new PH_Mail() )
				->template(
					ph_locate_template( 'email/resolved-thread-email.php' ), [
						'commenter'    => sanitize_text_field( $current_user->display_name ),
						'avatar'       => get_avatar(
							$current_user->ID, 42, '', '', array(
								'force_display' => true,
								'extra_attr'    => 'style="border-radius: 42px"',
							)
						),
						'content'      => wpautop( wp_kses_post( $first_comment[0]->comment_content ) ),
						'item_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['item'] ) ) ),
						'page_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['item'] ) ) ),
						'project_name' => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['project'] ) ) ),
						'link'         => ph_email_link( get_permalink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) ),
					]
				)
				->subject( apply_filters( 'ph_resolved_thread_email_subject', sprintf( __( '%1$1s resolved an issue on %2$2s.', 'project-huddle' ), '{{commenter}}', '{{project_name}}' ), $value, $thread ) )
				->to( $email )
				->send();
		}
	} else {
		$subject = sprintf( __( '%1$1s marked an issue on %2$2s unresolved.', 'project-huddle' ), '{{commenter}}', '{{project_name}}' );

		if ( 'ph-website' === get_post_type( $parents['project'] ) ) {
			$subject = apply_filters( 'ph_website_resolved_thread_email_subject', $subject, $value, $thread );
		}

		// send each email individually
		foreach ( $to_emails as $email ) {
			$email = ( new PH_Mail() )
				->template(
					ph_locate_template( 'email/unresolved-thread-email.php' ), [
						'commenter'    => sanitize_text_field( $current_user->display_name ),
						'avatar'       => get_avatar(
							$current_user->ID, 42, '', '', array(
								'force_display' => true,
								'extra_attr'    => 'style="border-radius: 42px"',
							)
						),
						'content'      => wpautop( wp_kses_post( $first_comment[0]->comment_content ) ),
						'item_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['item'] ) ) ),
						'page_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['item'] ) ) ),
						'project_name' => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['project'] ) ) ),
						'link'         => ph_email_link( get_permalink( (int) $thread->ID ), __( 'View Comment', 'project-huddle' ) ),
					]
				)
				->subject( apply_filters( 'ph_unresolved_thread_email_subject', $subject ) )
				->to( $email )
				->send();
		}
	}
}

add_action( 'ph_website_rest_update_thread_attribute', 'ph_resolved_thread_email', 10, 3 );
add_action( 'ph_mockup_rest_update_thread_attribute', 'ph_resolved_thread_email', 10, 3 );

/**
 * Email a new member that they've been subscribe to a project
 *
 * @param $data
 *
 * @throws Exception
 */
function ph_website_email_new_member( $attr, $value, $post  ) {
	// bail if not our info
	if ( 'project_members' !== $attr ) {
		return;
	}

	$members = array_filter( array_unique( (array) $value ) );
	if ( ! $members || empty( $members ) ) {
		return;
	}

	$current_members = (array) ph_get_project_member_ids( $post->ID );
	$new_members = array_diff($members, $current_members);

	if ( ! isset( $new_members ) || empty( $new_members ) ) {
		return;
	}

	$emails = array();
	foreach ( $new_members as $member_id ) {
		$data = get_userdata( $member_id );
		if ( $data ) {
			$emails[] = $data->user_email;
		}
	}
	// filter
	$to_emails = (array) apply_filters( 'ph_website_member_subscribed_email_to', $emails, $post, $value );


	$current_user = wp_get_current_user();

	global $access_token_override;
	$access_token_override = true;

	// send each email individually
	foreach ( $to_emails as $email ) {
		// don't get your own
		if ( $email == $current_user->user_email ) {
			return;
		}

		$email = ( new PH_Mail() )
			->template(
				ph_locate_template( 'email/subscribe-project-email.php' ), [
					'commenter'    => sanitize_text_field( $current_user->display_name ),
					'avatar'       => get_avatar(
						$current_user->ID, 42, '', '', array(
							'force_display' => true,
							'extra_attr'    => 'style="border-radius: 42px"',
						)
					),
					'project_name' => sanitize_text_field( html_entity_decode( ph_get_the_title( $post->ID ) ) ),
					'link'         => ph_email_link( get_permalink( $post->ID ), __( 'View Project', 'project-huddle' ) ),
				]
			)
			->subject( apply_filters( 'ph_website_member_subscribed_email_subject', sprintf( __( 'You have been invited to collaborate on %1s.', 'project-huddle' ), '{{project_name}}' ), $post->ID, $email ) )
			->to( $email )
			->send();
	}
}

// fires immediately before updating metadata
add_action( "ph_website_pre_rest_update_project_attribute", 'ph_website_email_new_member', 10, 3 );

/**
 * Add user to notifications
 *
 * @param integer    $id Id of comment.
 * @param WP_Comment $comment Comment object.
 */
function ph_website_add_user_to_notifications( $id, $comment ) {
	// get author of comment.
	$author = get_user_by( 'ID', $comment->user_id );

	// need an author.
	if ( ! $author ) {
		return;
	}

	// get ids of parent posts.
	$parents = ph_get_parents_ids( $id, 'comment' );

	// add as project member.
	ph_add_project_member( $parents['website'], $author );
}

add_action( 'ph_website_publish_comment', 'ph_website_add_user_to_notifications', 10, 2 );
