( function( window, undefined ) {
	'use strict';

	/**
	 * Handles managing all events for whatever you plug it into. Priorities for hooks are based on lowest to highest in
	 * that, lowest priority hooks are fired first.
	 */
	var EventManager = function() {
		var slice = Array.prototype.slice;
		
		/**
		 * Maintain a reference to the object scope so our public methods never get confusing.
		 */
		var MethodsAvailable = {
			removeFilter : removeFilter,
			applyFilters : applyFilters,
			addFilter : addFilter,
			removeAction : removeAction,
			doAction : doAction,
			addAction : addAction
		};

		/**
		 * Contains the hooks that get registered with this EventManager. The array for storage utilizes a "flat"
		 * object literal such that looking up the hook utilizes the native object literal hash.
		 */
		var STORAGE = {
			actions : {},
			filters : {}
		};

		/**
		 * Adds an action to the event manager.
		 *
		 * @param action Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addAction( action, callback, priority, context ) {
			if( typeof action === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'actions', action, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs an action if it exists. You can pass as many arguments as you want to this function; the only rule is
		 * that the first argument must always be the action.
		 */
		function doAction( /* action, arg1, arg2, ... */ ) {
			var args = slice.call( arguments );
			var action = args.shift();

			if( typeof action === 'string' ) {
				_runHook( 'actions', action, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified action if it contains a namespace.identifier & exists.
		 *
		 * @param action The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeAction( action, callback ) {
			if( typeof action === 'string' ) {
				_removeHook( 'actions', action, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Adds a filter to the event manager.
		 *
		 * @param filter Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addFilter( filter, callback, priority, context ) {
			if( typeof filter === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'filters', filter, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs a filter if it exists. You should only ever pass 1 argument to be filtered. The only rule is that
		 * the first argument must always be the filter.
		 */
		function applyFilters( /* filter, filtered arg, arg2, ... */ ) {
			var args = slice.call( arguments );
			var filter = args.shift();

			if( typeof filter === 'string' ) {
				return _runHook( 'filters', filter, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified filter if it contains a namespace.identifier & exists.
		 *
		 * @param filter The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeFilter( filter, callback ) {
			if( typeof filter === 'string') {
				_removeHook( 'filters', filter, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified hook by resetting the value of it.
		 *
		 * @param type Type of hook, either 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to remove
		 * @private
		 */
		function _removeHook( type, hook, callback, context ) {
			var handlers, handler, i;
			
			if ( !STORAGE[ type ][ hook ] ) {
				return;
			}
			if ( !callback ) {
				STORAGE[ type ][ hook ] = [];
			} else {
				handlers = STORAGE[ type ][ hook ];
				if ( !context ) {
					for ( i = handlers.length; i--; ) {
						if ( handlers[i].callback === callback ) {
							handlers.splice( i, 1 );
						}
					}
				}
				else {
					for ( i = handlers.length; i--; ) {
						handler = handlers[i];
						if ( handler.callback === callback && handler.context === context) {
							handlers.splice( i, 1 );
						}
					}
				}
			}
		}

		/**
		 * Adds the hook to the appropriate storage container
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to add to our event manager
		 * @param callback The function that will be called when the hook is executed.
		 * @param priority The priority of this hook. Must be an integer.
		 * @param [context] A value to be used for this
		 * @private
		 */
		function _addHook( type, hook, callback, priority, context ) {
			var hookObject = {
				callback : callback,
				priority : priority,
				context : context
			};

			// Utilize 'prop itself' : http://jsperf.com/hasownproperty-vs-in-vs-undefined/19
			var hooks = STORAGE[ type ][ hook ];
			if( hooks ) {
				hooks.push( hookObject );
				hooks = _hookInsertSort( hooks );
			}
			else {
				hooks = [ hookObject ];
			}

			STORAGE[ type ][ hook ] = hooks;
		}

		/**
		 * Use an insert sort for keeping our hooks organized based on priority. This function is ridiculously faster
		 * than bubble sort, etc: http://jsperf.com/javascript-sort
		 *
		 * @param hooks The custom array containing all of the appropriate hooks to perform an insert sort on.
		 * @private
		 */
		function _hookInsertSort( hooks ) {
			var tmpHook, j, prevHook;
			for( var i = 1, len = hooks.length; i < len; i++ ) {
				tmpHook = hooks[ i ];
				j = i;
				while( ( prevHook = hooks[ j - 1 ] ) &&  prevHook.priority > tmpHook.priority ) {
					hooks[ j ] = hooks[ j - 1 ];
					--j;
				}
				hooks[ j ] = tmpHook;
			}

			return hooks;
		}

		/**
		 * Runs the specified hook. If it is an action, the value is not modified but if it is a filter, it is.
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook ( namespace.identifier ) to be ran.
		 * @param args Arguments to pass to the action/filter. If it's a filter, args is actually a single parameter.
		 * @private
		 */
		function _runHook( type, hook, args ) {
			var handlers = STORAGE[ type ][ hook ], i, len;
			
			if ( !handlers ) {
				return (type === 'filters') ? args[0] : false;
			}

			len = handlers.length;
			if ( type === 'filters' ) {
				for ( i = 0; i < len; i++ ) {
					args[ 0 ] = handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			} else {
				for ( i = 0; i < len; i++ ) {
					handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			}

			return ( type === 'filters' ) ? args[ 0 ] : true;
		}

		// return all of the publicly available methods
		return MethodsAvailable;

	};
	
	window.wp = window.wp || {};
	window.wp.hooks = new EventManager();

} )( window );

/*
 * Content-Type:text/javascript
 *
 * A bridge between touch events and jquery draggable,
 * sortable etc. mouse interactions.
 * @author Oleg Slobodskoi
 *
 * modified by John Hardy to use with any touch device
 * fixed breakage caused by jquery.ui so that mouseHandled internal flag is reset
 * before each touchStart event
 *
 */
(function( $ ) {
	$.support.touch = typeof Touch === 'object';

	if (!$.support.touch) {
		return;
	}

	var proto =  $.ui.mouse.prototype,
	    _mouseInit = proto._mouseInit;

	$.extend( proto, {
		_mouseInit: function() {
			this.element
				.bind( "touchstart." + this.widgetName, $.proxy( this, "_touchStart" ) );
			_mouseInit.apply( this, arguments );
		},

		_touchStart: function( event ) {
			if ( event.originalEvent.targetTouches.length != 1 ) {
				return false;
			}

			this.element
				.bind( "touchmove." + this.widgetName, $.proxy( this, "_touchMove" ) )
				.bind( "touchend." + this.widgetName, $.proxy( this, "_touchEnd" ) );

			this._modifyEvent( event );

			$( document ).trigger($.Event("mouseup")); //reset mouseHandled flag in ui.mouse
			this._mouseDown( event );

			return false;
		},

		_touchMove: function( event ) {
			this._modifyEvent( event );
			this._mouseMove( event );
		},

		_touchEnd: function( event ) {
			this.element
				.unbind( "touchmove." + this.widgetName )
				.unbind( "touchend." + this.widgetName );
			this._mouseUp( event );
		},

		_modifyEvent: function( event ) {
			event.which = 1;
			var target = event.originalEvent.targetTouches[0];
			event.pageX = target.clientX;
			event.pageY = target.clientY;
		}

	});

})( jQuery );
/**
 * Defines Project Huddle namespace
 * @since 1.0
 */

// strict mode
'use strict';

window.wp = window.wp || {};

// global namespace
var Huddle = window.Huddle = {

	start: function ( data ) {
		console.log(data);

		this.setGlobals( data );

		// one main global container to store view
		Huddle.container = new Huddle.ImageCollectionView( { collection: Huddle.images } );
		Huddle.container.inject( '.ph-projects-wrapper' );

		// broadcast start
		wp.hooks.doAction( 'Huddle.start', this );
	},

	setGlobals: function ( data ) {
		// get options
		Huddle.options = data ? data.options : false;

		// get translations
		Huddle.translations = data ? data.translations : false;

		// create image collection model
		Huddle.images = data ? new Huddle.ImageCollection( data.images ) : new Huddle.ImageCollection();
	}
};
/**
 * Mixins used in the app. Used to sync admin-ajax.php with backbone
 *
 * @package     ProjectHuddle
 * @subpackage  js
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0
 */

/**
 * A mixin for collections/models that syncs with app-ajax.php
 *
 * @since 1.0
 */
var adminAjaxSyncableMixin = {

    // ajax url
    url:  ajaxurl,

    // sync function
    sync: function (method, object, options) {

        // check if data is provided
        if (typeof options.data === 'undefined') {
            options.data = {};
        }

        // send default nonce if not provided
        if ( ! options.data.nonce ) {
            options.data.nonce = PH.nonce;
        }

        options.data.action_type = method;

        // set default action if nothing is set
        if (typeof options.data.action === 'undefined') {
            options.data.action = false;
        }

        /**
         * Convert model or collection to json
         */
        var json = this.toJSON();
        var formattedJSON = {};

        /**
         * If the json is an array, it's a collection
         * Otherwise, its a model
         */
        if (json instanceof Array) {
            formattedJSON.models = json;
        } else {
            formattedJSON.model = json;
        }

        /**
         * Extend the options.data object with the formatted json
         */
        _.extend(options.data, formattedJSON);

        /**
         * Prevent needing to json decode in php and use $_POST instead
         */
        options.emulateJSON = true;

        /**
         * Call Backbone sync method
         */
        return Backbone.sync.call(this, 'create', object, options);
    }
};

/**
 * A model for all the syncable models to extend
 *
 * @since 1.0
 */
var BaseModel = Backbone.Model.extend(_.defaults({
    parse: function (response) {
        return response
    }
}, adminAjaxSyncableMixin));

/**
 * A collection for all the syncable collections to extend
 *
 * @since 1.0
 */
var BaseCollection = Backbone.Collection.extend(_.defaults({
    parse: function (response) {
        return response
    }
}, adminAjaxSyncableMixin));

/**
 * A view for all wp views to extend
 *
 * Extends wp.Backbone.View with .prepare() and .inject()
 *
 * @since 1.0.0
 */
Huddle.View = wp.Backbone.View.extend({

    inject: function( selector ) {
        this.render();
        jQuery(selector).html( this.el );
        this.views.ready();
    },

    injectTo: function( selector ) {
        this.render();
        jQuery(selector).append( this.el );
        this.views.ready();
    },

    prepare: function() {
        if ( ! _.isUndefined( this.model ) && _.isFunction( this.model.toJSON ) ) {
            this._parseTriggers();
            return this.model.toJSON();
        } else {
            return {};
        }
    },

    _parseTriggers: function () {
        _.each( this.triggers || {}, function ( fn, event ) {
            var handler = this[fn];
            if ( _.isFunction( handler ) ) {
                this.listenTo( this.model, event, handler );
            }
        }, this );
    }
});

/**
 * Declare global events in namespace
 */
Huddle.Event = _.extend( {}, Backbone.Events );
/**
 * Model to store our individual images
 *
 * @extends BaseModel Sync, save, fetch, update functions with app-ajax
 *
 * @since 1.0
 */
Huddle.Image = wp.api.models.MockupImage.extend(_.defaults({
	defaults: _.defaults({}, wp.api.models.MockupImage.prototype.defaults, {
        id: null,
        url: null,
        imageID: null,
        title: 'Image',
        parent: 0,
        image: {},
        comment_count: 0,
        hash: '',
        options: {
            ph_image_alignment: 'center', // left, right or center
            ph_image_size: 'normal', // normal or scale
            ph_image_background_color: '', // hex value
            ph_image_background_image: '', // url
            ph_image_background_image_position: 'center' // repeat, repeat-x, cover
        }
    }),
}, Huddle.BaseModel));
/**
 * Collection to store all the images for this post
 *
 * @extends BaseCollection Sync, save, fetch, update functions with app-ajax
 *
 * @since 1.0
 */
Huddle.ImageCollection = BaseCollection.extend({

    /**
     * Use image model in the collection
     */
    model: Huddle.Image,

	/**
     * Broadcast initialize
     */
    initialize: function(){
        wp.hooks.doAction( 'Huddle.ImageCollection.initialize', this );
    },

    /**
     * Keep track of model order
     */
    comparator: function(model) {
        return model.get('ordinal');
    }

} );
/**
 * Container that holds the all the images
 */
Huddle.ImageCollectionView = Huddle.View.extend({
    template: wp.template('project-collection'),
    collection: Huddle.ImageCollection,

    events: {
        'click #ph-button-add-media': 'openAddMedia'
    },

    initialize: function () {
        this.wp_media_post_id = wp.media.model.settings.post.id; // Store the old id
        this.file_frame       = false;

        // add image views
        _.each(this.collection.models, this.loadImageSubviews, this);

        this.listenTo(this.collection, 'add', this.loadImageSubviews);
        this.listenTo(this.collection, 'add', this.showNotice);
        this.listenTo(this.collection, 'change', this.showNotice);
        this.listenTo(this.collection, 'dropped', this.showNotice);

        this.$('input').on('change', _.bind(this.showNotice, this));

        // broadcast init
        wp.hooks.doAction('Huddle.ImageCollectionView.initialize', this);
    },

    // create a new subview for each image
    loadImageSubviews: function (image) {
        this.views.add('#ph-gallery-indicator', new Huddle.ImageView({
            model: image
        }));
    },

    // dom ready event
    ready: function () {
        // create sortable event
        this.$el.find("#ph-gallery-attachments").sortable({
            items: ".ph-project",
            handle: '.ph-image-wrapper',
            start: function (event, ui) {
                ui.item.data('start_pos', ui.item.index());
            },
            stop: _.bind(function (event, ui) {
                var start_pos = ui.item.data('start_pos');

                // trigger only if order changed
                if (start_pos != ui.item.index()) {
                    this.collection.trigger('dropped', this.model);
                }
            }, this)
        });

        // remove loading
        this.$('.spinner').remove();
    },

    /**
     * Shows save notice
     */
    showNotice: function () {
        window.onbeforeunload = Huddle.confirmOnPageExit;
        this.$('.ph-update-notice').addClass('show-notice');
    },

    /**
     * Event when add media link in clicked
     */
    openAddMedia: function (e) {

        // prevent default action
        e.preventDefault();

        // If the media frame already exists, reopen it.
        if (this.file_frame) {
            this.file_frame.open();
            return;
        }

        // Create the media frame
        this.file_frame = wp.media({
            title: jQuery(e.target).data('uploader-title'),
            button: {
                text: jQuery(e.target).data('uploader-button-text')
            },
            multiple: 'toggle'  // Set to true to allow multiple files to be selected
        });

        // select event for frame
        this.file_frame.on('select', _.bind(function () {

            // When an image is selected, run a callback.
            var selection = this.file_frame.state().get('selection');

            if (selection.length) {
                selection.map(function (attachment) {

                    // convert attachment info to json
                    attachment = attachment.toJSON();

                    var imageURL = false;

                    if (typeof attachment.sizes.large == 'undefined') {
                        imageURL = attachment.sizes.full.url
                    } else {
                        imageURL = attachment.sizes.large.url
                    }

                    // create new image
                    var image = new Huddle.Image({
                        id: attachment.id,
                        thumbnail: imageURL,
                        url: attachment.sizes.full.url,
                        imageID: attachment.id,
                        title: attachment.title
                    });

                    // add image to collection and render
                    Huddle.images.add(image);
                });

                // Restore the main post ID
                wp.media.model.settings.post.id = this.wp_media_post_id;
            }

        }, this));

        // Finally, open the modal
        this.file_frame.open();

    }
});
/**
 * View to show our settings item
 *
 * @since 1.0
 */

// this lets us to call initialization and events is subviews
Huddle.ImageOptionsMenuItemView = function (options) { Huddle.View.apply(this, arguments); };
Huddle.ImageOptionsMenuItemView.extend = Huddle.View.extend;

/**
 * ImageOptionsMenuItemView
 */
_.extend( Huddle.ImageOptionsMenuItemView.prototype, Huddle.View.prototype, {

	tagName: 'li',
	template: wp.template('display-options-item'),
	className: 'ph-project-action ph-display-options',

	// events to trigger on model
	triggers: {
		'click': 'openOptionsWindow'
	}
});

Huddle.ImageThumbnailView = Huddle.View.extend({
	template: wp.template('image-thumbnail'),
	className: 'ph-image',

	initialize: function () {
		this.model.getFeaturedMedia().done(_.bind(function (media) {
			this.model.set('image', media);
		}, this));

		this.listenTo(this.model, 'change:image', this.render);
	}
});
/**
 * View to show our individual image
 *
 * @since 1.0
 */

// this lets us to call initialization and events is subviews
Huddle.ImageView = function (options) { Huddle.View.apply(this, arguments); };
Huddle.ImageView.extend = Huddle.View.extend;

/**
 * ImageView
 */
_.extend( Huddle.ImageView.prototype, Huddle.View.prototype, {

    tagName: 'div',
    template: wp.template('single-project'),
    className: 'ph-project',

    events: {
        "click .ph-title .ph-title-inner": "editTitle",
        'click .ph-edit-cancel': 'cancelEdit',
	    'click .ph-edit-save:enabled': 'updateComment',
        "click .ph-display-options": "openOptionsWindow",
	    'focus .ph-edit-content': 'elastic'
    },

    /**
     * Simple object to store any UI elements we need to use
     * over the life of the view.
     */
    ui: {
        modal: undefined,
	    versionsModal: undefined
    },

    /**
     * Set templates and listen for model events
     *
     * Not called if a extended view is constructed.
     */
    initialize: function () {
        // when model event 'moveToTrash' is triggered, run moveToTrash function
        this.listenTo( this.model, 'moveToTrash', this.moveToTrash );

        // when model event 'titleTextChanged' is triggered, run titleChanged function
        this.listenTo( this.model, 'titleTextChanged', this.titleChanged );

        // when attributes are changed, update the view
        this.listenTo( this.model, 'change:options', this.render );

        // open options window event
        this.listenTo( this.model, 'openOptionsWindow', this.openOptionsWindow );

        // set image thumbnail view
		this.views.set('.ph-image', new Huddle.ImageThumbnailView({
			model: this.model
		}));

        // add menu items
        this.addMenuViews();

        // broadcast init
        wp.hooks.doAction( 'Huddle.ImageView.initialize', this );
    },

    addMenuViews: function() {
        // add trash item
        this.views.add( '.ph-settings-menu', new Huddle.TrashImageMenuItemView( {
            model: this.model
        } ) );
        // add options item
        this.views.add( '.ph-settings-menu', new Huddle.ImageOptionsMenuItemView( {
            model: this.model
        } ) );
    },

    editTitle: function(e) {
        typeof e.preventDefault == 'function' && e.preventDefault();

        this.$( '.ph-title-inner' ).hide();
        this.$( '.ph-edit-textarea' ).show();
        this.$( '.ph-edit-content' ).putCursorAtEnd();
    },

    cancelEdit: function( e ) {
        typeof e.preventDefault == 'function' && e.preventDefault();

        this.$( '.ph-edit-textarea' ).hide();
	    this.$( '.ph-edit-content' ).val( this.model.attributes.title ).blur();
        this.$( '.ph-title-inner' ).show();
    },

	/**
	 * Saves new comment update
	 *
	 * @param e jquery event
	 */
	updateComment: function(e) {
		this.model.set( 'title', this.$( 'textarea.ph-edit-content' )[0].value );

		// render again to update input
		this.render();
	},

	/**
	 * Elasicizes textarea
	 */
	elastic: function( e ){
		autosize( e.currentTarget );
	},

    /**
     * Opens options window to change project image properties
     *
     * @param e {object} A jQuery-normalized event object.
     */
    openOptionsWindow: function (e) {
        // new modal box
        this.ui.modal = new Huddle.Modal({
            model: this.model, // pass model
            imageView: this // pass this view
        });
        return false;
    },

    /**
     * Move model to the trashbin
     */
    moveToTrash: function () {
        // move to trash
        this.model.save(undefined, {
            data: {
                action: 'move_image_to_trash'
            },
            success: function (model, response, options) {
                console.log(response);
                console.log("The model has been moved to the trash.");
            },
            error: function (model, xhr, options) {
                console.log(xhr);
                console.log("Something went wrong while moving the model to the trash.");
            }
        });

        // remove view
        this.remove();
    }

});

/**
 * View to show our modal box
 *
 * @since 1.0
 */

/**
 * Primary Modal Application Class
 */
Huddle.Modal = Huddle.View.extend( {

	id: "backbone_modal_dialog",

	events: {
		"click .backbone_modal-close": "closeModal",
		"click #btn-cancel": "closeModal",
		"click #btn-ok": "saveModal"
	},

	initialize: function () {
		// use WordPress templating
		this.template = wp.template( 'modal-window' );

		// listen to move to trash model event
		this.listenTo( this.model, 'moveToTrash', this.closeModal );

		// run render on init
		this.render();

		// broadcast init
		wp.hooks.doAction( 'Huddle.Modal.initialize', this );
	},

	/**
	 * Renders modal box.
	 */
	render: function () {
		this.$el.attr( 'tabindex', '0' )
			.append( this.template( this.model.attributes ) );

		// Handle any attempt to move focus out of the modal.
		jQuery( document ).on( "focusin", _.bind ( function ( e ) {
			this.preserveFocus( e, this );
		}, this ) );

		// set overflow to "hidden" on the body so that it ignores any scroll events while the modal is active
		// append modal to body
		jQuery( "body" ).addClass( 'ph-modal-no-overflow' ).append( this.$el );

		// Set focus on the modal to prevent accidental actions in the underlying page
		// Not strictly necessary, but nice to do.
		this.$el.focus();

		this.$( ".ph-button-set" ).buttonset();

		this.colorPicker();
	},

	/**
	 * ColorPicker
	 */
	colorPicker: function () {
		this.$( '.colorpicker' ).wpColorPicker();
	},

	/**
	 * Ensures that keyboard focus remains within the Modal dialog.
	 *
	 * @param e {object} A jQuery-normalized event object.
	 * @param base {object} Base view object
	 */
	preserveFocus: function ( e, base ) {
		if ( base.$el[0] !== e.target && !base.$el.has( e.target ).length ) {
			base.$el.focus();
		}
	},

	/**
	 * Closes the modal and cleans up after the instance.
	 *
	 * @param e {object} A jQuery-normalized event object.
	 */
	closeModal: function ( e ) {
		typeof e.preventDefault == 'function' && e.preventDefault();

		// Undelegate
		this.undelegateEvents();

		// Remove focus and body css
		jQuery( document ).off( "focusin" );
		jQuery( "body" ).removeClass( 'ph-modal-no-overflow' );

		// remove and unbind
		this.$el.removeData().unbind();

		//Remove view from DOM
		this.remove();

		// Remove
		Backbone.View.prototype.remove.call( this );
	},

	getFormData: function () {
		var arr = this.$el.find( 'form' ).serializeArray();
		return _( arr ).reduce( function ( acc, field ) {
			acc[field.name] = field.value;
			return acc;
		}, {} );
	},

	/**
	 * Responds to the btn-ok.click event
	 *
	 * @param e {object} A jQuery-normalized event object
	 */
	saveModal: function ( e ) {
		typeof e.preventDefault == 'function' && e.preventDefault();

		// extend form data with defaults
		this.model.set( {
			'options': _.defaults( this.getFormData(), this.model.attributes.options )
		} );

		this.closeModal( e );
	}

} );
/**
 * View to show our settings item
 *
 * @since 1.0
 */

// this lets us to call initialization and events is subviews
Huddle.TrashImageMenuItemView = function (options) { Huddle.View.apply(this, arguments); };
Huddle.TrashImageMenuItemView.extend = Huddle.View.extend;

/**
 * TrashImageMenuItemView
 */
_.extend( Huddle.TrashImageMenuItemView.prototype, Huddle.View.prototype, {

	tagName: 'li',
	template: wp.template('trash-image-item'),
	className: 'ph-project-action ph-delete',

	// events to trigger on model
	events: {
		'click': 'confirmTrash'
	},

	/**
	 * Adds a confirmation message before moving to trash.
	 *
	 * @param e {object} A jQuery-normalized event object.
	 */
	confirmTrash: function ( e ) {
		// prevent default
		typeof e.preventDefault == 'function' && e.preventDefault();

		var confirmString = Huddle.translations.confirmTrash ? Huddle.translations.confirmTrash : 'Are you sure you want to trash';

		// filter confirm text
		var r = confirm( wp.hooks.applyFilters( 'huddle.trashConfirm', confirmString + ' "' + this.model.attributes.title + '"?' ) );
		if (r == true) {
			// trigger model event
			this.model.trigger( 'moveToTrash', this.model );
		}
	}
});

/**
 * Setup page events and collections
 *
 * @package     ProjectHuddle
 * @subpackage  js
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0
 */

(function($){

    // use strict mode
    'use strict';

    /**
     * Setup Page
     *
     * @since 1.0
     */
    $(document).ready(function(){

        // page exit confirmation
        Huddle.confirmOnPageExit = function ( e ) {
            // If we haven't been passed the event get the window.event
            e = e || window.event;

            var message = Huddle.translations && Huddle.translations.unsaved_changes
                ? Huddle.translations.unsaved_changes
                : 'It looks like there are unsaved changes on this project.';

            // For IE6-8 and Firefox prior to version 4
            if ( e ) {
                e.returnValue = message;
            }

            // For Chrome, Safari, IE8+ and Opera 12+
            return e.returnValue;
        };

	    $('#publish, #delete-action a').on( 'click', function () {
		    window.onbeforeunload = null;
	    });

	    // hide private functionality
	    $("#visibility-radio-private, label[for='visibility-radio-private']").hide().next( 'br' ).hide();

	    // set default
	    var dontClick = true;

	    // if submit box post visibility changes
	    $( '#post-visibility-select input[type="radio"]' ).change( function(){
		    if ($(this).is(':checked') && $(this).val() == 'password') {
		    	$( '#project_access_password' ).click();
		    } else {
			    if ( dontClick ) {
				    $( '#project_access_public' ).click();
			    }
		    }
	    });

	    // if our custom meta box visibility radio changes
	    $('.ph_project_access input[type="radio"]' ).change( function(){
            if ($(this).is(':checked') && $(this).val() == 'password') {

	            // edit visibility
                $( '.edit-visibility' ).click();

	            // password radio
                $( 'input#visibility-radio-password' ).click();

	            // scroll to div
	            $( 'html,body' ).animate({
		            scrollTop: $('#submitdiv' ).offset().top
	            }, 500);

	            // focus
                setTimeout(function(){
	                $( 'input#post_password' ).click().focus();
                }, 500 );
            } else {
	            var $postVisibilitySelect = $('#post-visibility-select');

	            // change to public
	            dontClick = false;
		        $( '#visibility-radio-public' ).click();
	            dontClick = true;

	            // slideup
	            $postVisibilitySelect.slideUp('fast');

	            // change text
	            $('#post-visibility-display').html(	postL10n[ $postVisibilitySelect.find('input:radio:checked').val() ]	);

	            $('#visibility .edit-visibility').show();
            }
        });
    });

}(jQuery));
/*!
 Autosize 3.0.5
 license: MIT
 http://www.jacklmoore.com/autosize
 */
!function(e,t){if("function"==typeof define&&define.amd)define(["exports","module"],t);else if("undefined"!=typeof exports&&"undefined"!=typeof module)t(exports,module);else{var o={exports:{}};t(o.exports,o),e.autosize=o.exports}}(this,function(e,t){"use strict";function o(e){function t(){var t=window.getComputedStyle(e,null);"vertical"===t.resize?e.style.resize="none":"both"===t.resize&&(e.style.resize="horizontal"),l="content-box"===t.boxSizing?-(parseFloat(t.paddingTop)+parseFloat(t.paddingBottom)):parseFloat(t.borderTopWidth)+parseFloat(t.borderBottomWidth),n()}function o(t){var o=e.style.width;e.style.width="0px",e.offsetWidth,e.style.width=o,u=t,a&&(e.style.overflowY=t),n()}function n(){var t=e.style.height,n=document.documentElement.scrollTop,i=document.body.scrollTop,r=e.style.height;e.style.height="auto";var d=e.scrollHeight+l;if(0===e.scrollHeight)return void(e.style.height=r);e.style.height=d+"px",document.documentElement.scrollTop=n,document.body.scrollTop=i;var s=window.getComputedStyle(e,null);if(s.height!==e.style.height){if("visible"!==u)return void o("visible")}else if("hidden"!==u)return void o("hidden");if(t!==e.style.height){var a=document.createEvent("Event");a.initEvent("autosize:resized",!0,!1),e.dispatchEvent(a)}}var i=void 0===arguments[1]?{}:arguments[1],r=i.setOverflowX,d=void 0===r?!0:r,s=i.setOverflowY,a=void 0===s?!0:s;if(e&&e.nodeName&&"TEXTAREA"===e.nodeName&&!e.hasAttribute("data-autosize-on")){var l=null,u="hidden",v=function(t){window.removeEventListener("resize",n),e.removeEventListener("input",n),e.removeEventListener("keyup",n),e.removeAttribute("data-autosize-on"),e.removeEventListener("autosize:destroy",v),Object.keys(t).forEach(function(o){e.style[o]=t[o]})}.bind(e,{height:e.style.height,resize:e.style.resize,overflowY:e.style.overflowY,overflowX:e.style.overflowX,wordWrap:e.style.wordWrap});e.addEventListener("autosize:destroy",v),"onpropertychange"in e&&"oninput"in e&&e.addEventListener("keyup",n),window.addEventListener("resize",n),e.addEventListener("input",n),e.addEventListener("autosize:update",n),e.setAttribute("data-autosize-on",!0),a&&(e.style.overflowY="hidden"),d&&(e.style.overflowX="hidden",e.style.wordWrap="break-word"),t()}}function n(e){if(e&&e.nodeName&&"TEXTAREA"===e.nodeName){var t=document.createEvent("Event");t.initEvent("autosize:destroy",!0,!1),e.dispatchEvent(t)}}function i(e){if(e&&e.nodeName&&"TEXTAREA"===e.nodeName){var t=document.createEvent("Event");t.initEvent("autosize:update",!0,!1),e.dispatchEvent(t)}}var r=null;"undefined"==typeof window||"function"!=typeof window.getComputedStyle?(r=function(e){return e},r.destroy=function(e){return e},r.update=function(e){return e}):(r=function(e,t){return e&&Array.prototype.forEach.call(e.length?e:[e],function(e){return o(e,t)}),e},r.destroy=function(e){return e&&Array.prototype.forEach.call(e.length?e:[e],n),e},r.update=function(e){return e&&Array.prototype.forEach.call(e.length?e:[e],i),e}),t.exports=r});
( function( window, undefined ) {
	'use strict';

	/**
	 * Handles managing all events for whatever you plug it into. Priorities for hooks are based on lowest to highest in
	 * that, lowest priority hooks are fired first.
	 */
	var EventManager = function() {
		var slice = Array.prototype.slice;
		
		/**
		 * Maintain a reference to the object scope so our public methods never get confusing.
		 */
		var MethodsAvailable = {
			removeFilter : removeFilter,
			applyFilters : applyFilters,
			addFilter : addFilter,
			removeAction : removeAction,
			doAction : doAction,
			addAction : addAction
		};

		/**
		 * Contains the hooks that get registered with this EventManager. The array for storage utilizes a "flat"
		 * object literal such that looking up the hook utilizes the native object literal hash.
		 */
		var STORAGE = {
			actions : {},
			filters : {}
		};

		/**
		 * Adds an action to the event manager.
		 *
		 * @param action Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addAction( action, callback, priority, context ) {
			if( typeof action === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'actions', action, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs an action if it exists. You can pass as many arguments as you want to this function; the only rule is
		 * that the first argument must always be the action.
		 */
		function doAction( /* action, arg1, arg2, ... */ ) {
			var args = slice.call( arguments );
			var action = args.shift();

			if( typeof action === 'string' ) {
				_runHook( 'actions', action, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified action if it contains a namespace.identifier & exists.
		 *
		 * @param action The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeAction( action, callback ) {
			if( typeof action === 'string' ) {
				_removeHook( 'actions', action, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Adds a filter to the event manager.
		 *
		 * @param filter Must contain namespace.identifier
		 * @param callback Must be a valid callback function before this action is added
		 * @param [priority=10] Used to control when the function is executed in relation to other callbacks bound to the same hook
		 * @param [context] Supply a value to be used for this
		 */
		function addFilter( filter, callback, priority, context ) {
			if( typeof filter === 'string' && typeof callback === 'function' ) {
				priority = parseInt( ( priority || 10 ), 10 );
				_addHook( 'filters', filter, callback, priority, context );
			}

			return MethodsAvailable;
		}

		/**
		 * Performs a filter if it exists. You should only ever pass 1 argument to be filtered. The only rule is that
		 * the first argument must always be the filter.
		 */
		function applyFilters( /* filter, filtered arg, arg2, ... */ ) {
			var args = slice.call( arguments );
			var filter = args.shift();

			if( typeof filter === 'string' ) {
				return _runHook( 'filters', filter, args );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified filter if it contains a namespace.identifier & exists.
		 *
		 * @param filter The action to remove
		 * @param [callback] Callback function to remove
		 */
		function removeFilter( filter, callback ) {
			if( typeof filter === 'string') {
				_removeHook( 'filters', filter, callback );
			}

			return MethodsAvailable;
		}

		/**
		 * Removes the specified hook by resetting the value of it.
		 *
		 * @param type Type of hook, either 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to remove
		 * @private
		 */
		function _removeHook( type, hook, callback, context ) {
			var handlers, handler, i;
			
			if ( !STORAGE[ type ][ hook ] ) {
				return;
			}
			if ( !callback ) {
				STORAGE[ type ][ hook ] = [];
			} else {
				handlers = STORAGE[ type ][ hook ];
				if ( !context ) {
					for ( i = handlers.length; i--; ) {
						if ( handlers[i].callback === callback ) {
							handlers.splice( i, 1 );
						}
					}
				}
				else {
					for ( i = handlers.length; i--; ) {
						handler = handlers[i];
						if ( handler.callback === callback && handler.context === context) {
							handlers.splice( i, 1 );
						}
					}
				}
			}
		}

		/**
		 * Adds the hook to the appropriate storage container
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook (namespace.identifier) to add to our event manager
		 * @param callback The function that will be called when the hook is executed.
		 * @param priority The priority of this hook. Must be an integer.
		 * @param [context] A value to be used for this
		 * @private
		 */
		function _addHook( type, hook, callback, priority, context ) {
			var hookObject = {
				callback : callback,
				priority : priority,
				context : context
			};

			// Utilize 'prop itself' : http://jsperf.com/hasownproperty-vs-in-vs-undefined/19
			var hooks = STORAGE[ type ][ hook ];
			if( hooks ) {
				hooks.push( hookObject );
				hooks = _hookInsertSort( hooks );
			}
			else {
				hooks = [ hookObject ];
			}

			STORAGE[ type ][ hook ] = hooks;
		}

		/**
		 * Use an insert sort for keeping our hooks organized based on priority. This function is ridiculously faster
		 * than bubble sort, etc: http://jsperf.com/javascript-sort
		 *
		 * @param hooks The custom array containing all of the appropriate hooks to perform an insert sort on.
		 * @private
		 */
		function _hookInsertSort( hooks ) {
			var tmpHook, j, prevHook;
			for( var i = 1, len = hooks.length; i < len; i++ ) {
				tmpHook = hooks[ i ];
				j = i;
				while( ( prevHook = hooks[ j - 1 ] ) &&  prevHook.priority > tmpHook.priority ) {
					hooks[ j ] = hooks[ j - 1 ];
					--j;
				}
				hooks[ j ] = tmpHook;
			}

			return hooks;
		}

		/**
		 * Runs the specified hook. If it is an action, the value is not modified but if it is a filter, it is.
		 *
		 * @param type 'actions' or 'filters'
		 * @param hook The hook ( namespace.identifier ) to be ran.
		 * @param args Arguments to pass to the action/filter. If it's a filter, args is actually a single parameter.
		 * @private
		 */
		function _runHook( type, hook, args ) {
			var handlers = STORAGE[ type ][ hook ], i, len;
			
			if ( !handlers ) {
				return (type === 'filters') ? args[0] : false;
			}

			len = handlers.length;
			if ( type === 'filters' ) {
				for ( i = 0; i < len; i++ ) {
					args[ 0 ] = handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			} else {
				for ( i = 0; i < len; i++ ) {
					handlers[ i ].callback.apply( handlers[ i ].context, args );
				}
			}

			return ( type === 'filters' ) ? args[ 0 ] : true;
		}

		// return all of the publicly available methods
		return MethodsAvailable;

	};
	
	window.wp = window.wp || {};
	window.wp.hooks = new EventManager();

} )( window );

/*
 Tipr 2.0
 Copyright (c) 2015 Tipue
 Tipr is released under the MIT License
 http://www.tipue.com/tipr
 */


(function ( $ ) {

	$.fn.tipr = function ( options ) {

		var set = $.extend( {

			'speed': 200,
			'mode': 'bottom'

		}, options );

		return this.each( function () {

			var tipr_cont = '.tipr_container_' + set.mode;

			$( this ).hover(
				function () {
					var d_m = set.mode;

					if ( $( this ).attr( 'data-mode' ) ) {
						d_m = $( this ).attr( 'data-mode' )
						tipr_cont = '.tipr_container_' + d_m;
					}

					var out = '<div class="tipr_container_' + d_m + '"><div class="tipr_point_' + d_m + '"><div class="tipr_content">' + $( this ).attr( 'data-tip' ) + '</div></div></div>';

					$( this ).append( out );

					var w_t = $( tipr_cont ).outerWidth();
					var w_e = $( this ).width();
					var m_l = (w_e / 2) - (w_t / 2);

					$( tipr_cont ).css( 'margin-left', m_l + 'px' );
					$( this ).removeAttr( 'title alt' );

					if ( d_m == 'bottom' ) {
						$( tipr_cont ).css( 'margin-top', ( $( this ).outerHeight() / 2 ) + 'px' );
					}

					$( tipr_cont ).fadeIn( set.speed );
				},
				function () {
					$( tipr_cont ).remove();
				}
			);

		} );
	};

})( jQuery );