<?php
/**
 * Project Email notifications meta box
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PH_Meta_Box_Project_Emails Class
 *
 * @since 1.0
 */
class PH_Meta_Box_Project_Email_Notifications {

	public static $meta_key = 'ph_project_email_notifications_disable_all';

	/**
     * Store the fields
     *
	 * @var array
	 */
	public static $fields = array();

	/**
	 * Get disabled post ids for the user
	 *
	 * @param int $user_id
	 *
	 * @return array Post ids to suppress for user
	 */
	public static function get_suppressed_ids( $user_id = 0 ) {
		if ( ! $user_id ) {
			$user_id = get_current_user_id();
		}

		return (array) get_user_meta( $user_id, self::$meta_key, true );
	}

	/**
	 * Is the current post notifications disabled?
	 *
	 * @param int $post_id Post ID
	 * @param int $user_id User ID
	 *
	 * @return bool
	 */
	public static function is_disabled( $post_id = 0, $user_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}
		if ( ! $user_id ) {
			$user_id = get_current_user_id();
		}

		return (bool) in_array( $post_id, self::get_suppressed_ids( $user_id ) );
	}

	public static function add_disabled( $post_id = 0, $user_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}
		if ( ! $user_id ) {
			$user_id = get_current_user_id();
		}

		// get notifications
		$disabled = self::get_suppressed_ids();

		// merge
		$unique = array_unique( array_merge( $disabled, array( $post_id ) ) );

		// update
		return update_user_meta( $user_id, self::$meta_key, $unique );
	}

	public static function remove_disabled( $post_id = 0, $user_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}
		if ( ! $user_id ) {
			$user_id = get_current_user_id();
		}

		$disabled = self::get_suppressed_ids( $user_id );
		$remove   = array_search( $post_id, $disabled );

		// remove if exists
		if ( $remove !== false ) {
			unset( $disabled[ $remove ] );

			return update_user_meta( $user_id, self::$meta_key, $disabled );
		}

		return false;
	}

	/**
	 * Output the metabox
	 *
	 * @access public
	 * @since  1.0
	 * @return void
	 */
	public static function output( $post ) {
		// create nonce field
		wp_nonce_field( 'project_huddle_save_data', 'project_huddle_meta_nonce' ); ?>

        <div id="project_emails_notifications_container" class="ph_meta_box">
			<?php
			if ( self::is_disabled( $post->ID ) ) {
				$checked = 'checked="checked"';
			} else {
				$checked = '';
			}
			?>

            <div class="ph-field checkbox <?php echo esc_attr( self::$meta_key ); ?>">
                <input id="<?php echo esc_attr( self::$meta_key ); ?>" type="checkbox" name="<?php echo esc_attr( self::$meta_key ); ?>" <?php echo $checked ?>/>
                <label for="<?php echo esc_attr( self::$meta_key ); ?>"><span class="description"><?php _e( 'Disable all my emails for this project.', 'project-huddle' ); ?></span></label>
            </div>
            <div class="ph-field">
                <br>
                <a href="<?php echo esc_url( admin_url( 'profile.php#ph-emails' ) ); ?>" class="button right">
                    <?php _e( 'Edit Global Email Settings', 'project-huddle'); ?> &rarr;
                </a>
            </div>
        </div>
		<?php
	}

	/**
	 * Save meta box data
	 *
	 * @access public
	 * @since  1.0
	 * @return void
	 */
	public static function save( $post_id, $post ) {
		$value = self::sanitize_field();
		// add
		if ( $value == 'on' ) {
			$suppressed = self::get_suppressed_ids();

			$unique = array_unique( array_merge( $suppressed, array( $post->ID ) ) );

			update_user_meta( get_current_user_id(), self::$meta_key, $unique );
			// remove
		} else {
			$suppressed = self::get_suppressed_ids();
			$remove     = array_search( $post->ID, $suppressed );
			if ( $remove !== false ) {
				unset( $suppressed[ $remove ] );
				update_user_meta( get_current_user_id(), self::$meta_key, $suppressed );
			}
		}
	}

	/**
	 * Sanitize checkbox to string instead of false
	 *
	 * @return bool|string
	 */
	public static function sanitize_field() {
		$value = isset( $_POST[ self::$meta_key ] ) ? $_POST[ self::$meta_key ] : false;
		$value = $value ? esc_html( $value ) : 'off';

		return $value;
	}
}