<?php
/**
 * Register Settings
 *
 * @package     Project Huddle
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class PH_Extensions extends PH_Settings {
	/**
	 * Stores settings within class
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $extensions = array();

	private $cta = array();

	/**
	 * Setup the options page
	 *
	 * Creates settings, menu items and settings link on plugin page
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// Initialise settings
//		add_action( 'admin_init', array( $this, 'init' ) );

		// register available extensions
		add_action( 'admin_init', array( $this, 'register_extensions' ) );

		// Add settings page to menu
		add_action( 'admin_menu', array( $this, 'add_menu_item' ), 20 );
	}

	/**
	 * Initialise settings
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function init() {

	}

	/**
	 * Add settings page to admin menu
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function add_menu_item() {
		$page = add_submenu_page(
			'edit.php?post_type=ph-project',
			__( 'ProjectHuddle Addons', 'project-huddle' ),
			__( 'Addons', 'project-huddle' ),
			'manage_ph_settings',
			'project_huddle_extensions',
			array( $this, 'extensions_page' )
		);
		add_action( 'admin_print_styles-' . $page, array( $this, 'extensions_assets' ) );
	}

	/**
	 * Load settings JS & CSS
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function extensions_assets() {

		// store javascript directory
		$js_dir = PH_PLUGIN_URL . 'assets/js/';

		// store css directory
		$css_dir = PH_PLUGIN_URL . 'assets/css/';

		// Use minified libraries if SCRIPT_DEBUG is turned off
		$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		wp_enqueue_style( 'ph-extensions-admin', $css_dir . 'ph-extensions-admin' . $suffix . '.css', false, PH_VERSION );
	}


	/**
	 * Register plugin settings
	 * @return void
	 */
	public function register_extensions() {
		global $pagenow;

		// only on our extensions page for now
		if ( ! isset( $_GET['page'] ) || 'edit.php' !== $pagenow || 'project_huddle_extensions' !== $_GET['page'] ) {
			return;
		}

		$feed = (array) ph_add_ons_get_feed();

		if ( empty( $feed ) || empty( $feed['extensions'] ) ) {
			return;
		}

		$this->extensions = (array) $feed['extensions'];
		$this->cta        = $feed['cta'];

		if ( is_array( $this->extensions ) ) {
			foreach ( $this->extensions as $section => $data ) {
				// Add section to page
				add_settings_section(
					$section,
					$data->title,
					array( $this, 'settings_section' ),
					'project_huddle_addons_' . $section
				);

				foreach ( $data->fields as $field ) {
					$field    = (array) $field;
					$defaults = array(
						'id'    => false,
						'type'  => false,
						'label' => false,
					);
					$field    = wp_parse_args( $field, $defaults );

					// Add field to page
					add_settings_field(
						$field['id'],
						$field['label'],
						array( $this, 'display_extension' ),
						'project_huddle_addons_' . $section,
						$section,
						array( 'field' => $field )
					);
				}
			}
		}
	}

	public function settings_section( $section ) {
		if ( ! isset( $this->extensions[ $section['id'] ]->description ) ) {
			return false;
		}

		$html = '<p> ' . $this->extensions[ $section['id'] ]->description . '</p>' . "\n";
		echo $html;
	}

	/**
	 * Generate HTML for displaying fields
	 *
	 * @param  array $args Field data
	 *
	 * @return void
	 */
	public function display_extension( $args ) {
		$field = $args['field'];

		$html = '';

		switch ( $field['type'] ) {
			case 'extension':
				ob_start();
				?>
                <div class="plugin-card plugin-card-<?php echo sanitize_text_field( $field['id'] ); ?>">
                    <div class="plugin-card-top">
                        <div class="name column-name" style="margin-right: 0;">
                            <h3>
                                <a href="<?php echo esc_url( $field['url'] ); ?>">
									<?php echo esc_html( $field['title'] ); ?>
									<?php if ( isset( $field['image'] ) ) : ?>
                                        <img src="<?php echo esc_url( $field['image'] ); ?>" class="plugin-icon" alt="">
									<?php endif; ?>
                                </a>
                            </h3>
                        </div>
                        <div class="desc column-description" style="margin-right: 0;">
							<?php echo wp_kses_post( wpautop( $field['description'] ) ); ?>
                            <p class="authors"><cite>By <a href="https://projecthuddle.io">ProjectHuddle</a></cite></p>
                        </div>
                    </div>
                    <div class="plugin-card-bottom">
                        <div class="vers column-rating">
                        </div>
                        <div class="column-compatibility">
							<?php if ( is_plugin_active( $field['plugin_dir'] ) ) { ?>
                                <button type="button" class="button button-disabled" disabled="disabled">
									<?php _e( 'Active', 'project-huddle' ); ?>
                                </button>
							<?php } else if ( $this->plugin_installed( $field['name'] ) ) { ?>
								<?php
								$activate_url = sprintf( admin_url( 'plugins.php?action=activate&plugin=%s&plugin_status=all&paged=1&s' ), $field['plugin_dir'] );
								// change the plugin request to the plugin to pass the nonce check
								$_REQUEST['plugin'] = $field['plugin_dir'];
								$activate_url       = wp_nonce_url( $activate_url, 'activate-plugin_' . $field['plugin_dir'] );
								?>
                                <a href="<?php echo esc_url( $activate_url ); ?>" class="button activate-now">
									<?php _e( 'Activate', 'project-huddle' ); ?>
                                </a>
							<?php } else { ?>
                                <a href="<?php echo esc_url( $field['url'] ); ?>" class="button button-primary" target="_blank">
									<?php _e( 'Get It', 'project-huddle' ); ?>
                                </a>
							<?php } ?>
                        </div>
                    </div>
                </div>
				<?php

				$html = ob_get_clean();

		}

		echo $html;
	}


	/**
	 * Load settings page content
	 * @return void
	 */
	public function extensions_page() {
		$active_tab = isset( $_GET['tab'] ) ? $_GET['tab'] : 1;

		ob_start();
		?>

        <div class="wrap" id="project_huddle_extensions">
            <h1>
				<?php _e( 'Addons for ProjectHuddle', 'project-huddle' ); ?>
            </h1>
            <h2 class="nav-tab-wrapper">
				<?php
				foreach ( $this->extensions as $section => $data ) {
					$tab_url = add_query_arg(
						'tab',
						$section,
						remove_query_arg( 'settings-updated' )
					);

					$active = $active_tab == $section ? ' nav-tab-active' : '';

					echo '<a href="' . esc_url( $tab_url ) . '" title="' . esc_attr( $data->title ) . '" class="nav-tab' . $active . '">';
					echo esc_html( $data->title );
					echo '</a>';
				}
				?>
            </h2>

            <div id="tab_container">
				<?php
				if ( empty( $this->extensions ) ) {
					echo '<p>' . __( 'Coming Soon! Please check back shortly.', 'project-huddle' ) . '</p>';

					return;
				}
				?>
				<?php $this->do_extensions_sections( 'project_huddle_addons_' . $active_tab ); ?>
				<?php
				if ( $this->cta && is_string( $this->cta ) ) {
					echo wp_kses_post( $this->cta );
				}
				?>
            </div>
            <!-- #tab_container-->
        </div><!-- .wrap -->

		<?php
		// output it
		echo ob_get_clean();
	}

	public function do_extensions_sections( $page ) {
		global $wp_settings_sections, $wp_settings_fields;

		if ( ! isset( $wp_settings_sections[ $page ] ) ) {
			return;
		}

		foreach ( (array) $wp_settings_sections[ $page ] as $section ) {
			if ( $section['title'] ) {
				echo "<h2>{$section['title']}</h2>\n";
			}

			if ( $section['callback'] ) {
				call_user_func( $section['callback'], $section );
			}

			if ( ! isset( $wp_settings_fields ) || ! isset( $wp_settings_fields[ $page ] ) || ! isset( $wp_settings_fields[ $page ][ $section['id'] ] ) ) {
				continue;
			}

			$this->do_extensions_fields( $page, $section['id'] );
		}
	}

	public function do_extensions_fields( $page, $section ) {
		global $wp_settings_fields;

		if ( ! isset( $wp_settings_fields[ $page ][ $section ] ) ) {
			return;
		}

		echo '<div>';
		foreach ( (array) $wp_settings_fields[ $page ][ $section ] as $field ) {
			$class = '';

			if ( ! empty( $field['args']['class'] ) ) {
				$class = esc_attr( $field['args']['class'] );
			}

			call_user_func( $field['callback'], $field['args'] );
		}
		echo '<div class="clear"></div></div>';
	}
}

/**
 * Add-ons Get Feed
 *
 * Gets the add-ons page feed.
 *
 * @return string
 */
function ph_add_ons_get_feed() {
	$cache = get_transient( 'projecthuddle_add_ons_feed' );


	if ( false === $cache ) {
		$url = 'https://projecthuddle.io/?addons_feed=2';

		$feed = wp_remote_get( esc_url_raw( $url ), array( 'sslverify' => false ) );

		if ( ! is_wp_error( $feed ) ) {
			if ( isset( $feed['body'] ) && strlen( $feed['body'] ) > 0 ) {
				$cache = json_decode( $feed['body'] );

				// cache hourly
				set_transient( 'projecthuddle_add_ons_feed', $cache, HOUR_IN_SECONDS );
			}
		} else {
			$cache = '<div class="error"><p>' . __( 'There was an error retrieving the add-ons list from the server. Please try again later.', 'project-huddle' ) . '</div>';
		}

	}

	return $cache;
}