<?php
/**
 * Setup Wizard Class
 *
 * Takes new users through some basic steps to setup the plugin
 *
 * @author      Andre Gagnon | ProjectHuddle
 * @category    Admin
 * @version     1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WPSW_Admin_Setup_Wizard class.
 */
if ( ! class_exists( 'WPSW_Admin_Setup_Wizard' ) ) :
	class WPSW_Admin_Setup_Wizard {
		/**
		 * Version number
         *
         * @var string
		 */
	    private $version = '1.0.0';

		/**
		 * Current step
		 *
		 * @var string
		 */
		private $step = '';

		/**
		 * Steps for the setup wizard
		 *
		 * @var array
		 */
		private $steps = array();

		/**
		 * Wizard settings
		 *
		 * @var array
		 */
		protected $config = array();

		/**
		 * Get things going
		 */
		public function __construct( $args ) {
		    // get config
			$this->config = wp_parse_args( $args, $this->defaults() );

			// add notice
			add_action( 'admin_notices', array( $this, 'setup_notice' ) );

			// run wizard
			add_action( 'init', array( $this, 'run_wizard' ) );

			// dismissable notice
			add_action( 'wp_ajax_wpsw_dismissed_notice_handler', array( $this, 'dismiss_notice' ) );
		}

		/**
         * Some default configuration
         *
		 * @return array
		 */
		public function defaults() {
			return array(
				'name'        => 'My Plugin', // Plugin name
				'type'        => 'plugin', // Plugin or theme
				'id'          => 'wpsw',  // Unique ID for hashing notices for multiple instances
				'notice_text' => __( '<strong>Welcome!</strong> &#8211; You&lsquo;re almost ready to start. :)', 'project-huddle' ), // notice text
				'steps'       => array()
			);
        }

		/**
		 * Setup notice
		 */
		public function setup_notice() {
			if ( ! get_option( 'dismissed-' . $this->config['id'] . '-setup', false ) ) { ?>
                <div id="message" class="notice updated wpsw-message is-dismissible wpsw-notice" data-notice="<?php echo esc_attr( $this->config['id'] . '-setup' ); ?>">
                    <p><?php echo wp_kses_post( $this->config['notice_text'] ); ?></p>
                    <p class="submit">
                        <a href="<?php echo esc_url( admin_url( 'admin.php?page=' . $this->config['id'] . '-setup' ) ); ?>" class="button-primary"><?php _e( 'Run the Setup Wizard', 'wp-setup-wizard' ); ?></a>
                        <a class="button-secondary wpsw-dismiss-notice" href="<?php echo esc_url( wp_nonce_url( add_query_arg( 'wc-hide-notice', 'install' ), 'wp-setup-wizard_hide_notices_nonce', '_wc_notice_nonce' ) ); ?>"><?php _e( 'Skip setup', 'wp-setup-wizard' ); ?></a>
                    </p>
                </div>
				<?php
                $this->dismiss_notice_js();
			}
		}

		/**
		 * Dismissible notice javascript
		 */
		protected function dismiss_notice_js() { ?>
            <script>
				jQuery(function ($) {
					$(document).on('click', '.wpsw-notice .notice-dismiss, .wpsw-notice .wpsw-dismiss-notice', function () {
						// Read the "data-notice" information to track which notice
						var slug = $(this).closest('.wpsw-notice').data('notice');
						// Since WP 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php
						$.ajax(ajaxurl,
							{
								type: 'POST',
								data: {
									action     : 'wpsw_dismissed_notice_handler',
									slug: slug
								}
							});
					});
				});
            </script>
        <?php }

		/**
		 * Handle dismiss notice ajax requests
		 */
		public function dismiss_notice() {
			$slug = isset( $_POST['slug'] ) ? $_POST['slug'] : false;
			// Store it in the options table
			if ( $slug ) {
				update_option( "dismissed-$slug", true );
			}
		}

		/**
		 * Run the setup wizard
		 */
		public function run_wizard() {
			// Setup/welcome
			if ( ! empty( $_GET['page'] ) ) {
				switch ( $_GET['page'] ) {
					case $this->config['id'] . '-setup' :
						if ( apply_filters( $this->config['id'] . '_enable_setup_wizard', true ) && current_user_can( apply_filters( $this->config['id'] . '_install_capability', 'manage_options' ) ) ) {
							add_action( 'admin_menu', array( $this, 'admin_menus' ) );
							add_action( 'admin_init', array( $this, 'setup_wizard' ) );
							add_action( 'admin_notices', array( $this, 'install_notice' ) );
						}
						break;
				}
			}
		}

		/**
		 * Add admin menus/screens.
		 */
		public function admin_menus() {
			add_dashboard_page( '', '', 'manage_options', $this->config['id'] . '-setup', '' );
		}

		/**
		 * Show the setup wizard.
		 */
		public function setup_wizard() {
			if ( empty( $_GET['page'] ) || $this->config['id'] . '-setup' !== $_GET['page'] || ! is_admin() ) {
				return;
			}

			// get steps and allow filtering by other plugins
			$this->steps = apply_filters( $this->config['id'] . '_setup_wizard_steps', $this->config['steps'] );

			// get current step
			$this->step  = isset( $_GET['step'] ) ? sanitize_key( $_GET['step'] ) : current( array_keys( $this->steps ) );

			$suffix   = '';//defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
			$redirect = true;

			// color picker
			wp_enqueue_style( 'wp-color-picker' );
			// Image Upload
			wp_enqueue_media();

			// custom styles and scripts
			if ( $this->config['type'] === 'theme' ) {
				wp_enqueue_style( 'wp-setup-wizard', get_theme_file_uri( __FILE__ ) . 'css/wp-setup-wizard' . $suffix . '.css', array(), $this->version );
				wp_enqueue_script( 'wp-setup-wizard', get_theme_file_uri( __FILE__ ) . 'js/wp-setup-wizard' . $suffix . '.js', array(
					'wp-color-picker',
					'jquery'
				), $this->version );
			} else {
				wp_enqueue_style( 'wp-setup-wizard', plugin_dir_url( __FILE__ ) . 'css/wp-setup-wizard' . $suffix . '.css', array(), $this->version );
				wp_enqueue_script( 'wp-setup-wizard', plugin_dir_url( __FILE__ ) . 'js/wp-setup-wizard' . $suffix . '.js', array(
					'jquery',
					'wp-color-picker',
				), $this->version );
			}

			// if we are saving, call save function then redirect
			if ( ! empty( $_POST['save_step'] ) ) {
				// call user function if it exists
				if ( isset( $this->steps[ $this->step ]['handler'] ) ) {
				    // returning false will short circuit redirect
					$redirect = call_user_func( $this->steps[ $this->step ]['handler'], $this );
				}

				// redirect to next step
                if ( $redirect ) {
	                wp_redirect( esc_url_raw( $this->get_next_step_link() ) ); // redirect
	                exit;
                }
			}

			ob_start();
			$this->setup_wizard_header();
			$this->setup_wizard_steps();
			$this->setup_wizard_content();
			$this->setup_wizard_footer();
			exit;
		}

		/**
		 * Get the URL for the next step's screen.
		 *
		 * @param string $step slug (default: current step).
		 *
		 * @return string       URL for next step if a next step exists.
		 *                      Admin URL if it's the last step.
		 *                      Empty string on failure.
		 * @since 3.0.0
		 */
		public function get_next_step_link( $step = '' ) {
			if ( ! $step ) {
				$step = $this->step;
			}

			$keys = array_keys( $this->steps );
			if ( end( $keys ) === $step ) {
				return admin_url();
			}

			$step_index = array_search( $step, $keys );
			if ( false === $step_index ) {
				return '';
			}

			return add_query_arg( 'step', $keys[ $step_index + 1 ], remove_query_arg( 'activate_error' ) );
		}

		/**
		 * Setup Wizard Header.
		 */
		public function setup_wizard_header() {
			?>
            <!DOCTYPE html>
            <html <?php language_attributes(); ?>>
            <head>
                <meta name="viewport" content="width=device-width"/>
                <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
                <title><?php esc_html_e( $this->config['name'] . ' &rsaquo; Setup Wizard', 'project-huddle' ); ?></title>
				<?php wp_print_scripts( 'wp-setup-wizard' ); ?>
				<?php do_action( 'admin_print_styles' ); ?>
				<?php do_action( 'admin_head' ); ?>
            </head>
            <body class="ph-setup wp-core-ui">
			<?php // TODO: swap out logo ?>
            <h2 id="wpsw-logo">
                <a href="<?php echo esc_url( $this->config['site_url'] ); ?>"><img src="<?php echo esc_url( $this->config['logo_url'] ); ?>" alt="<?php echo esc_attr( sprintf( __( '%s Setup', 'wordpress-importer' ), $this->config['name'] ) ); ?>"/></a>
            </h2>
			<?php
		}

		/**
		 * Setup Wizard Footer.
		 */
		public function setup_wizard_footer() {
			$keys = array_keys( $this->steps );
			$step_index = array_search( $this->step, $keys );
			?>
			<?php if ( $step_index === 0 ) : ?>
                <div class="text-center"><a class="btn btn-link ph-return-to-dashboard" href="<?php echo esc_url( admin_url() ); ?>"><?php esc_html_e( 'Not right now', 'project-huddle' ); ?></a></div>
			<?php elseif ( end( $keys ) === $this->step ) :  ?>
                <div class="text-center"><a class="btn btn-link ph-return-to-dashboard" href="<?php echo esc_url( admin_url() ); ?>"><?php esc_html_e( 'Return to your dashboard', 'project-huddle' ); ?></a></div>
			<?php endif; ?>
            <?php
            do_action( 'admin_footer' );
            do_action( 'admin_print_footer_scripts' );
            ?>
            </body>
            </html>
			<?php
		}

		/**
		 * Output the steps.
		 */
		public function setup_wizard_steps() {
			$output_steps = $this->steps;
			?>
            <ul class="step">
				<?php foreach ( $output_steps as $step_key => $step ) : ?>
                    <li class="step-item
					<?php
					if ( $step_key === $this->step ) {
						echo 'active';
					} elseif ( array_search( $this->step, array_keys( $this->steps ) ) > array_search( $step_key, array_keys( $this->steps ) ) ) {
						echo 'done';
					}
					?>
				"><span><?php echo esc_html( $step['name'] ); ?></span></li>
				<?php endforeach; ?>
            </ul>
			<?php
		}

		/**
		 * Output the content for the current step.
		 */
		public function setup_wizard_content() {
			echo '<div class="wpsw-setup-content">';
			    echo '<form method="post" class="' . sanitize_html_class($this->step) . '">';
                    wp_nonce_field( 'ph-setup' );
                    if ( ! empty( $this->steps[ $this->step ]['fields'] ) ) {
                        foreach($this->steps[ $this->step ]['fields'] as $id => $field ) {
                            ph_display_field($field, $id);
                        }
                        echo '<p class="ph-setup-actions text-center">
                            <button type="submit" class="btn btn-primary btn-lg button-next" value="' . esc_attr( $this->steps[ $this->step ]['next_text'] ) . '" name="save_step">' . esc_html( $this->steps[ $this->step ]['next_text'] ) . '</button>
                        </p>';
                    }
			    echo '</form>';
			echo '</div>';
		}
	}
endif;

