<?php
/**
 * Image Data Class
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * PH_Image Class
 *
 * Child class of PH_Project
 *
 * This class handles the image data
 *
 * @since 1.0
 */
class PH_Image {

	/**
	 * Stores post id used in class
	 *
	 * @var int
	 * @access private
	 * @since  1.0.0
	 */
	private $post_id;

	/**
	 * Stores project images array
	 *
	 * @var array
	 * @access private
	 * @since  1.0.0
	 */
	private $project_images = array();

	/**
	 * Register actions
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'untrash_post', array( $this, 'add_to_project_meta' ) );
		add_action( 'before_delete_post', array( $this, 'delete_comment_locations' ) );
	}

	/**
	 * Stores project options array
	 *
	 * @var array
	 * @access pubic
	 * @since  1.0.0
	 */
	public $project_options = array(
		'ph_image_alignment'                 => 'center', // left, right or center
		'ph_image_size'                      => 'normal', // normal or scale
		'ph_image_background_color'          => '#222', // hex value
		'ph_image_background_image'          => '', // url
		'ph_image_background_image_position' => 'center' // repeat, repeat-x, cover
	);

	/**
	 * Gets all images associated with project
	 *
	 * @param int $post_id Post Id
	 *
	 * @return array Multi-Dimensional array of project images
	 *
	 * @access public
	 * @since  1.0.0
	 */
	public function get_saved_project_images( $post_id = 0 ) {
		global $post;

		// get globally if not set
		if ( ! $post_id ) {
			$post_id = $post ? $post->ID : false;
		}

		// needs post id
		if ( ! $post_id ) {
			return false;
		}

		// store post id
		$this->post_id = $post_id;

		// store project image
		$this->project_images = PH()->project->image_ids( $project_id );

		// setup output array
		$project_output = array();

		// prepare each post for output
		if ( isset( $this->project_images ) && is_array( $this->project_images ) ) {
			foreach ( $this->project_images as $key => $image_id ) {
				$image = $this->get_image( $image_id );
				if ( $image ) {
					$project_output[] = $image;
				}
			}
		}

		// return array of project data
		return $project_output;
	}

	/**
	 * Gets all images associated with project
	 *
	 * @param $image_id ID of image to retrieve
	 *
	 * @return boolean Whether the image is a valid post
	 *
	 * @access   private
	 * @since    1.1.0
	 */
	private function validate_image_post( $image_id ) {
		// check for image
		if ( ! isset( $image_id ) || ! get_post( $image_id ) ) {
			return false;
		}

		// check post type and status
		if ( 'project_image' != get_post_type( $image_id ) || get_post_status( $image_id ) == 'trash' ) {
			return false;
		}

		return true;
	}

	/**
	 * Gets all images associated with project
	 *
	 * @param $image_id ID of image to retrieve
	 *
	 * @return array An array of image data
	 *
	 * @access   public
	 * @since    1.0.0
	 */
	public function get_image( $image_id ){
		// check image
		if ( ! $this->validate_image_post($image_id) ) {
			return false;
		}

		// get images
		$thumb = wp_get_attachment_image_src( get_post_thumbnail_id( $image_id ), 'large', false );
		$full  = wp_get_attachment_image_src( get_post_thumbnail_id( $image_id ), 'full', false );

		// add an array of image data for each image
		$image = apply_filters( 'ph_image_data', array(
			'id'            => (int) $image_id,
			'ID'            => (int) $image_id,
			'imageID'       => (int) get_post_thumbnail_id( $image_id ),
			'comment_count' => (int) PH()->comment->get_comment_count( $image_id ),
			'thread_count'  => (int) PH()->comment->get_comment_thread_count( $image_id ),
			'comments'      => PH()->comment->get_comment_items( $image_id ),
			'title'         => htmlspecialchars_decode( sanitize_text_field( get_the_title( $image_id ) ) ),
			'hash'          => esc_attr( get_post( $image_id )->post_name ),
			'date'          => esc_attr( ph_human_time_diff( get_the_time( 'U', $image_id ), current_time( 'timestamp' ) ) ),
			'url'           => esc_url( $full[0] ),
			'thumbnail'     => esc_url( $thumb[0] ),
			'options'       => (array) $this->get_image_options( $image_id )
		), $image_id );

		return $image;
	}

	/**
	 * Get global defaults and set default for options
	 *
	 * @access protected
	 * @since  1.0.0
	 */
	protected function get_global_defaults() {
		if ( $bg_color = get_option( 'ph_image_bg' ) ) {
			$this->project_options['ph_image_background_color'] = esc_attr( $bg_color );
		}
	}

	/**
	 * Gets post meta options and merges with defaults
	 *
	 * @param $image_id (int) ID of image post
	 *
	 * @return array Options for the image
	 *
	 * @access public
	 * @since  1.0.0
	 */
	public function get_image_options( $image_id ) {
		// find any defaults from settings page
		$this->get_global_defaults();

		// filter project options
		$this->project_options = apply_filters( 'ph_save_image_options', $this->project_options );

		// Parse incoming $options into an array and merge it with our defaults
		return wp_parse_args( $this->get_image_option_meta( $image_id ), $this->project_options );
	}

	/**
	 * Gets project options from image post id meta
	 *
	 * @param $image_id int Post id for the image post
	 *
	 * @return array Saved options from meta
	 */
	public function get_image_option_meta( $image_id ) {
		$options = array();
		// get post meta for each default
		foreach ( $this->project_options as $option => $value ) {
			if ( $post_meta = get_post_meta( $image_id, $option, true ) ) {
				$options[ $option ] = esc_attr( $post_meta );
			}
		}
		return $options;
	}

	/**
	 * If image is deleted, delete comment location as well
	 *
	 * @access public
	 * @since  1.0.0
	 *
	 * @param int $post_id Post ID
	 */
	public function delete_comment_locations( $post_id ) {
		// check post type
		if ( get_post_type( $post_id ) != 'project_image' ) {
			return;
		}

		// get the array of comment location ids
		$comment_locations = get_post_meta( $post_id, 'comment_locations', true );

		if ( isset( $comment_locations ) && is_array( $comment_locations ) ) {
			foreach ( $comment_locations as $key => $comment_location ) {
				// delete comment permanently
				wp_delete_post( (int) $comment_location, true );
			}
		}
	}

	public function add_to_project_meta( $post_id ) {
		// check post type
		if ( get_post_type( $post_id ) != 'project_image' ) {
			return;
		}

		// get project it belongs to
		$project_id = get_post_meta( $post_id, 'ph_project_id', true );

		// check project_id
		if ( ! $project_id ) {
			return;
		}

		// get images
		$images = PH()->project->image_ids( $project_id );

		// filter add post ID to project meta
		$new = (int) apply_filters( 'ph_untrash_image_add_project_meta', $post_id, $images, $project_id );

		// add post ID to project meta
		if ( $new ) {
			$images[] = (int) apply_filters( 'ph_add_project_meta', $post_id );
		}

		// update post meta
		update_post_meta( $project_id, 'ph_project_images', $images);
	}

	/**
	 * Get project id of image
	 *
	 * @param $image_id
	 *
	 * @return int
	 */
	public function get_project( $image_id ) {
		return (int) get_post_meta( $image_id, 'ph_project_id', true );
	}
}