<?php
/**
 * Project Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PH_Project Class
 *
 * This class handles the project data
 *
 * @since 1.0
 */
class PH_Project {

	/**
	 * Stores post id used in class
	 *
	 * @var int
	 * @access protected
	 * @since  1.0.0
	 */
	protected $post_id;

	/**
	 * Stores project images array
	 *
	 * @var array
	 * @access protected
	 * @since  1.0.0
	 */
	protected $image_ids = array();

	/**
	 * Stores project images array
	 *
	 * @var array
	 * @access protected
	 * @since  1.0.0
	 */
	protected $images = array();

	/**
	 * Register actions
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'before_delete_post', array( $this, 'delete_images' ) );
	}

	/**
	 * Gets current user info stored in WordPress
	 *
	 * @since 1.0
	 * @return array
	 */
	public function get_current_user_info() {
		$current_user = wp_get_current_user();

		$current_user->capabilities = (object) array();

		if ( is_user_logged_in() ) {
			$current_user->logged_in = true;
			$current_user->id        = $current_user->ID;
		} else {
			$current_user->logged_in = false;
		}

		// add caps
		$data = get_userdata( get_current_user_id() );
		if ( is_object( $data ) ) {
			$current_user->capabilities = $data->allcaps;
		}

		// unset encrypted password
		unset( $current_user->data->user_pass );

		return $current_user->data;
	}

	/**
	 * Gets options used for project
	 *
	 * @since 1.0
	 *
	 * @param int $post_id ID of project
	 *
	 * @return array
	 */
	public function get_project_options( $post_id = 0 ) {
		global $post;
		$post_id = $post_id ? $post_id : $post->ID;

		$options = array(
			// global options
			'global'  => (array) $this->get_global_project_options(),
			// project options
			'project' => array(
				'project_access'      => get_post_meta( $post_id, 'ph_project_access', true ),
				'allow_anon_comments' => get_post_meta( $post_id, 'ph_project_comments', true ),
				'retina_images'       => get_post_meta( $post_id, 'ph_retina', true ),
				'sharing'             => get_post_meta( $post_id, 'ph_project_sharing', true ),
				'zoom'                => get_post_meta( $post_id, 'ph_zoom', true ),
				'tooltip'             => apply_filters( 'ph_comment_tooltip', true, get_post( $post_id ) ),
			),
		);

		return $options;
	}

	public function get_global_project_options() {
		$options = array(
			'avatar_default'            => get_option( 'avatar_default' ),
			'auto_close'                => get_option( 'ph_auto_close', 'on' ) == 'on' ? true : false,
			'image_background_color' => get_option( 'ph_image_bg' ) ? get_option( 'ph_image_bg' ) : false,
			'require_terms'             => get_option( 'ph_require_terms', false ),
			'approval_type'             => get_option( 'ph_approval_type', false )
		);

		return apply_filters( 'ph_global_project_options', $options );
	}

	/**
	 * Gets project translation strings used in js
	 *
	 * @since 1.0
	 * @return array
	 */
	public function get_project_translations() {

		$translations = array(
			// errors
			'generic_error'       => __( 'Hmmm... something went wrong. Please try again.', 'project-huddle' ),
			'no_comment_text'     => __( 'You need to enter some comment text.', 'project-huddle' ),
			'save_comment_first'  => __( 'Please save your comment first.', 'project-huddle' ),
			'no_images'           => __( 'No images found.', 'project-huddle' ),
			'image_not_founds'    => __( 'Image doesn\'t exist or cannot be found.', 'project-huddle' ),
			'hide_comments'       => __( 'Hide Comments', 'project-huddle' ),
			'show_comments'       => __( 'Show Comments', 'project-huddle' ),
			'comment_placeholder' => __( 'Leave A Comment', 'project-huddle' ),
			'no_permission'       => __( 'You don\'t have permission to do that.', 'project-huddle' ),
			'email_sent'          => __( 'Email sent!', 'project-huddle' ),
			'login'               => __( 'Login', 'project-huddle' ),
			'discard'             => __( 'Discard your comment?', 'project-huddle' ),
			'unsaved_changes'     => __( 'It looks like there are unsaved changes on this project.', 'project-huddle' ),
			'tooltip_text'        => __( 'Click To Leave A Comment', 'project-huddle' ),
			'comments_changed'    => __( 'The comments have changed! You may want to double check your comment again before submitting.', 'project-huddle' ),
			'image_slug'          => __( 'image', 'project-huddle' ),
			'confirmTrash'        => __( 'Are you sure you want to trash', 'project-huddle' ),
			'login_required'      => __( 'You need to login in order to post a comment on this image.', 'project-huddle' ),
			'session_expired'     => __( 'Your session expired. Please reload the page.', 'project-huddle' ),
		);

		return $translations;
	}

	/**
	 * Gets the number of comment threads
	 *
	 * @param $post_id int Project ID
	 *
	 * @return int
	 */
	public function get_project_comments_number( $post_id = 0 ) {
		global $post;
		$post_id = $post_id ? $post_id : $post->ID;

		$project_images = (array) PH()->project->images( (int) $post_id );

		$comments = 0;
		foreach ( $project_images as $image ) {
			$comments = $comments + (int) PH()->comment->get_comment_thread_count( $image['ID'] );
		}

		return (int) $comments;
	}

	/**
	 * Gets all stored project data
	 *
	 * @since 1.0
	 *
	 * @param int $post_id
	 *
	 * @return array
	 */
	public function get_project_info( $post_id = 0 ) {
		global $post;

		if ( ! $post_id ) {
			$post_id = $post->ID;
		}

		$project_output = array(
			'permalink'    => get_permalink( $post_id ),
			'site_url'     => get_site_url(),
			'id'           => $post_id,
			'project_id'   => $post_id,
			'name'         => get_the_title( $post_id ),
			'user'         => PH()->user->get_current_user_info(),
			'options'      => $this->get_project_options(),
			'translations' => $this->get_project_translations(),
			'images'       => PH()->project->images(),
		);

		// return array of project data
		return apply_filters( 'ph_project_data', $project_output );
	}

	/**
	 * If image is trashed, trash comment location as well
	 *
	 * @access public
	 * @since  1.0.0
	 *
	 * @param int $post_id Post ID
	 */
	public function delete_images( $post_id ) {
		// check post type
		if ( get_post_type( $post_id ) != 'ph-project' ) {
			return;
		}

		// get the array of comment location ids
		$project_images = get_post_meta( $post_id, 'ph_project_images', true );

		if ( isset( $project_images ) && is_array( $project_images ) ) {
			foreach ( $project_images as $key => $project_image ) {
				// delete comment permanently
				wp_delete_post( (int) $project_image, true );
			}
		}
	}

	/**
	 * Total number of images in the project
	 *
	 * @param $id integer Project ID
	 *
	 * @return int Number of Project Images
	 */
	public function total_images( $id ) {
		// get the array of comment location ids
		$project_images = $this->image_ids( $id );
		return is_array( $project_images ) ? count( $project_images ) : 0;
	}

	/**
	 * Gets all images associated with project
	 *
	 * @param int $post_id Post Id
	 *
	 * @return array|bool Multi-Dimensional array of project images or false
	 *
	 * @access public
	 * @since  1.0.0
	 */
	public function images( $post_id = 0 ) {
		global $post;

		// get globally if not set
		if ( ! $post_id ) {
			$post_id = $post ? $post->ID : false;
		}

		// needs post id
		if ( ! $post_id ) {
			return false;
		}

		// store project image
		$this->image_ids = $this->image_ids( $post_id );

		// setup output array
		$project_output = array();
		// prepare each post for output
		if ( isset( $this->image_ids ) && is_array( $this->image_ids ) ) {
			foreach ( $this->image_ids as $key => $image_id ) {
				$image = PH()->image->get_image( $image_id );
				if ( $image ) {
					$project_output[] = $image;
				}
			}
		}

		// return array of project data
		return $project_output;
	}

	/**
	 * Get a project's image IDs list
	 *
	 * @param int $id
	 *
	 * @return bool|mixed
	 */
	public function image_ids( $id = 0 ) {
		if ( ! $id ) {
			$id = get_the_ID() ? get_the_ID() : false;
		}
		if ( ! $id ) {
			return false;
		}

		$this->image_ids = get_post_meta( $id, 'ph_project_images', true );

		// prepare each post for output
		if ( $this->image_ids && is_array( $this->image_ids ) ) {
			foreach ( $this->image_ids as $key => $image_id ) {
				if ( get_post_status( $image_id ) != 'publish' ) {
					unset( $this->image_ids[ $key ] );
				}
			}
		}

		return $this->image_ids;
	}
}
