<?php
/**
 * Filters and actions to improve 3rd party plugin compatibility
 */

/**
 * Don't add redirect when we create our custom url slug for mockups
 *
 * @return bool True doesn't change slug
 */
function ph_wpseo_premium_post_redirect_slug_change() {
	global $post;

	if ( get_post_type( $post ) == 'ph-project' ) {
		return true;
	}

	return false;
}

add_filter( 'wpseo_premium_post_redirect_slug_change', 'ph_wpseo_premium_post_redirect_slug_change' );

/**
 * Allow users that have "Custom Role" to  view the Dashboard
 *
 * @param $access
 *
 * @return bool
 */
function ph_show_admin( $access ) {
	if ( current_user_can( 'project_editor' ) || current_user_can( 'project_collaborator' ) || current_user_can( 'project_admin' ) ) {
		$access = false;
	}

	return $access;
}

add_filter( 'woocommerce_prevent_admin_access', 'ph_show_admin' );
add_filter( 'woocommerce_disable_admin_bar', 'ph_show_admin' );

/**
 * After login or registration, redirect users with ProjectHuddle
 * Roles to the Dashboard instead of WooCommerce's My Account
 *
 * @param $redirect
 *
 * @return string
 */
function ph_admin_redirect( $redirect ) {
	if ( current_user_can( 'project_editor' ) || current_user_can( 'project_collaborator' ) || current_user_can( 'project_admin' ) ) {
		$user     = get_current_user_id();
		$redirect = get_dashboard_url( $user );
	}

	return $redirect;
}

add_filter( 'woocommerce_login_redirect', 'ph_admin_redirect' );
add_filter( 'woocommerce_registration_redirect', 'ph_admin_redirect' );

/**
 * Check that permalinks are enabled
 */
function ph_check_api_permalinks() {
	if ( ! get_option( 'permalink_structure' ) ) {
		$class   = 'notice notice-error';
		$message = __( 'ProjectHuddle requires permalinks to be enabled in order to access the WordPress API. Please enable permalinks under Settings > Permalinks.', 'project-huddle' );

		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
	}
}

add_action( 'admin_notices', 'ph_check_api_permalinks' );

/**
 * Really simple ssl compatibility
 */
function ph_really_simple_ssl() {
	$options = get_option( 'rlrsssl_options', false );

	if ( $options && is_array( $options ) ) {
		if ( isset( $options['javascript_redirect'] ) && $options['javascript_redirect'] ) {
			$class   = 'notice notice-error';
			$message = __( 'You must turn off Really Simple SSL\'s javascript redirection feature or ProjectHuddle won\'t work on non-ssl sites.', 'project-huddle' );
			$link    = '<a href="' . esc_url( admin_url( 'options-general.php?page=rlrsssl_really_simple_ssl&tab=settings' ) ) . '">' . __( 'Change It', 'project-huddle' ) . '</a>';
			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) . ' ' . $link );
		}
	}
}

add_action( 'admin_notices', 'ph_really_simple_ssl' );

/**
 * Remove SelectWoo from our pages
 */
function ph_remove_select_woo() {
	global $post;

	if ( get_post_type( $post ) === 'ph-website' || get_post_type( $post ) === 'ph-project' ) {
		wp_deregister_script( 'selectWoo' );
		wp_dequeue_script( 'selectWoo' );
	}
}

add_action( 'admin_enqueue_scripts', 'ph_remove_select_woo', 11 );

/**
 * Fix Astra orphan ob_start()
 */
function ph_custom_fix_astra_ob_start() {
	global $post;

	if ( get_post_type( $post ) === 'ph-website' ) {
		remove_filter( 'astra_dynamic_css', 'astra_typography_dynamic_css' );
	}
}

add_action( 'wp_enqueue_scripts', 'ph_custom_fix_astra_ob_start' );

/**
 * Make sure disable comments plugin isn't active
 */
function ph_check_disable_comments() {
	if ( is_plugin_active( 'disable-comments/disable-comments.php' ) || is_plugin_active( 'disable-comments-rb/disable-comments-rb.php' ) ) {
		$class   = 'notice notice-error';
		$message = __( 'Notice! You must turn off your Disable Comments plugin or ProjectHuddle won\'t be able to save any comments!', 'project-huddle' );
		$link    = '<a href="' . esc_url( admin_url( 'plugins.php' ) ) . '">' . __( 'Plugins Page', 'project-huddle' ) . '</a>';
		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) . ' ' . $link );
	}
}

add_action( 'admin_notices', 'ph_check_disable_comments' );

/**
 * Make sure old trello plugin isn't active
 */
function ph_check_trello_plugin() {
	$plugin_dir = 'projecthuddle-trello-integration/projecthuddle-trello-integration.php';

	if ( is_plugin_active( $plugin_dir ) ) {
		$class   = 'notice notice-error';
		$message = __( 'Notice! ProjectHuddle 3.0 and above is no longer compatible with the WordPress.org Trello extension. It has been removed from the repository. Please deactivate it or you may experience issues with ProjectHuddle', 'project-huddle' );

		$deactivate_url = sprintf( admin_url( 'plugins.php?action=deactivate&plugin=%s&plugin_status=all&paged=1&s' ), $plugin_dir  );
		// change the plugin request to the plugin to pass the nonce check
		$_REQUEST['plugin'] = $plugin_dir ;
		$deactivate_url = wp_nonce_url($deactivate_url, 'deactivate-plugin_' . $plugin_dir );

		$link    = '<a href="' . esc_url( $deactivate_url ) . '">' . __( 'Deactivate Plugin', 'project-huddle' ) . '</a>';
		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) . ' ' . $link );
	}
}

add_action( 'admin_notices', 'ph_check_trello_plugin' );

/**
 * Fix missing comments with WPML
 */
function ph_fix_wpml_missing_comments( $filtered, $post_id ) {
	// if the post is one of our post types, don't filter
	if ( in_array( get_post_type( $post_id ), ph_get_child_post_types() ) ) {
		$filtered = false;
	}

	return $filtered;
}

add_filter( 'wpml_is_comment_query_filtered', 'ph_fix_wpml_missing_comments', 10, 2 );