<?php
/**
 * Emails Controller
 *
 * ProjectHuddle Emails Class which handles the sending of email templates. This class loads in available emails.
 *
 * @class 		PH_Emails
 * @version		1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Email Class
 *
 * @author Andre Gagnon
 * @version 1.0.0
 */
class PH_Email {

	/**
	 * @var array Array of email notification classes.
	 *
	 * @access private
	 * @since  1.0.0
	 */
	public $emails;

	/**
	 * @var string Stores the emailer's address.
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $from_address;

	/**
	 * @var string Stores the emailer's name.
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $from_name;

	/**
	 * @var mixed Content type for sent emails
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $content_type;

	/**
	 * Holds the email headers
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $headers;

	/**
	 * Whether to send email in HTML
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $html = true;

	/**
	 * The header text for the email
	 *
	 * @access private
	 * @since  1.0.0
	 */
	private $heading = '';

	/**
	 * Get things going
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'ph_email_send_before', array( $this, 'send_before' ) );
		add_action( 'ph_email_send_after', array( $this, 'send_after' ) );
	}

	/**
	 * Get the site name to use for the email
	 *
	 * @return string Email from name
	 *
	 * @since 1.0.0
	 */
	public function get_from_name() {
		if ( ! $this->from_name ) {
			$this->from_name = get_bloginfo( 'name' );
		}

		return apply_filters( 'ph_email_from_name', wp_specialchars_decode( $this->from_name ), $this );
	}

	/**
	 * Get the email from address from admin
	 *
	 * @return string Email From Address
	 *
	 * @since 1.0.0
	 */
	public function get_from_address() {
		if ( ! $this->from_address ) {
			$this->from_address = get_option( 'admin_email' );
		}

		return apply_filters( 'ph_email_from_address', esc_attr( $this->from_address ), $this );
	}

	/**
	 * Get the email content type
	 *
	 * @return string Email content type
	 *
	 * @since 1.0.0
	 */
	public function get_content_type() {
		if ( ! $this->content_type && $this->html ) {
			$this->content_type = apply_filters( 'ph_email_default_content_type', 'text/html', $this );
		} else if ( ! $this->html ) {
			$this->content_type = 'text/plain';
		}

		return apply_filters( 'ph_email_content_type', $this->content_type, $this );
	}

	/**
	 * Get the email headers
	 *
	 * @return string Email headers
	 *
	 * @since 1.0.0
	 */
	public function get_headers() {
		if ( ! $this->headers ) {
			$this->headers  = "From: {$this->get_from_name()} <{$this->get_from_address()}>\r\n";
			$this->headers .= "Reply-To: {$this->get_from_address()}\r\n";
			$this->headers .= "Content-Type: {$this->get_content_type()}; charset=utf-8\r\n";
		}

		return apply_filters( 'ph_email_headers', $this->headers, $this );
	}

	/**
	 * Send the email
	 *
	 * @param  string  $to          The To address.
	 * @param  string  $subject     The subject line of the email.
	 * @param  string  $message     The body of the email.
	 *
	 * @return boolean $sent        Success/fail of sending
	 *
	 * @since 1.0.0
	 */
	public function send( $to, $subject, $message ) {
		do_action( 'ph_email_send_before', $this );

		$message = $this->text_to_html( $message );

		$sent = wp_mail( $to, $subject, $message, $this->get_headers() );

		// validate
		if ( ! $sent ) {
			error_log( __METHOD__ . ': Something went wrong when trying to send the email. Double-check your server mail configuration or try using SMTP to send your email.' );
		}

		do_action( 'ph_email_send_after', $this );

		return $sent;
	}

	/**
	 * Add filters / actions before the email is sent
	 *
	 * @since 1.0.0
	 */
	public function send_before() {
		add_filter( 'wp_mail_from', array( $this, 'get_from_address' ) );
		add_filter( 'wp_mail_from_name', array( $this, 'get_from_name' ) );
		add_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );
	}

	/**
	 * Remove filters / actions after the email is sent
	 *
	 * @since 1.0.0
	 */
	public function send_after() {
		remove_filter( 'wp_mail_from', array( $this, 'get_from_address' ) );
		remove_filter( 'wp_mail_from_name', array( $this, 'get_from_name' ) );
		remove_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );

		// Reset heading to an empty string
		$this->heading = '';
	}

	/**
	 * Set a property
	 *
	 * @since 1.0.0
	 */
	public function __set( $key, $value ) {
		$this->$key = $value;
	}

	/**
	 * Converts text to formatted HTML. This is primarily for turning line breaks into <p> and <br/> tags.
	 *
	 * @since 2.1
	 */
	public function text_to_html( $message ) {
		if ( 'text/html' == $this->content_type || true === $this->html ) {
			$message = wpautop( $message );
		}

		return $message;
	}
}