<?php
/**
 * Utility functions for emails
 *
 * @since 2.8.0
 * @package ProjectHuddle
 */

/**
 * Turns email link into formatted button
 *
 * @param string $link Link url.
 * @param string $text Link text.
 * @param array  $attrs Inline attributes for the link.
 *
 * @return mixed|void
 */
function ph_email_link( $link, $text, $attrs = array() ) {
	$defaults = array(
		'style' => 'text-align:inherit; margin-top: 35px; margin-bottom: 0;',
	);

	$attrs = wp_parse_args( $attrs, $defaults );

	// force block.
	if ( $attrs['style'] ) {
		$attrs['style'] = $attrs['style'] . 'display:block;';
	}

	$attrs_string = '';

	foreach ( $attrs as $field => $attr ) {
		$attr         = esc_html( $attr );
		$attrs_string = "$field=\"$attr\" ";
	}

	return apply_filters(
		'ph_email_link', '<span ' . $attrs_string . '>
		<a bgcolor="' . get_option( 'ph_highlight_color', '#4353ff' ) . '" height="40" href="' . esc_url( $link ) . '" style="display:inline-block;margin:0 auto;background-color:' . get_option( 'ph_highlight_color', '#4353ff' ) . ';color:#fff;min-width:200px;padding:0 25px;text-align:center;text-decoration:none;height:40px;line-height:40px;width:auto;border-radius:3px;font-size:14px; font-family:Helvetica Neue,Helvetica,Arial,sans-serif;" width="auto" target="_blank">
		' . esc_html( $text ) . '</a></span>', $link, $text, $attrs
	);
}

/**
 * Gets all email addresses for users on the project
 *
 * @since 1.0.0
 *
 * @param int   $project_id ID of the project.
 * @param mixed $user       If a user array is set, check against list so they don't get their own notification.
 *
 * @return array List of emails
 */
function ph_get_project_emails( $project_id, $user = false ) {
	// Project user ids.
	$ids = get_post_meta( $project_id, 'project_members', true );

	if ( ! $user ) {
		$user = wp_get_current_user();
	}

	// Backwards compatibility.
	if ( empty( $ids ) ) {
		// get backwards compat project member ids.
		$ids = ph_backwards_compat_get_project_member_ids( $project_id );
	}

	// Variable to store our emails.
	$emails = array();

	// If we have user ids.
	if ( ! empty( $ids ) ) {
		// User shouldn't get their own emails.
		$key = array_search( $user->user_id, $ids );
		if ( false !== $key ) {
			unset( $emails[ $key ] );
		}

		// Loop through and get emails.
		foreach ( $ids as $key => $id ) {
			// don't get your own email
			if( $id === get_current_user_id() ) {
				continue;
			}

			$member = get_user_by( 'id', $id );

			if ( $member ) {
				$emails[] = sanitize_email( $member->user_email );
			}

			// if user has disabled email notifications.
			if ( $user && isset( $user->ph_project_email_notifications_disable_all ) ) {
				$disabled = (array) $user->ph_project_email_notifications_disable_all; // Meta key via magic method.

				if ( in_array( $project_id, $disabled ) ) {
					unset( $emails[ $key ] );
				}
			}
		}
	}

	// get admin notice option.
	$admin_notice = get_option( 'ph_admin_emails' );

	// email admin if applicable.
	if ( apply_filters( 'ph_admin_notification_emails', $admin_notice ) == 'on' ) {
		$emails[] = get_option( 'admin_email' );
	}

	return array_filter( array_unique( apply_filters( 'ph_project_emails', $emails, $project_id, $user ) ) );
}

/**
 * Get project members
 *
 * @param int $project_id Project ID.
 * @return array Array of user objects
 */
function ph_get_project_members( $project_id = 0 ) {
	if ( ! $project_id ) {
		global $post;
		if ( $post ) {
			$project_id = $post->ID;
		} else {
			return false;
		}
	}

	// Project user ids.
	$ids = get_post_meta( $project_id, 'project_members', true );

	// Backwards compatibility.
	if ( empty( $ids ) ) {
		// get backwards compat project members.
		$members = ph_backwards_compat_get_project_members( $project_id );
	} else {
		// Loop through and get emails.
		foreach ( $ids as $key => $id ) {
			$member = get_user_by( 'id', $id );

			if ( $member ) {
				$members[] = $member;
			}
		}
	}

	// include post author.
	if ( apply_filters( 'ph_include_author_as_project_member', true ) ) {
		$author = get_post_field( 'post_author', $project_id );

		if ( $author && is_a( $author, 'WP_User' ) ) {
			$members = $author;
		}
	}

	return $members;
}

/**
 * Get project members
 *
 * @param int $project_id Project ID.
 * @return array Array of user objects
 */
function ph_get_project_member_ids( $project_id = 0 ) {
	if ( ! $project_id ) {
		global $post;
		if ( $post ) {
			$project_id = $post->ID;
		} else {
			return false;
		}
	}

	// Project user ids.
	$ids = get_post_meta( $project_id, 'project_members', true );

	// Backwards compatibility.
	if ( empty( $ids ) ) {
		// get backwards compat project member ids.
		$ids = ph_backwards_compat_get_project_member_ids( $project_id );
	}

	if ( apply_filters( 'ph_include_author_as_project_member', true ) ) {
		$author = get_post_field( 'post_author', $project_id );

		if ( $author && is_a( $author, 'WP_User' ) ) {
			$ids[] = $author->ID;
		}
	}
	return array_unique( array_filter( $ids ) );
}

/**
 * Get project members by old storage method
 *
 * @param integer $project_id Project post id.
 * @return array Array of ids.
 */
function ph_backwards_compat_get_project_members( $project_id ) {
	if ( ! $project_id ) {
		global $post;
		if ( $post ) {
			$project_id = $post->ID;
		} else {
			return false;
		}
	}

	$emails  = (array) get_post_meta( (int) $project_id, 'ph_project_emails_enable', true );
	$emails  = array_unique( $emails );
	$members = array();

	// Store user ids in ids array.
	foreach ( $emails as $key => $email ) {
		if ( ! $email ) {
			continue;
		}

		// Get user by email.
		$user = get_user_by( 'email', $email );

		// users only.
		if ( $user && is_a( $user, 'WP_User' ) ) {
			// store user in array.
			$members[] = $user;
		}
	}

	// update in new post meta.
	if ( ! empty( $members ) ) {
		$ids = array();
		foreach ( $members as $member ) {
			if ( is_a( $member, 'WP_User' ) ) {
				$ids[] = $member->ID;
			}
		}

		$user_ids = (array) get_post_meta( $project_id, 'project_members', true );
		$ids = array_unique( array_merge( $user_ids, $ids ) );
		update_post_meta( $project_id, 'project_members', $ids );
	}

	return $members;
}

/**
 * Get project member ids backwards compat
 *
 * @param integer $project_id Project post id.
 * @return array
 */
function ph_backwards_compat_get_project_member_ids( $project_id ) {
	$members = ph_backwards_compat_get_project_members( $project_id );

	$ids = array();

	// update in new post meta.
	if ( ! empty( $members ) ) {
		foreach ( $members as $member ) {
			if ( is_a( $member, 'WP_User' ) ) {
				$ids[] = $member->ID;
			}
		}

		$user_ids = (array) get_post_meta( $project_id, 'project_members', true );
		$ids = array_unique( array_merge( $user_ids, $ids ) );
		update_post_meta( $project_id, 'project_members', $ids );
	}

	return $ids;
}

/**
 * Add an email to a project programmatically
 *
 * @param integer $project_id ID for the project.
 * @param WP_User $user     User to add.
 */
function ph_add_project_member( $project_id, $user ) {
	$ids   = (array) get_post_meta( $project_id, 'project_members', true );

	// if user isn't member yet, trigger subscribe email
	if ( ! in_array ( $user->ID, $ids ) ) {
		do_action( 'ph_new_email_admin_subscribe', array( $user->user_email ), wp_get_current_user(), $project_id );
	}

	$ids[] = $user->ID;
	$ids   = array_unique( $ids );
	update_post_meta( $project_id, 'project_members', $ids );
}
