<?php
/**
 * Email Functions
 *
 * Functions that send specific emails
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Email image comment notification to everyone on the project
 *
 * @since    1.0.0
 */
function ph_email_image_comment( $id, $comment ) {
	// get ids of parent posts
	$parents = ph_get_parents_ids( $id, 'comment' );

	// must have a project
	if ( ! isset( $parents['mockup'] ) || ! isset( $parents['mockup-image'] ) || ! isset( $parents['mockup-thread'] ) ) {
		return;
	}

	// set project and image
	$project    = $parents['mockup'];
	$page_image = isset( $parents['mockup-image'] ) ? $parents['mockup-image'] : 0;

	$current_user = wp_get_current_user();

	// to emails
	$to_emails = (array) apply_filters( 'ph_mockup_new_comment_email_to', ph_get_project_emails( $project, get_user_by( 'email', $comment->comment_author_email ) ), $id, $comment );

	// send each email individually
	foreach ( $to_emails as $email ) {
		// send email
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/new-comment-email.php' ), [
					'commenter'    => sanitize_text_field( $comment->comment_author ),
					'avatar'       => get_avatar( get_current_user_id(), 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px"'
					) ),
					'project_name' => sanitize_text_field( ph_get_the_title( (int) $project ) ),
					'item_name'   => sanitize_text_field( ph_get_the_title( (int) $page_image ) ),
					'content'      => wpautop( $comment->comment_content ),
					'link'         => ph_email_link( get_permalink( $parents['mockup-thread'] ), __( 'View Comment', 'project-huddle' ) )
				] )
				->to( $email )
				->subject( apply_filters( 'ph_mockup_new_comment_email_subject', sprintf( __( '%1s made a new comment on %2s.', 'project-huddle' ), '{{commenter}}', '{{project_name}}' ), $id, $comment ) )
				->send( $id );
		} catch ( Exception $e ) {
			// log error.
			error_log( $e );
		}
	}
}

add_action( 'ph_mockup_publish_comment', 'ph_email_image_comment', 10, 2 );

/**
 * Email share comment message with link
 *
 * @since    1.0.0
 *
 * @param array   $address Email addresses
 * @param string  $subject Subject link
 * @param string  $message Message body
 * @param integer $post_id Post ID to share
 */
function ph_email_share_post( $address, $subject = '', $message = '', $post_id ) {
	global $access_token_override; $access_token_override = true;

	// must be one of our post types
	if ( ! in_array( get_post_type( $post_id ), ph_get_child_post_types() ) && ! in_array( get_post_type( $post_id ), ph_get_post_types() ) ) {
		return;
	}

	$user = wp_get_current_user();

	// email each new person
	foreach ( $address as $email ) {
		// send the email
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/generic-email.php' ), [
					'commenter' => sanitize_text_field( $user->display_name ),
					'avatar'    => get_avatar( $user->user_email, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px;"'
					) ),
					'content'   => wpautop( $message ),
					'link'      => ph_email_link( get_permalink( $post_id ), __( 'View', 'project-huddle' ) )
				] )
				->to( apply_filters( 'ph_share_post_email_to', $email, $post_id ) )
				->subject( apply_filters( 'ph_share_post_email_subject', $subject, $post_id ) )
				->send( $post_id );
		} catch ( Exception $e ) {
			// log error
			error_log( $e );
		}

		$access_token_override = false;
	}
}

add_action( 'ph_email_share_post', 'ph_email_share_post', 10, 4 );

/**
 * Email user they've been manually added to a project
 *
 * @since    1.3.1
 *
 * @param $new_emails array New project emails
 * @param $user       object Current user object
 * @param $post_id    int ID of project
 */
function ph_subscribe_user_admin_email( $new_emails = array(), $user = null, $post_id = 0 ) {
	if ( get_post_type( $post_id ) != 'ph-project' ) {
		return;
	}

	$data = apply_filters( 'ph_subscribe_user_admin_email_data', array(
		'user'    => $user,
		'post_id' => $post_id
	) );

	// make sure we have emails
	if ( empty( $new_emails ) || ! $post_id || ! $user ) {
		return;
	}

	// project title and link
	$project_title = get_the_title( $post_id );
	$project_link  = get_the_permalink( $post_id );

	// email each new person
	foreach ( $new_emails as $email ) {
		// don't get your own
		if ( $email == $user->user_email ) {
			return;
		}

		// send the email
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/subscribe-project-email.php' ), [
					'commenter'    => sanitize_text_field( $user->display_name ),
					'avatar'       => get_avatar( $user->user_email, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px;"'
					) ),
					'email'        => sanitize_email( $user->user_email ),
					'project_name' => sanitize_text_field( $project_title ),
					'link'         => ph_email_link( esc_url( $project_link ), __( 'View Project', 'project-huddle' ) )
				] )
				->to( apply_filters( 'ph_mockup_subscribe_project_email_to', $email ) )
				->subject( apply_filters( 'ph_mockup_subscribe_project_email_subject', sprintf( __( 'You have been invited to collaborate on %1s.', 'project-huddle' ), '{{project_name}}' ), $user, $post_id ) )
				->send( $post_id );
		} catch ( Exception $e ) {
			// log error
			error_log( $e );
		}
	}
}

add_action( 'ph_new_email_admin_subscribe', 'ph_subscribe_user_admin_email', 10, 3 );

/**
 * Email new user their login information
 *
 * @since  2.7.0
 *
 * @param WP_User         $user    User who sent the image
 * @param WP_REST_Request $request Project Model
 */
function ph_email_user_password_reset( $user, $request ) {
	//  get user login
	if ( ! is_a( $user, 'WP_User' ) && is_string( $user ) ) {
		if ( filter_var( $user, FILTER_VALIDATE_EMAIL ) ) {
			$user = get_user_by( 'email', $user );
		} else {
			$user = get_user_by( 'login', $user );
		}

	}
	// to emails
	$to_emails = (array) apply_filters( 'ph_email_user_password_reset_to', array( $user->user_email ) );

	// password reset key
	$key        = get_password_reset_key( $user );
	$reset_link = network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $user->user_login ), 'login' );

	// send each email individually
	foreach ( $to_emails as $email ) {
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/registration-email.php' ), [
					'link'         => ph_email_link( $reset_link, __( 'Set Your Password', 'project-huddle' ) ),
					'username'     => sanitize_text_field( $user->user_login ),
					'project_name' => sanitize_text_field( get_the_title( (int) $request['project_id'] ) ),
					'project_link' => esc_url( get_permalink( (int) $request['project_id'] ) ),
				] )
				->to( $email )
				->subject( apply_filters( 'ph_set_password_email_subject', __( 'Please set your password.', 'project-huddle' ), $user, $request ) )
				->send( (int) $request['project_id'] );
		} catch ( Exception $e ) {
			// log error
			error_log( $e );
		}
	}
}

add_action( 'ph_rest_insert_user', 'ph_email_user_password_reset', 10, 2 );
add_action( 'ph_sent_user_password_reset', 'ph_email_user_password_reset', 10, 2 );

/**
 * Email project approval notification to everyone on the project
 *
 * @param $data
 *
 * @since    1.1.1
 * @throws Exception
 */
function ph_email_image_approved( $post, $value ) {
	global $is_ph_batch;

	$status = $value ? ph_approval_completed_status() : ph_get_default_approval_status();

	if ( 'project_image' !== get_post_type( $post ) ) {
		return;
	}

	// get parents
	$parents = ph_get_mockup_parent_ids( $post );

	// need mockup
	if ( ! $parents['mockup'] ) {
		return;
	}

	// make sure we're not running other batches
	static $batch_running;
	if ( $batch_running ) {
		return;
	}

	// if we're doing batch, short circuit and send project email
	if ( $is_ph_batch ) {
		$batch_running = true;
		ph_email_project_approved( $status, $parents['mockup'] );
		return;
	}

	$current_user = wp_get_current_user();
	$to_emails    = (array) apply_filters( 'ph_mockup_approved_image_to', ph_get_project_emails( $parents['mockup'] ), $post );

	$approval_term = get_term_by( 'slug', $status, ph_approval_term_taxonomy() );
	$approval_name = $approval_term->name;

	// send each email individually
	foreach ( $to_emails as $email ) {
		// send.
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/item-approval-email.php' ), [
					'commenter'       => sanitize_text_field( $current_user->display_name ),
					'avatar'          => get_avatar( $current_user->ID, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px"'
					) ),
					'approval_status' => $approval_name,
					'item_name'      => sanitize_text_field( html_entity_decode( ph_get_the_title( $post ) ) ),
					'project_name'    => sanitize_text_field( html_entity_decode( ph_get_the_title( $parents['mockup'] ) ) ),
					'link'            => ph_email_link( esc_url( get_permalink( $post ) ), __( 'View Image', 'project-huddle' ) )
				] )
				->subject( apply_filters( 'ph_mockup_approved_image_subject', sprintf( __( '%1s marked an image in %2s as %3s', 'project-huddle' ), '{{commenter}}', '{{project_name}}', '{{approval_status}}' ), $post, $status ) )
				->to( $email )
				->send( $post->ID );
		} catch ( Exception $e ) {
			// log error
			error_log( $e );
		}
	}
}

add_action( 'ph_rest_set_approval', 'ph_email_image_approved', 10, 2 );

/**
 * Email project approval notification to everyone on the project
 *
 * @param array $data model data
 *
 * @since    2.2.0
 * @throws Exception
 */
function ph_email_project_approved( $status, $post_id ) {
	// get all emails
	$current_user = wp_get_current_user();

	$to_emails = (array) apply_filters( 'ph_mockup_approved_project_to', ph_get_project_emails( $post_id ), $post_id );

	// get approval status name
	$approval_term = get_term_by( 'slug', $status, ph_approval_term_taxonomy() );
	$approval_name = $approval_term->name;

	// send each email individually
	foreach ( $to_emails as $email ) {
		// send.
		try {
			( new PH_Mail )
				->template( ph_locate_template( 'email/project-approval-email.php' ), [
					'commenter'       => sanitize_text_field( $current_user->display_name ),
					'avatar'          => get_avatar( $current_user->ID, 42, '', '', array(
						'force_display' => true,
						'extra_attr'    => 'style="border-radius: 42px"'
					) ),
					'approval_status' => $approval_name,
					'project_name'    => sanitize_text_field( ph_get_the_title( $post_id ) ),
					'link'            => ph_email_link( esc_url( get_permalink( $post_id ) ), __( 'View Project', 'project-huddle' ) )
				] )
				->subject( apply_filters( 'ph_mockup_approved_project_subject', sprintf( __( '%1s marked the project %2s as %3s', 'project-huddle' ), '{{commenter}}', '{{project_name}}', '{{approval_status}}' ), $post_id ) )
				->to( $email )
				->send( $post_id );
		} catch ( Exception $e ) {
			// log error
			error_log( $e );
		}
	}
}