<?php
/**
 * Mockup Image
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Mockup Image Model Class
 *
 * This class handles the project data
 *
 * @since 1.0
 */
class PH_Mockup_Image extends PH_Rest_Object {
	/**
	 * Post type
	 *
	 * @var string
	 */
	public $post_type = 'project_image';

	/**
	 * Parent collection post type
	 *
	 * @var string
	 */
	public $parent_post_type = 'ph-project';

	/**
	 * Collection name
	 *
	 * @var string
	 */
	public $collection_name = 'images';

	/**
	 * Slug for actions
	 *
	 * @var string
	 */
	public $action_slug = 'image';

	/**
	 * For website commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'mockup';

	/**
	 * Stores image options array
	 *
	 * @var array
	 * @access pubic
	 * @since  1.0.0
	 */
	public $options = array(
		'alignment'                 => 'center', // left, right or center.
		'size'                      => 'normal', // normal or scale.
		'background_color'          => '#222', // hex value.
		'background_image'          => '', // url.
		'background_image_position' => 'center', // repeat, repeat-x, cover.
	);

	/**
	 * Actions and filters
	 */
	public function __construct() {
		// add schema.
		$this->schema = $this->schema();

		// set defaults.
		$this->options = apply_filters( 'ph_image_options_defaults', $this->options );

		// add fields.
		$this->rest_fields_from_schema();

		// standardize actions.
		$this->add_actions();

		// add filters.
		$this->add_filters();

		// register fields.
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );
		add_action( 'rest_api_init', array( $this, 'version_field' ) );
		add_action( 'rest_api_init', array( $this, 'options_field' ) );
		add_action( 'rest_api_init', array( $this, 'approval_field' ) );
		add_action( 'rest_api_init', array( $this, 'resolved_field' ) );

		// store approval data when approval is set.
		add_action( "ph_website_rest_update_{$this->action_slug}_attribute", array( $this, 'store_approval_comment' ), 10, 3 );

		// dynamic comment text for approval comments to make sure user display name and translations are up to date.
		add_filter( 'comment_text', array( $this, 'approval_comment_text' ), 8, 2 );

		add_filter( "ph_{$this->collection_name}_collection_data", array( $this, 'sort_by_menu_order' ) );
	}

	/**
	 * Schema for post meta
	 *
	 * @return array
	 */
	public function schema() {
		return apply_filters(
			'ph_mockup_image_meta', array(
				'parent_id' => array(
					'description' => esc_html__( 'ID of parent website.', 'project-huddle' ),
					'type'        => 'integer',
					'default'     => 0,
					'required'    => true,
				),
			)
		);
	}

	/**
	 * Approval field
	 *
	 * @return void
	 */
	public function approval_field() {
		register_rest_field(
			$this->post_type, 'approval', array(
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					return ph_post_is_approved( $post['id'] );
				},
				'update_callback' => function ( $value, $post, $attr, $request, $object_type ) {
					$approval = ph_post_set_approval( $value, $post );

					// trigger email if not a new post
					if ( 'POST' !== $request->get_method() ) {
						do_action( 'ph_rest_set_approval', $post, $value, $attr, $request );
					}

					return $approval;
				},
				'schema'          => array(
					'description' => esc_html__( 'Approval.', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => false,
				),
			)
		);
	}

	/**
	 * Resolved field
	 *
	 * @return void
	 */
	public function resolved_field() {
		register_rest_field(
			$this->post_type, 'resolved', array(
				'get_callback'    => null, // don't allow get
				'update_callback' => function ( $value, $post, $attr, $request, $object_type ) {
					global $is_ph_batch;
					$is_ph_batch = true;

					$threads = PH()->thread->get_items( array(
						'parent_id' => $post->ID,
					) );

					foreach ( $threads as $thread ) {
						$request = new WP_REST_Request( 'PATCH', PH()->thread->route() . '/' . $thread['id'] );
						$request->set_query_params(
							array(
								'resolved' => true
							)
						);

						$response = rest_do_request( $request );

						// handle error and short circuit
						if ( $response->is_error() ) {
							// Convert to a WP_Error object and bail
							error_log(print_r($response->as_error(), 1));
							return $response->as_error();
						}
					}
					return true;
				},
				'schema'          => array(
					'description' => esc_html__( 'Whether to resolve all threads in this image.', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => false,
				),
			)
		);
	}

	/**
	 * Register versions field for schema
	 */
	public function version_field() {
		register_rest_field(
			$this->post_type, 'version', array(
				'schema' => array(
					'description' => esc_html__( 'Save previous version history for this change.', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => false,
				),
			)
		);
	}

	/**
	 * Image options field
	 *
	 * @return void
	 */
	public function options_field() {
		register_rest_field(
			$this->post_type, 'options', array(
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$options = $this->options; // set defaults.

					foreach ( $this->options as $key => $option ) {
						$option_value = get_post_meta( $post['id'], $key, true );
						if ( $option_value ) {
							$options[ $key ] = $option_value;
						}
					}

					return $options;
				},
				'update_callback' => function ( $options, $post, $attr, $request, $object_type ) {
					if ( empty( $options ) || ! is_array( $options ) ) {
						return $options;
					}

					foreach ( $options as $key => $option ) {
						if ( array_key_exists( $key, $this->options ) ) {
							update_post_meta( $post->ID, $key, esc_html( $option ) );
						}
					}

					return $this->options;
				},
				'schema'          => array(
					'type'  => 'object',
					'items' => array(
						'description' => esc_html__( 'Image display options.', 'project-huddle' ),
						'type'        => 'string',
					),
				),
			)
		);
	}

	/**
	 * Fallback filters
	 *
	 * @param WP_Post         $prepared_post Prepared post.
	 * @param WP_Rest_Request $request Rest request.
	 *
	 * @return object
	 */
	public function filters( $prepared_post, $request ) {
		$method = $this->map_method( $request->get_method() );

		return apply_filters( 'ph_' . $method . '_mockup_image', $prepared_post, $request );
	}

	/**
	 * Store approval date and user data when image is approved
	 *
	 * @param string  $attr Attribute key.
	 * @param string  $value Attribute value.
	 * @param WP_Post $post Post object.
	 */
	public function store_approval_comment( $attr, $value, $post ) {
		// if not our meta info or post type, bail.
		if ( 'approval' !== $attr || get_post_type( $post->ID ) !== $this->post_type ) {
			return;
		}

		// get current user.
		$user = wp_get_current_user();

		// Insert new comment and get the comment ID.
		wp_insert_comment(
			array(
				'comment_post_ID'      => $post->ID,
				'comment_author'       => $user->display_name,
				'comment_author_email' => $user->user_email,
				'user_id'              => $user->ID,
				'comment_content'      => $value ? __( 'Approved', 'project-huddle' ) : __( 'Unapproved', 'project-huddle' ),
				'comment_type'         => 'ph_approval',
				'comment_approved'     => 1, // force approval.
				'comment_meta'         => array(
					'approval' => (bool) $value,
				),
			)
		);
	}

	/**
	 * Make approval comment text dynamic
	 *
	 * @param string     $comment_text Comment text.
	 * @param WP_Comment $comment Comment object.
	 *
	 * @return string Comment text
	 */
	public function approval_comment_text( $comment_text = '', $comment = null ) {
		if ( 'ph_approval' !== $comment->comment_type || get_post_type( $comment->comment_post_ID ) !== $this->post_type ) {
			return $comment_text;
		}

		// get dynamic author and approval meta.
		$author   = $comment->user_id ? get_userdata( $comment->user_id )->display_name : $comment->comment_author;
		$approval = get_comment_meta( $comment->comment_ID, 'approval', true ) ? __( 'approved', 'project-huddle' ) : __( 'unapproved', 'project-huddle' );

		// translators: {author} {approved} {title}.
		$comment_text = apply_filters( 'ph_approval_comment_text', sprintf( __( '%1$s %2$s %3$s.' ), esc_html( $author ), esc_html( $approval ), '<strong>' . esc_html( get_the_title( $comment->comment_post_ID ) ) . '</strong>' ), $comment );

		return $comment_text;
	}

	/**
	 * Sort images by menu order to keep order correct.
	 *
	 * @param array $args Arguments for get request.
	 *
	 * @return array
	 */
	public function sort_by_menu_order( $args ) {
		$args['order_by'] = 'menu_order';
		return $args;
	}
}
