<?php
/**
 * Mockup Project Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PH_Project Class
 *
 * This class handles the project data
 *
 * @since 2.6.0
 */
class PH_Mockup_Project extends PH_Rest_Object {
	/**
	 * Post type
	 *
	 * @var string
	 */
	public $post_type = 'ph-project';

	/**
	 * Slug for actions
	 *
	 * @var string
	 */
	public $action_slug = 'project';

	/**
	 * For website commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'mockup';

	/**
	 * Register actions
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// add schema.
		$this->schema = $this->schema();

		// add fields.
		$this->register_fields_from_schema();

		// standardize actions.
		$this->add_actions();
		$this->add_filters();

		// add token field.
		add_action( 'rest_api_init', array( $this, 'add_token_field' ) );

		// register fields.
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// force boolean on strings.
		add_filter( "ph_mockup_rest_get_{$this->action_slug}_attribute", array( $this, 'force_boolean' ), 10, 3 );

		add_filter( "rest_{$this->post_type}_query", array( $this, 'filter_member_projects' ), 10, 2 );
	}


	/**
	 * Schema for post meta
	 *
	 * @return array
	 */
	public function schema() {
		return apply_filters(
			'ph_website_meta', array(
				'images'             => array(
					'description' => esc_html__( 'An array of image post objects belonging to project.', 'project-huddle' ),
					'type'        => 'array',
					'default'     => array(),
					'items'       => array(
						'description' => esc_html__( 'Image post object.', 'project-huddle' ),
						'type'        => 'object',
						'readonly'    => true,
					),
					'readonly'    => true,
				),
				'project_access'     => array(
					'description' => esc_html__( 'Access options for the project', 'project-huddle' ),
					'type'        => 'string',
					'default'     => 'login',
				),
				'retina'             => array(
					'description' => esc_html__( 'Whether to serve the images as retina', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => false,
				),
				'sharing'            => array(
					'description' => esc_html__( 'Is the project sharing UI enabled?', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => true,
				),
				'zoom'               => array(
					'description' => esc_html__( 'Is the project zoom functionality enabled', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => true,
				),
				'tooltip'            => array(
					'description' => esc_html__( 'Show the Leave A Comment tooltip.', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => true,
				),
				'project_comments'   => array(
					'description' => esc_html__( 'Allow non-users to make comments', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => false,
				),
				'project_approval'   => array(
					'description' => esc_html__( 'Approvals allowed?', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => true,
				),
				'project_unapproval' => array(
					'description' => esc_html__( 'Unapprovals allowed?', 'project-huddle' ),
					'type'        => 'boolean',
					'default'     => true,
				),
				'project_members'    => array(
					'description' => esc_html__( 'A list of user IDs that have access to the project.', 'project-huddle' ),
					'type'        => 'array',
					'default'     => array(),
					'items'       => array(
						'description' => esc_html__( 'User id.', 'project-huddle' ),
						'type'        => 'number',
					),
				),
			)
		);
	}

	/**
	 * Access Token Field
	 */
	public function add_token_field() {
		register_rest_field(
			$this->post_type, 'access_token', array(
				'get_callback'   => 'ph_get_access_token',
				'project_access' => array(
					'description' => esc_html__( 'Access token for the project', 'project-huddle' ),
					'type'        => 'string',
					'readonly'    => true,
				),
			)
		);
	}

	/**
	 * Force boolean for "on", "off" on REST requests
	 *
	 * @param string  $data Attribute value.
	 * @param string  $attr Attribute.
	 * @param WP_Post $post Post object.
	 * @return bool
	 */
	public function force_boolean( $data, $attr, $post ) {
		if ( in_array(
			$attr, array(
				'retina',
				'sharing',
				'zoom',
				'tooltip',
				'project_comments',
				'project_approval',
				'project_unapproval',
			)
		) ) {
			$data = 'on' === $data ? true : false;
		}
		return $data;
	}

	/**
	 * Filter project requests to only get projects where the person is a member
	 *
	 * @param array           $args    Key value array of query var to query value.
	 * @param WP_REST_Request $request The request used.
	 * @return array Args.
	 */
	public function filter_member_projects( $args, $request ) {
		// if it's an edit context, simply return query so permissions are used.
		if ( 'edit' === $request['context'] ) {
			return $args;
		}

		if ( ! isset( $args['meta_query'] ) ) {
			$args['meta_query'] = array();
		}

		if ( ! isset( $args['meta_query'] ) || ! is_array( $args['meta_query'] ) ) {
			$args['meta_query'] = array();
		}

		$args['meta_query'][] = array(
			'key'     => 'project_members',
			'value'   => get_current_user_id(),
			'compare' => 'LIKE',
		);

		return $args;
	}

	/**
	 * Get project images
	 *
	 * @param integer $project_id Project ID.
	 * @deprecated 3.0.0 Use ph_get_mockup_images()
	 * @return array
	 */
	public function images( $project_id ) {
		_deprecated_function( __METHOD__, '3.0.0', 'ph_get_mockup_images()' );

		$images = ph_get_mockup_images(
			array(
				'mockup_id'      => $project_id,
				'posts_per_page' => -1,
			)
		);

		foreach ( $images as $key => $image ) {
			// phpcs:ignore
			$image->imageID = get_post_thumbnail_id( $image->ID );
			$image->hash    = $image->post_name;
			$image->title   = $image->post_title;
			$images[ $key ] = (array) $image;
		}

		return $images;
	}
}
