<?php
/**
 * Comment Thread Data
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2017, Andre Gagnon
 * @since       2.6.0
 */

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) { exit; }

/**
 * Website Thread Class
 * @since 2.6.0
 */
class PH_Mockup_Thread extends PH_Rest_Object {
	/**
	 * Post type
	 * @var string
	 */
	public $post_type = 'ph_comment_location';

	/**
	 * Parent collection post type
	 * @var string
	 */
	public $parent_post_type = 'project_image';

	/**
	 * Collection name
	 * @var string
	 */
	public $collection_name = 'threads';

	/*
	 * Slug for actions
	 * @var string
	 */
	public $action_slug = 'thread';

	/**
	 * For mockup commenting
	 *
	 * @var string
	 */
	public $endpoint_type = 'mockup';

	/**
	 * Add object meta and filters
	 */
	public function __construct() {
		// add schema
		$this->schema = $this->schema();

		// add fields
		$this->register_fields_from_schema();

		// register fields
		add_action( 'rest_api_init', array( $this, 'rest_fields_from_schema' ) );

		// auto add comment data
		add_action( 'rest_api_init', array( $this, 'add_comment_data' ) );

		// dynamic thread title
		add_filter( 'the_content', array( $this, 'dynamic_thread_content' ) );

		// comments count field.
		add_filter( 'rest_api_init', array( $this, 'comments_count' ) );

		// add filters
		$this->add_filters();

		// standardize actions
		$this->add_actions();

		// force comments open for this post type
		add_filter( "comments_open", array( $this, 'force_comments_open' ), 10, 2 );

		// store approval comment when approval is set.
		add_action( "ph_website_rest_update_{$this->action_slug}_attribute", array(
			$this,
			'store_approval_comment'
		), 10, 3 );

		// dynamic comment text for approval comments to make sure user display name and translations are up to date.
		add_filter( 'comment_text', array( $this, 'approval_comment_text' ), 8, 2 );

		// sort threads by menu order
		add_filter( "ph_{$this->collection_name}_collection_data", array( $this, 'sort_by_menu_order' ) );
	}

	/**
	 * Schema for post meta
	 * @return array
	 */
	public function schema() {
		return apply_filters( 'ph_mockup_thread_meta', array(
			'relativeX'      => array(
				'description' => esc_html__( 'Relative horizontal click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'relativeY'      => array(
				'description' => esc_html__( 'Relative vertical click position of the element.', 'project-huddle' ),
				'type'        => 'number',
				'default'     => 0
			),
			'parent_id'       => array(
				'description' => esc_html__( 'ID of parent image.', 'project-huddle' ),
				'type'        => 'integer',
				'default'     => 0,
			),
			'resolved'       => array(
				'description' => esc_html__( 'Issue resolve status.', 'project-huddle' ),
				'type'        => 'boolean',
				'default'     => false
			),
			'mockup_id' => array(
				'description' => esc_html__( 'Mockup project id for the thread.', 'project-huddle' ),
				'type'        => 'integer'
			)
		) );
	}

	/**
	 * Collection query params
	 *
	 * Allows querying collections via the following meta data.
	 *
	 * @return array
	 */
	public function collection_params() {
		return apply_filters( 'ph_mockup_thread_collection_parameters', array(
			'resolved' => array(
				'description' => esc_html__( 'Limit results by resolved status.', 'project-huddle' ),
				'type'        => 'boolean',
				'meta'        => true
			),
			'assigned' => array(
				'description' => esc_html__( 'Limit by ID of user who is assigned.', 'project-huddle' ),
				'type'        => 'integer',
				'meta'        => true
			)
		) );
	}

	/**
	 * Add comment data as rest field
	 */
	public function add_comment_data() {
		register_rest_field( $this->post_type, 'comments',
			array(
				'update_callback' => function( $comments, $post, $attr, $request, $object_type ) {
					if ( empty( $comments ) ) {
						return new WP_Error( 'rest_missing_comment', __( 'You must provide at least one comment to create a thread.', 'project-huddle' ), array( 'status' => '400' ) );
					}

					// save comments to post
					foreach ( $comments as $comment ) {
						$comment['post'] = $post->ID; // set the post id

						// new item
						$request = new WP_REST_Request( 'POST', '/' . PH()->comment->namespace . '/' . PH()->comment->rest_base );

						// set new comment params
						$request->set_body_params( apply_filters( 'ph_new_mockup_thread_comment', $comment, $post, $request ));

						// do request
						$created = rest_do_request( $request );
					}

					if ( is_wp_error( $created ) ) {
						return $created;
					}

					return $comments;
				},
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$url = '/' . PH()->comment->namespace . '/' . PH()->comment->rest_base;

					$request = new WP_REST_Request( 'GET', $url );

					$request->set_query_params(
						array(
							'post'     => array( $post['id'] ),
							'per_page' => apply_filters( 'ph_comments_per_page', 25 ),
							'order'    => 'desc'
						)
					);
					$response = rest_do_request( $request );

					return $response->get_data();
				},
				'schema' => array(
					'description' => esc_html__( 'Array of comment objects in thread.', 'project-huddle' ),
					'type'        => 'array',
					'items'       => array(
						'description' => esc_html__( 'Comment object.', 'project-huddle' ),
						'type'        => 'object',
					),
				)
			)
		);
	}

	// thread title defaults to first comment
	function dynamic_thread_content($content) {
		global $post;

		if ( $post->post_type !== $this->post_type ) {
			return $content;
		}

		$comments = ph_get_comments( array(
			'post_id'  => $post->ID,
			'order_by' => 'comment_date',
			'order'    => 'asc',
			'type__in' => array(
				'ph_image_comment',
				'ph_comment',
			),
			'number'   => 1,
		) );

		if ( ! empty( $comments ) ) {
			return $comments[0]->comment_content;
		} else {
			$comments = ph_get_comments( array(
				'post_id'  => $post->ID,
				'order_by' => 'comment_date',
				'order'    => 'desc',
				'type__in' => array(
					'ph_approval',
				),
				'number'   => 1,
			) );

			if ( ! empty( $comments ) ) {
				return $comments[0]->comment_content;
			} else {
				return __( 'No comment text', 'project-huddle' );
			}
		}
	}

	/**
	 * Make sure comments are always open on our parent post
	 * type before comment is saved
	 *	 *
	 * @return array
	 */
	public function force_comments_open( $open, $post_id ) {
		if ( get_post_type( $post_id ) === $this->post_type ) {
			if ( ! $open ) {
				$open = true;
			}
		}

		return $open;
	}

	/**
	 * Store approval date and user data when image is approved
	 *
	 * @param string  $attr  Updated attribute
	 * @param mixed   $value Attribute value
	 * @param WP_Post $post  Post object
	 */
	public function store_approval_comment( $attr, $value, $post ) {
		// if not our meta info or post type, bail
		if ( $attr !== 'resolved' || get_post_type( $post->ID ) !== $this->post_type ) {
			return;
		}

		// don't add if there are no comments yet
		if ( ! get_comments_number( $post->ID ) ) {
			return;
		}

		// get current user
		$user = wp_get_current_user();

		// Insert new comment and get the comment ID
		wp_insert_comment( array(
			'comment_post_ID'      => $post->ID,
			'comment_author'       => $user->display_name,
			'comment_author_email' => $user->user_email,
			'user_id'              => $user->ID,
			'comment_content'      => $value ? __( 'Approved', 'project-huddle' ) : __( 'Unapproved', 'project-huddle' ),
			'comment_type'         => 'ph_approval',
			'comment_approved'     => 1, // force approval
			'comment_meta'         => array(
				'approval' => (bool) $value,
			)
		) );
	}

	/**
	 * Make approval comment text dynamic
	 *
	 * @param string $comment_text
	 * @param null   $comment
	 *
	 * @return string Comment text
	 */
	function approval_comment_text( $comment_text = '', $comment = null ) {
		if ( $comment->comment_type !== 'ph_approval' || get_post_type( $comment->comment_post_ID ) !== $this->post_type ) {
			return $comment_text;
		}

		// get dynamic author and approval meta
		$user = get_userdata( $comment->user_id );
		if ( $user && $user->display_name ) {
			$author = $user->display_name;
		} else {
			$author = $comment->comment_author;
		}

		$approval = get_comment_meta( $comment->comment_ID, 'approval', true ) ? __( 'resolved', 'project-huddle' ) : __( 'unresolved', 'project-huddle' );

		// create comment text
		$comment_text = apply_filters( 'ph_approval_comment_text', sprintf( __( '%1$s %2$s this conversation.' ), esc_html( $author ), esc_html( $approval ) ), $comment );

		return $comment_text;
	}

	/**
	 * Sort images by menu order to keep order correct.
	 * Uses date as a fallback
	 *
	 * @param array $args Arguments for get request.
	 *
	 * @return array
	 */
	public function sort_by_menu_order( $args ) {
		$args['order_by'] = 'menu_order date';
		return $args;
	}

	function comments_count() {
		register_rest_field( $this->post_type, 'comments_count',
			array(
				'update_callback' => null,
				'get_callback'    => function ( $post, $attr, $request, $object_type ) {
					$comments_count = ph_count_comments( $post['id'] );
					return (int) $comments_count->approved;
				},
				'schema'          => array(
					'description' => esc_html__( 'Array of comment objects in thread.', 'project-huddle' ),
					'type'        => 'integer',
					'readonly'    => true,
				)
			)
		);
	}
}