<?php
/**
 * Define default approval status on new posts
 */
function ph_set_default_object_approval( $post_id, $post ) {
	if ( 'publish' === $post->post_status ) {
		// post type must support approvals.
		if ( ! post_type_supports( $post->post_type, 'approvals' ) ) {
			return;
		}

		// set defaults.
		$defaults = array(
			ph_approval_term_taxonomy() => array( ph_get_default_approval_status() ),
		);

		$taxonomies = get_object_taxonomies( $post->post_type );
		foreach ( (array) $taxonomies as $taxonomy ) {
			$terms = wp_get_post_terms( $post_id, $taxonomy );
			if ( empty( $terms ) && array_key_exists( $taxonomy, $defaults ) ) {
				wp_set_object_terms( $post_id, $defaults[ $taxonomy ], $taxonomy );
			} elseif ( ph_approval_term_taxonomy() === $taxonomy && count( $terms ) > 1 ) {
				return new WP_Error( 'You can only add one approval status to this post', 'project-huddle' );
			}
		}
	}
}
add_action( 'save_post', 'ph_set_default_object_approval', 100, 2 );

/**
 * Abstract the term taxonomy
 *
 * @return string
 */
function ph_approval_term_taxonomy() {
    return 'ph_approval';
}
/**
 * Default Approval Status
 *
 * @return string
 */
function ph_get_default_approval_status( $post_id = 0 ) {
	if ( ! $post_id ) {
		global $post;
		$post_id = $post ? $post->ID : 0;
	}

	return apply_filters( 'ph_default_approval_status', 'unapproved', $post_id );
}

function ph_get_default_workflow_status( $post_id = 0 ) {
	if ( ! $post_id ) {
		global $post;
		$post_id = $post ? $post->ID : 0;
	}

	return apply_filters( 'ph_default_workflow_status', 'backlog', $post_id );
}

/**
 * Completed status
 *
 * @return string
 */
function ph_approval_completed_status() {
	return apply_filters( 'ph_completed_status', 'approved' );
}

/**
 * Approval Meta Box Select
 */
function ph_approval_meta_box_select( $post ) {
	// get the terms.
	$terms = get_terms( ph_approval_term_taxonomy(), array( 'hide_empty' => false ) );

	// get the post.
	$post = get_post();

	// get approval terms.
	$approval = wp_get_object_terms(
		$post->ID, ph_approval_term_taxonomy(), array(
			'orderby' => 'term_id',
			'order'   => 'ASC',
		)
	);

	$id = 0;
	if ( ! is_wp_error( $approval ) ) {
		if ( isset( $approval[0] ) && isset( $approval[0]->ID ) ) {
			$id = $approval[0]->ID;
		} else {
			$slug    = ph_get_default_approval_status();
			$default = get_term_by( 'slug', $slug, ph_approval_term_taxonomy() );
			$id      = $default->ID;
		}
	}

	foreach ( $terms as $term ) {?>
		<label title='<?php esc_attr_e( $term->ID ); ?>'>
			<input type="radio" name="ph_approval" value="<?php esc_attr_e( $term->ID ); ?>" <?php checked( $term->ID, $id ); ?>>
			<span><?php esc_html_e( $term->name ); ?></span>
		</label><br>
		<?php
	}
	?>
	<p><a href="<?php echo esc_url( admin_url( 'edit-tags.php?taxonomy=ph_approval' ) ); ?>"><?php esc_html_e( 'Edit Approval Statuses', 'project-huddle' ); ?></a></p>
	<?php
}

/**
 * Save the approval meta box
 *
 * @param int $post_id The ID of the post that's being saved.
 */
function save_ph_approval_meta_box( $post_id ) {
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return;
	}
	if ( ! isset( $_POST[ph_approval_term_taxonomy()] ) ) {
		return;
	}
	$approval = sanitize_text_field( $_POST[ph_approval_term_taxonomy()] );

	// A valid approval is required, so don't let this get published without one.
	if ( ! empty( $approval ) ) {
		$term = get_term_by( 'name', $approval, ph_approval_term_taxonomy() );
		if ( ! empty( $term ) && ! is_wp_error( $term ) ) {
			wp_set_object_terms( $post_id, $term->term_id, ph_approval_term_taxonomy(), false );
		}
	}
}
add_action( 'save_post', 'save_ph_approval_meta_box' );

/**
 * Is the mockup image approved
 *
 * @param WP_Post|int $post_id Post id or post object.
 *
 * @return bool
 */
function ph_post_is_approved( $post_id = 0 ) {
	if ( ! is_int( $post_id ) && is_a( $post_id, 'WP_Post' ) ) {
		$post_id = $post_id->ID;
	}

	return has_term( ph_approval_completed_status(), ph_approval_term_taxonomy(), $post_id );
}

/**
 * Set mockup image approval status
 *
 * @param boolean $approved Approved or not.
 * @param WP_Post $post     Post object to approve.
 *
 * @return array|WP_Error Term taxonomy IDs of the affected terms.
 */
function ph_post_set_approval( $approved, $post ) {
	if ( ! $post ) {
		global $post;
	}
	$post_object = get_post( $post );

	$status = $approved ? ph_approval_completed_status() : ph_get_default_approval_status();

	do_action( 'ph_post_set_approval', $status, $post, $approved );

	wp_delete_object_term_relationships( $post_object->ID, ph_approval_term_taxonomy() );
	return wp_set_object_terms( $post_object->ID, $status, ph_approval_term_taxonomy(), true );
}

/**
 * Get mockup project approval status
 *
 * @param int $mockup_id
 *
 * @return array
 */
function ph_get_mockup_approval_status( $mockup_id = 0 ) {
	$defaults = array(
		'total'    => 0,
		'approved' => 0,
	);

	if ( ! $mockup_id ) {
		global $post;
		$mockup_id = isset( $post->ID ) ? $post->ID : false;
	}

	if ( ! $mockup_id ) {
		return $defaults;
	}

	$approval_status = get_transient( 'ph_mockup_approval_status_' . $mockup_id );

	// this code runs when there is no valid transient set.
	if ( false === $approval_status ) {
		// get pages.
		$images = new WP_Query(
			array(
				'post_type'      => 'project_image',
				'posts_per_page' => -1,
				'meta_value'     => $mockup_id,
				'meta_key'       => 'parent_id',
				'fields'         => 'ids',
			)
		);

		$image_ids         = $images->posts;
		$approval_comments = array();
		$approved          = 0;

		if ( ! empty( $image_ids ) ) {
			// count approved
			foreach ( $image_ids as $image_id ) {
				if ( ph_post_is_approved( $image_id ) ) {
					$approved ++;
				}
			}

			// get last approval comment
			$approval_comments = ph_get_comments(
				array(
					'type'     => ph_approval_term_taxonomy(),
					'post__in' => $image_ids,
					'number'   => 1,
				)
			);
		}

		$by = false;
		$on = false;

		if ( ! empty( $approval_comments ) && is_array( $approval_comments ) ) {
			$comment = $approval_comments[0];

			if ( is_a( $comment, 'WP_Comment' ) ) {
				$by = $comment->user_id ? get_userdata( $comment->user_id )->display_name : $comment->comment_author;
				$on = $comment->comment_date;
			}
		}

		$approval_status = array(
			'total'    => count( $image_ids ),
			'approved' => $approved,
			'by'       => $by,
			'on'       => $on,
		);

		set_transient( 'ph_mockup_approval_status_' . $mockup_id, $approval_status, 30 * DAY_IN_SECONDS ); // expires in 1 month
	}

	return wp_parse_args( $approval_status, $defaults );
}

/**
 * Is the mockup approved?
 *
 * @param $id
 *
 * @return bool
 */
function ph_mockup_is_approved( $id ) {
	$status = ph_get_mockup_approval_status( $id );

	if ( $status['total'] ) {
		return $status['total'] <= $status['approved'];
	}

	return false;
}

/**
 * Get mockup project approval status
 *
 * @param int $image_id Image id.
 *
 * @return bool|array
 */
function ph_get_mockup_image_approval_status( $image_id = 0 ) {
	if ( ! $image_id ) {
		global $post;
		$image_id = isset( $post->ID ) ? $post->ID : false;
	}

	if ( ! $image_id ) {
		return false;
	}

	// get last approval comment.
	$approval_comments = ph_get_comments(
		array(
			'type'     => ph_approval_term_taxonomy(),
			'post__in' => array( $image_id ),
			'number'   => 1,
		)
	);

	$by = '';
	$on = '';

	if ( ! empty( $approval_comments ) && is_array( $approval_comments ) ) {
		$comment = $approval_comments[0];

		if ( is_a( $comment, 'WP_Comment' ) ) {
			$by = $comment->user_id ? get_userdata( $comment->user_id )->display_name : $comment->comment_author;
			$on = $comment->comment_date;
		}
	}

	return apply_filters(
		'ph_mockup_image_approval_status', array(
			'approved' => ph_post_is_approved( $image_id ),
			'by'       => $by,
			'on'       => $on,
		), $image_id
	);
}

/**
 * Clear approval transient when approval meta is updated
 *
 * @param int    $object_id Object ID.
 * @param array  $terms     An array of object terms.
 * @param array  $tt_ids    An array of term taxonomy IDs.
 * @param string $taxonomy  Taxonomy slug.
 */
function ph_clear_mockup_approval_transients( $object_id, $terms, $tt_ids, $taxonomy ) {
	if ( get_post_type( $object_id ) != 'project_image' || $taxonomy !== ph_approval_term_taxonomy() ) {
		return;
	}

	// get mockup ID
	$mockup_id = get_post_meta( $object_id, 'parent_id', true );

	delete_transient( 'ph_mockup_approval_status_' . $mockup_id );
}

// clear transients when post meta is updated
add_action( 'set_object_terms', 'ph_clear_mockup_approval_transients', 10, 4 );

/**
 * Clear approval transient when approval meta is updated
 *
 * @param int    $object_id Object ID.
 * @param array  $terms     An array of object terms.
 * @param array  $tt_ids    An array of term taxonomy IDs.
 * @param string $taxonomy  Taxonomy slug.
 */
function ph_store_approval_info( $object_id, $terms, $tt_ids, $taxonomy ) {
	if ( $taxonomy !== ph_approval_term_taxonomy() ) {
		return;
	}

	if ( empty( $terms ) ) {
		return;
	}

	// get current user
	$user = wp_get_current_user();

	// approval completed status
	$value = isset( $terms[0] ) && $terms[0] === ph_approval_completed_status();

	// Insert new comment and get the comment ID
	wp_insert_comment(
		array(
			'comment_post_ID'      => $object_id,
			'comment_author'       => $user->display_name,
			'comment_author_email' => $user->user_email,
			'user_id'              => $user->ID,
			'comment_content'      => $value ? __( 'Approved', 'project-huddle' ) : __( 'Unapproved', 'project-huddle' ),
			'comment_type'         => ph_approval_term_taxonomy(),
			'comment_approved'     => 1, // force approval
			'comment_meta'         => array(
				'approval' => (bool) $value,
			),
		)
	);
}

// clear transients when post meta is updated
add_action( 'set_object_terms', 'ph_store_approval_info', 10, 4 );

/**
 * Add custom approve text to project data
 *
 * @return mixed|void
 */
function ph_approval_text_options() {
	global $post;

	$translations = array();

	// approve/unapprove text
	if ( $approve_text = get_option( 'ph_approve_text', false ) ) {
		$translations['confirmApprove'] = wp_kses_post( $approve_text );
	}
	if ( $unapprove_text = get_option( 'ph_unapprove_text', false ) ) {
		$translations['confirmUnApprove'] = wp_kses_post( $unapprove_text );
	}

	// checkbox/terms
	if ( $checkbox_text = get_option( 'ph_approve_terms_checkbox_text', false ) ) {
		$translations['approveCheckbox'] = esc_html( $checkbox_text );
	}
	if ( $checkbox_link = get_option( 'ph_approve_terms_link_text', false ) ) {
		$translations['approveCheckboxLink'] = esc_html( $checkbox_link );
	}
	if ( $terms = get_option( 'ph_approve_terms', false ) ) {
		$translations['approveTerms'] = wp_kses_post( wpautop($terms) );
	}

	// return translations and filter in case we need to change on a per-post basis
	return apply_filters( 'ph_approval_text', $translations, $post );
}

function ph_get_image_resolve_status( $image_id = 0 ) {
	$defaults = array(
		'total'    => 0,
		'approved' => 0,
	);

	if ( ! $image_id ) {
		global $post;
		$image_id = isset( $post->ID ) ? $post->ID : false;
	}

	if ( ! $image_id ) {
		return $defaults;
	}

	$resolved_status = get_transient( 'ph_image_resolve_status_' . $image_id );

	// this code runs when there is no valid transient set.
	if ( false === $resolved_status ) {
		// get threads
		$threads = new WP_Query(
			array(
				'post_type'      => 'ph_comment_location',
				'posts_per_page' => -1,
				'meta_value'     => $image_id,
				'meta_key'       => 'parent_id',
				'fields'         => 'ids',
			)
		);

		$thread_ids         = $threads->posts;
		$approval_comments = array();
		$resolved          = 0;

		if ( ! empty( $thread_ids ) ) {
			// count approved
			foreach ( $thread_ids as $image_id ) {
			    // if resolved
			    if ( get_post_meta($image_id, 'resolved', true) ) {
				    $resolved ++;
                }
			}

			// get last approval comment
			$approval_comments = ph_get_comments(
				array(
					'type'     => ph_approval_term_taxonomy(),
					'post__in' => $thread_ids,
					'number'   => 1,
				)
			);
		}

		$by = false;
		$on = false;

		if ( ! empty( $approval_comments ) && is_array( $approval_comments ) ) {
			$comment = $approval_comments[0];

			if ( is_a( $comment, 'WP_Comment' ) ) {
				$by = $comment->user_id ? get_userdata( $comment->user_id )->display_name : $comment->comment_author;
				$on = $comment->comment_date;
			}
		}

		$resolved_status = array(
			'total'    => count( $thread_ids ),
			'resolved' => $resolved,
			'by'       => $by,
			'on'       => $on,
		);

		set_transient( 'ph_image_resolve_status_' . $image_id, $resolved_status, 30 * DAY_IN_SECONDS ); // expires in 1 month
	}

	return wp_parse_args( $resolved_status, $defaults );
}

/**
 * Clear approval transient when approval meta is updated
 *
 * @param int    $object_id Object ID.
 * @param array  $terms     An array of object terms.
 * @param array  $tt_ids    An array of term taxonomy IDs.
 * @param string $taxonomy  Taxonomy slug.
 */
function ph_clear_image_resolve_transients( $meta_id, $post_id, $meta_key, $meta_value ) {
	if ( get_post_type( $post_id ) != 'ph_comment_location' || 'resolved' !== $meta_key ) {
		return;
	}

	// get mockup ID
	$image_id = get_post_meta( $post_id, 'parent_id', true );

	// delete transient
	delete_transient( 'ph_image_resolve_status_' . $image_id );
}

// clear transients when post meta is updated
add_action( 'updated_post_meta', 'ph_clear_mockup_approval_transients', 10, 4 );