<?php
/**
 * Comment Functions
 *
 * Functions related to comments
 *
 * @package     Project Huddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Custom function for Human Time Diff
 *
 * Allows translations to be possible
 *
 * @param        $from
 * @param string $to
 *
 * @return string|void
 */
function ph_human_time_diff( $from, $to = '' ) {

	// If to is empty, get current time
	if ( empty( $to ) ) {
		$to = time();
	}

	$since = '';

	// get difference
	$diff = (int) abs( $to -  $from );

	// check hours, minutes days
	if ( $diff <= HOUR_IN_SECONDS ) {
		$mins = round( $diff / MINUTE_IN_SECONDS );

		// if it's less than one minute
		if ( $mins <= 1 ) {
			$since = __('Now', 'project-huddle');
		} else {
			/* translators: min=minute */
			$since = sprintf( _n( '%sm', '%sm', $mins ), $mins );
		}
	} elseif ( ( $diff <= DAY_IN_SECONDS ) && ( $diff > HOUR_IN_SECONDS ) ) {
		$hours = round( $diff / HOUR_IN_SECONDS );
		if ( $hours <= 1 ) {
			$hours = 1;
		}
		$since = sprintf( _n( '%sh', '%sh', $hours ), $hours );
	} elseif ( $diff >= DAY_IN_SECONDS ) {
		$days = round( $diff / DAY_IN_SECONDS );
		if ( $days <= 1 ) {
			$days = 1;
		}
		$since = sprintf( _n( '%sd', '%sd', $days ), $days );
	}

	return $since;
}

/**
 * Exclude notes (comments) on edd_payment post type from showing in Recent
 * Comments widgets
 *
 * @since 1.4.1
 * @param obj $query WordPress Comment Query Object
 * @return void
 */
function ph_hide_ph_comments( $query ) {
	global $wp_version;

	if ( version_compare( floatval( $wp_version ), '4.1', '>=' ) ) {
		$types = isset( $query->query_vars['type__not_in'] ) ? $query->query_vars['type__not_in'] : array();
		if ( ! is_array( $types ) ) {
			$types = array( $types );
		}

		foreach( ph_get_comment_types() as $type ) {
			$types[] = $type;
		}

		$query->query_vars['type__not_in'] = $types;
	}
}

add_action( 'pre_get_comments', 'ph_hide_ph_comments', 10 );

function ph_get_comment_types() {
	return array(
		'ph_image_comment',
		'ph_website_comment',
		'ph_comment',
		'ph_approval'
	);
}

function ph_comment_type_name( $type ) {
	switch ( $type ) {
		case 'ph_website_comment':
		case 'ph_image_comment':
		case 'ph_comment':
			return 'comment';
		case 'ph_approval':
			return 'approval';
		default:
			return apply_filters( 'ph_comment_type_name', $type );
	}
}

/**
 * Gets the comment project type
 *
 * @param int $id
 *
 * @return string
 */
function ph_get_comment_project_type( $id ) {
	// get parent ids
	$parent_ids = ph_get_parents_ids( $id, 'comment' );

	// get endpoint type based on parent
	$type = isset( $parent_ids['website'] ) ? 'website' : 'mockup';

	// get comment project type
	return apply_filters( 'ph_get_comment_project_type', $type, $id );
}

/**
 * Remove EDD Comments from the wp_count_comments function
 *
 * @access public
 * @since 1.5.2
 * @param array $stats (empty from core filter)
 * @param int $post_id Post ID
 * @return array Array of comment counts
 */
function ph_remove_comments_in_comment_counts( $stats, $post_id ) {
	global $wpdb, $pagenow;

	$array_excluded_pages = array( 'index.php', 'edit-comments.php' );
	if( ! in_array( $pagenow, $array_excluded_pages )  ) {
		return $stats;
	}

	$post_id = (int) $post_id;

	if ( apply_filters( 'edd_count_payment_notes_in_comments', false ) )
		return $stats;

	$stats = wp_cache_get( "comments-{$post_id}", 'counts' );

	if ( false !== $stats )
		return $stats;

	$where = 'WHERE ';
	$count = count( ph_get_comment_types() );
	$i     = 0;

	foreach ( ph_get_comment_types() as $type ) {
		$where .= 'comment_type != "' . $type . '"';
		if ( ++ $i !== $count ) {
			$where .= ' AND ';
		}
	}

	if ( $post_id > 0 )
		$where .= $wpdb->prepare( " AND comment_post_ID = %d", $post_id );

	$count = $wpdb->get_results( "SELECT comment_approved, COUNT( * ) AS num_comments FROM {$wpdb->comments} {$where} GROUP BY comment_approved", ARRAY_A );

	$total = 0;
	$approved = array( '0' => 'moderated', '1' => 'approved', 'spam' => 'spam', 'trash' => 'trash', 'post-trashed' => 'post-trashed' );
	foreach ( (array) $count as $row ) {
		// Don't count post-trashed toward totals
		if ( 'post-trashed' != $row['comment_approved'] && 'trash' != $row['comment_approved'] )
			$total += $row['num_comments'];
		if ( isset( $approved[$row['comment_approved']] ) )
			$stats[$approved[$row['comment_approved']]] = $row['num_comments'];
	}

	$stats['total_comments'] = $total;
	foreach ( $approved as $key ) {
		if ( empty($stats[$key]) )
			$stats[$key] = 0;
	}

	$stats = (object) $stats;
	wp_cache_set( "comments-{$post_id}", $stats, 'counts' );

	return $stats;
}
add_filter( 'wp_count_comments', 'ph_remove_comments_in_comment_counts', 10, 2 );

/**
 * Retrieve all notes attached to a purchase
 *
 * @since 1.4
 * @param array $args Comment query args
 * @return array $comments ProjectHuddle comments
 */
function ph_get_comments( $args ) {

	remove_action( 'pre_get_comments', 'ph_hide_ph_comments', 10 );

	$comments = get_comments( $args );

	add_action( 'pre_get_comments', 'ph_hide_ph_comments', 10 );

	return $comments;
}

/**
 * Count ProjectHuddle comments
 * @param int $post_id
 *
 * @return array|object
 */
function ph_count_comments( $post_id = 0 ) {
	remove_filter( 'wp_count_comments', 'ph_remove_comments_in_comment_counts', 10 );
	$comments_count = wp_count_comments( $post_id );
	add_filter( 'wp_count_comments', 'ph_remove_comments_in_comment_counts', 10, 2 );

	return $comments_count;
}

// add additional html tags for mockup comments
function ph_set_allowed_tags( $prepared_comment ) {
	global $allowedtags;

	$allowedtags['br'] = array();
	$allowedtags['p']  = array();
	$allowedtags['ol'] = array();
	$allowedtags['ul'] = array();
	$allowedtags['li'] = array();

	return $prepared_comment;
}
add_action( 'rest_pre_insert_comment', 'ph_set_allowed_tags' );