<?php
/**
 * Misc functions
 *
 * Miscellaneous functions for ProjectHuddle
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Adds an identity body class to prevent style conflicts
 *
 * @param  array $classes array of included body classes
 *
 * @return array          modified array of body classes
 */
function ph_body_class( $classes ) {
// add our body class identity
	$classes[] = 'projecthuddle';

	return $classes;
}

add_filter( 'body_class', 'ph_body_class' );

function ph_get_remote_IP() {
	if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
		return $_SERVER['HTTP_CLIENT_IP'];

	} else if ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
		return $_SERVER['HTTP_X_FORWARDED_FOR'];
	}

	return $_SERVER['REMOTE_ADDR'];
}

if ( ! function_exists( 'ph_adjust_brightness' ) ) :
    /**
     * ph_adjust_brightness
     */
	function ph_adjust_brightness( $hex, $steps ) {
		// Steps should be between -255 and 255. Negative = darker, positive = lighter
		$steps = max( - 255, min( 255, $steps ) );

		// Normalize into a six character long hex string
		$hex = str_replace( '#', '', $hex );
		if ( strlen( $hex ) == 3 ) {
			$hex = str_repeat( substr( $hex, 0, 1 ), 2 ) . str_repeat( substr( $hex, 1, 1 ), 2 ) . str_repeat( substr( $hex, 2, 1 ), 2 );
		}

		// Split into three parts: R, G and B
		$color_parts = str_split( $hex, 2 );
		$return      = '#';

		foreach ( $color_parts as $color ) {
			$color  = hexdec( $color ); // Convert to decimal
			$color  = max( 0, min( 255, $color + $steps ) ); // Adjust color
			$return .= str_pad( dechex( $color ), 2, '0', STR_PAD_LEFT ); // Make two char hex code
		}

		return $return;
    }
endif;

if ( ! function_exists( 'ph_badge_css' ) ) :
	/**
	 * Badge CSS conversions
	 *
	 * @var $color string Color
	 */
	function ph_badge_css( $color = '#000000' ) {
		list($r, $g, $b) = sscanf($color, "#%02x%02x%02x"); ?>

		background: <?php echo $r && $g && $b ? "rgba($r, $g, $b, 0.1)" : '#f3f3f3'; ?>;
        color: <?php echo esc_html( ph_adjust_brightness( $color, -80) ); ?>;

		<?php
    }
endif;

/*
* Inserts a new key/value before the key in the array.
*
* @param $key
*   The key to insert before.
* @param $array
*   An array to insert in to.
* @param $new_key
*   The key to insert.
* @param $new_value
*   An value to insert.
*
* @return
*   The new array if the key exists, FALSE otherwise.
*
* @see array_insert_after()
*/
function ph_array_insert_before($key, array &$array, $new_key, $new_value) {
	if (array_key_exists($key, $array)) {
		$new = array();
		foreach ($array as $k => $value) {
			if ($k === $key) {
				$new[$new_key] = $new_value;
			}
			$new[$k] = $value;
		}
		return $new;
	}
	return FALSE;
}

/**
 * Inserts a new key/value after the key in the array.
 *
 * @param string $key       The key to insert after.
 * @param array  $array     An array to insert in to.
 * @param string $new_key   The key to insert.
 * @param mixed  $new_value A value to insert.
 *
 * @return array|bool
 */
function ph_array_insert_after( $key, array &$array, $new_key, $new_value ) {
	if ( array_key_exists( $key, $array ) ) {
		$new = array();
		foreach ( $array as $k => $value ) {
			$new[ $k ] = $value;
			if ( $k === $key ) {
				$new[ $new_key ] = $new_value;
			}
		}

		return $new;
	}

	return false;
}

function ph_get_the_user_ip() {
	if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
        //check ip from share internet
		$ip = $_SERVER['HTTP_CLIENT_IP'];
	} elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
        //to check ip is pass from proxy
		$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	} else {
		$ip = $_SERVER['REMOTE_ADDR'];
	}
	return apply_filters( 'ph_get_ip', $ip );
}

/**
 * Convert Hex to RGBA
 *
 * @param string $c Hex color
 *
 * @return string
 */
function ph_hex2rgb( $c ) {
	if ( $c[0] == '#' ) {
		$c = substr( $c, 1 );
	}
	if ( strlen( $c ) == 6 ) {
		list( $r, $g, $b ) = array( $c[0] . $c[1], $c[2] . $c[3], $c[4] . $c[5] );
	} elseif ( strlen( $c ) == 3 ) {
		list( $r, $g, $b ) = array( $c[0] . $c[0], $c[1] . $c[1], $c[2] . $c[2] );
	} else {
		return false;
	}
	$r = hexdec( $r );
	$g = hexdec( $g );
	$b = hexdec( $b );

	return $r . ', ' . $g . ', ' . $b;
}

/**
 * Convert Hex to RGBA
 *
 * @param string $c Hex color
 * @param float  $a Alpha transparency
 *
 * @return string
 */
function ph_hex2rgba( $c, $a ) {
	if ( $c[0] == '#' ) {
		$c = substr( $c, 1 );
	}
	if ( strlen( $c ) == 6 ) {
		list( $r, $g, $b ) = array( $c[0] . $c[1], $c[2] . $c[3], $c[4] . $c[5] );
	} elseif ( strlen( $c ) == 3 ) {
		list( $r, $g, $b ) = array( $c[0] . $c[0], $c[1] . $c[1], $c[2] . $c[2] );
	} else {
		return false;
	}
	$r = hexdec( $r );
	$g = hexdec( $g );
	$b = hexdec( $b );

	return $r . ', ' . $g . ', ' . $b . ', ' . $a;
}

/**
 * Gets all post children and grandchildren
 *
 * Essentially the opposite of get_post_ancestors
 *
 * @param $post_id
 *
 * @return array
 */
function ph_get_all_post_children( $post_id ) {

	// get_children args
	$args = array(
		'post_parent' => $post_id,
		'post_type'   => 'project_image',
		'numberposts' => 1,
		'post_status' => 'publish'
	);

	// get children (one level deep)
	$child = get_children( $args );

	// bail if no children
	if ( ! $child || ! is_array( $child ) ) {
		return array();
	}

	$ancestors = array();

	$id = current($child)->ID;
	$ancestors[] = current($child);

	// loop through and get all children
	while ( $ancestor = get_children( array(
		'post_parent' => $id,
		'post_type'   => 'project_image',
		'numberposts' => 1,
		'post_status' => 'publish'
	) ) ) {
		// Loop detection: If the ancestor has been seen before, break.
		if ( empty( $ancestor ) || ( current($ancestor)->ID == $id ) || in_array( current($ancestor)->ID, $ancestors ) )
			break;

		$id = current($ancestor)->ID;
		$ancestors[] = current($ancestor);
	}

	return $ancestors;
}

/**
 * Get size information for all currently-registered image sizes.
 *
 * @global $_wp_additional_image_sizes
 * @uses   get_intermediate_image_sizes()
 * @return array $sizes Data for all currently-registered image sizes.
 */
function ph_get_image_sizes() {
	global $_wp_additional_image_sizes;

	$sizes = array();

	foreach ( get_intermediate_image_sizes() as $_size ) {
		if ( in_array( $_size, array('thumbnail', 'medium', 'medium_large', 'large') ) ) {
			$sizes[ $_size ]['width']  = get_option( "{$_size}_size_w" );
			$sizes[ $_size ]['height'] = get_option( "{$_size}_size_h" );
			$sizes[ $_size ]['crop']   = (bool) get_option( "{$_size}_crop" );
		} elseif ( isset( $_wp_additional_image_sizes[ $_size ] ) ) {
			$sizes[ $_size ] = array(
				'width'  => $_wp_additional_image_sizes[ $_size ]['width'],
				'height' => $_wp_additional_image_sizes[ $_size ]['height'],
				'crop'   => $_wp_additional_image_sizes[ $_size ]['crop'],
			);
		}
	}

	return $sizes;
}

/**
 * Get size information for a specific image size.
 *
 * @uses   get_image_sizes()
 * @param  string $size The image size for which to retrieve data.
 * @return bool|array $size Size data about an image size or false if the size doesn't exist.
 */
function ph_get_image_size( $size ) {
	$sizes = ph_get_image_sizes();

	if ( isset( $sizes[ $size ] ) ) {
		return $sizes[ $size ];
	}

	return false;
}

/**
 * Get the width of a specific image size.
 *
 * @uses   get_image_size()
 * @param  string $size The image size for which to retrieve data.
 * @return bool|string $size Width of an image size or false if the size doesn't exist.
 */
function ph_get_image_width( $size ) {
	if ( ! $size = ph_get_image_size( $size ) ) {
		return false;
	}

	if ( isset( $size['width'] ) ) {
		return $size['width'];
	}

	return false;
}

/**
 * Get the height of a specific image size.
 *
 * @uses   get_image_size()
 * @param  string $size The image size for which to retrieve data.
 * @return bool|string $size Height of an image size or false if the size doesn't exist.
 */
function ph_get_image_height( $size ) {
	if ( ! $size = ph_get_image_size( $size ) ) {
		return false;
	}

	if ( isset( $size['height'] ) ) {
		return $size['height'];
	}

	return false;
}