<?php
/**
 * Permission functions for ProjectHuddle
 *
 * @since   2.3.1
 * @package ProjectHuddle
 */

/**
 * Run a permissions check on our custom post types
 * to make sure they have access
 */
function ph_user_has_member_access() {
	global $post;

	if ( is_admin() ) {
		return;
	}

	// check to make sure unsilo isn't checked.
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return;
	}

	$get_post = isset( $_GET['post'] ) ? $_GET['post'] : false;

	// if not our post type.
	if ( ! $post || ! in_array( get_post_type( $get_post ), ph_get_post_types() ) ) {
		return;
	}

	// allow on front end if permssions allow.
	if ( get_post_meta( $post->ID, 'ph_project_access', true ) != 'login' ) {
		return;
	}

	// if user isn't logged in, do default project permission
	if ( ! is_user_logged_in() ) {
		return;
	}

	// check if member or should have access otherwise
	if ( ! ph_user_is_member( $post ) ) :
		wp_die( __( 'You don\'t have permission to access this project. Please contact the site owner to gain permissions.', 'project-huddle' ), 'ph-project-permissions-check', array( 'back_link' => true ) );
	endif;
}

add_action( 'wp', 'ph_user_has_member_access' );

function ph_user_has_member_access_admin() {
	global $pagenow;

	// check to make sure unsilo isn't checked
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return;
	}

	// get post object
	$post = isset( $_GET['post'] ) ? get_post( $_GET['post'] ) : false;

	// we're not on the edit post page
	if ( $pagenow != 'post.php' || ! isset( $post ) ) {
		return;
	}

	// we're not on one of our post types
	if ( ! in_array( get_post_type( $post ), ph_get_post_types() ) ) {
		return;
	}

	// check if member or should have access otherwise
	if ( ! ph_user_is_member( $post ) ) :
		wp_die( __( 'You don\'t have permission to access this project. Please contact the site owner to gain permissions.', 'project-huddle' ), 'ph-project-permissions-check', array( 'back_link' => true ) );
	endif;
}

add_action( 'admin_init', 'ph_user_has_member_access_admin' );

/**
 * Restricts All Mockups page to show only subscribed projects
 *
 * @param $query
 *
 * @return mixed
 */
function ph_restrict_for_current_author( $query ) {
	// check option
	if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
		return $query;
	}

	global $pagenow, $typenow;

	// need to be on all posts page and admin
	if ( 'edit.php' != $pagenow || ! $query->is_admin ) {
		return $query;
	}

	// check against post types
	if ( ! in_array( $typenow, ph_get_post_types() ) ) {
		return $query;
	}

	// can view unsubscribed projects
	foreach ( ph_get_post_types() as $type ) {
		if ( $type == $typenow ) {
			if ( current_user_can( 'view_unsubscribed_' . $type . 's' ) ) {
				return $query;
			}
		}
	}

	// if it's a project collaborator and the user is not the post author
	global $user_ID;
	// get user
	$user = get_userdata( $user_ID );

	$query->set(
		'meta_query', array(
			array(
				'key'     => 'ph_project_emails_enable',
				'value'   => $user->user_email,
				'compare' => 'LIKE',
			),
		)
	);

	return $query;
}

add_filter( 'pre_get_posts', 'ph_restrict_for_current_author' );

if ( ! function_exists( 'ph_is_user_subscribed' ) ) :
	/**
	 * Is the user subscribed?
	 *
	 * @param WP_User $user        int|object User object or integer.
	 * @param WP_Post $post_object Post object.
	 *
	 * @return boolean If the user is subscribed to the project
	 */
	function ph_is_user_subscribed( $user = 0, $post_object = 0 ) {
		// normalize user.
		if ( $user ) {
			if ( is_int( $user ) ) {
				$user = get_user_by( 'ID', $user );
			} elseif ( ! is_object( $user ) ) {
				return false;
			}
		} else {
			$user = wp_get_current_user();
		}

		global $post;
		$project = $post_object ? $post_object : $post;

		if ( is_a( $project, 'WP_Post' ) ) {
			$subscribed = (array) ph_get_project_member_ids( $project->ID );

			return in_array( $user->ID, $subscribed );
		} else {
			return false;
		}
	}
endif;

if ( ! function_exists( 'ph_user_is_member' ) ) :
	/**
	 * Checks if a user is a member of a specific project
	 *
	 * @param $post int|object Post ID
	 *
	 * @return bool
	 */
	function ph_user_is_member( $post ) {
		// if unsilo is on
		if ( get_option( 'ph_un_silo', 'off' ) === 'on' ) {
			return true;
		}

		// normalize post object
		if ( is_int( $post ) ) {
			$post = get_post( $post );
		}

		$user = wp_get_current_user();

		// true if subscribed, no matter the role
		if ( ph_is_user_subscribed( $user, $post ) ) {
			return true;
		}

		// otherwise true if current user can view unsubscribed projects or they are the author
		return current_user_can( 'edit_ph-project', $post->ID ) || $post->post_author == get_current_user_id();
	}
endif;

if ( ! function_exists( 'ph_visitor_can_access' ) ) :
	/**
	 * Can the visitor access the website project
	 *
	 * @param int    $post_id      Project ID
	 * @param string $access_token Unique private access token for the project
	 *
	 * @return bool|WP_Error True if visitor should have access
	 */
	function ph_visitor_can_access( $access_token = '', $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		// switch project access
		switch ( ph_get_project_access( $post_id ) ) {
			case 'public':
				return true;
				break;
			case 'link':
				// check access token
				if ( $access_token === get_post_meta( $post_id, 'access_token', true ) ) {
					return true;
				} else {
					if ( is_user_logged_in() && ph_user_is_member( $post_id ) ) {
						return true;
					} else {
						return new WP_Error( 'token_incorrect_missing', __( 'The access link provided is incorrect. Please check to make sure you copied it correctly!', 'project-huddle' ), array( 'status' => 403 ) );
					}
				}
				break;
			case 'password':
				if ( post_password_required() ) {
					return new WP_Error( 'enter_password', __( 'Please enter a password to view this project', 'project-huddle' ), array( 'status' => 403 ) );
				} else {
					return true;
				}
				break;
			default:
				if ( is_user_logged_in() && ph_user_is_member( $post_id ) ) {
					return true;
				} elseif ( is_user_logged_in() && ! ph_user_is_member( $post_id ) ) {
					return new WP_Error( 'not_member', __( 'You must be a member of this project to access it.', 'project-huddle' ), array( 'status' => 403 ) );
				} else {
					return new WP_Error( 'not_logged_in', __( 'You must login to access this project', 'project-huddle' ), array( 'status' => 403 ) );
				}
				break;
		}
	}
endif;

if ( ! function_exists( 'ph_user_validate_access' ) ) :
	/**
	 * This function processes user access
	 * Adds the user to the project if they have access
	 *
	 * @param int $post_id Project ID
	 *
	 * @return bool|WP_Error
	 */
	function ph_user_validate_access( $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		$access_token = isset( $_GET['access_token'] ) ? $_GET['access_token'] : '';

		// store access token in cookie if set in url
		if ( isset( $access_token ) && $access_token ) {
			PH()->session->set(
				'project_access', array(
					(int) $post_id => $access_token,
				)
			);
			// otherwise check for project access session
		} else {
			$access       = PH()->session->get( 'project_access' );
			$access_token = isset( $access[ $post_id ] ) ? $access[ $post_id ] : false;
		}

		// does the visitor have access
		$has_access = ph_visitor_can_access( $access_token, (int) $post_id );

		// add the project member if they have access
		if ( ! is_wp_error( $has_access ) && $has_access && $current_user = wp_get_current_user() ) {
			ph_add_project_member( (int) $post_id, $current_user );
		}

		return $has_access;
	}
endif;

if ( ! function_exists( 'ph_is_token_valid' ) ) :
	/**
	 * Checks if an access token is valid
	 *
	 * @param string  $token   Access Token
	 * @param integer $post_id Project ID
	 *
	 * @return boolean
	 */
	function ph_is_token_valid( $token = '', $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = $post->ID;
		}

		return $token === get_post_meta( $post_id, 'access_token', true );
	}
endif;

if ( ! function_exists( 'ph_get_access_token' ) ) :
	/**
	 * Get the access token for a website project
	 *
	 * @param $post mixed Post
	 *
	 * @return string|WP_Error
	 */
	function ph_get_access_token( $post ) {
		// get post id
		if ( is_a( $post, 'WP_Post' ) ) {
			$post_id = $post->ID;
		} elseif ( is_array( $post ) && ! empty( $post['id'] ) ) {
			$post_id = $post['id'];
		} elseif ( is_int( $post ) ) {
			$post_id = $post;
		}

		// get session
		$access = PH()->session->get( 'project_access' );
		$token  = ! empty( $access[ $post_id ] ) ? $access[ $post_id ] : '';

		// does the visitor have access
		$has_access = ph_visitor_can_access( $token, (int) $post_id );

		// return the access token if they have access
		if ( ! is_wp_error( $has_access ) && $has_access ) {
			return get_post_meta( $post_id, 'access_token', true ) ? get_post_meta( $post_id, 'access_token', true ) : true;
		} else {
			return $has_access;
		}
	}
endif;

if ( ! function_exists( 'ph_get_project_access' ) ) :
	/**
	 * ph_get_project_access
	 */
	function ph_get_project_access( $post_id = 0 ) {
		if ( ! $post_id ) {
			global $post;
			$post_id = is_object( $post ) ? $post->ID : false;
		}

		if ( ! $post_id ) {
			return false;
		}

		// fallback if meta data doesn't exist.
		if ( ! metadata_exists( 'post', $post_id, 'project_access' ) ) {
			return 'link';
		}

		return get_post_meta( $post_id, 'project_access', true );
	}
endif;

if ( ! function_exists( 'ph_should_script_load' ) ) :
	/**
	 * ph_should_script_load
	 */
	function ph_should_script_load() {
		global $post, $wp;

		if ( ! $post || ! ( $post instanceof WP_Post ) ) {
			return new WP_Error( 'invalid_post', __( 'This post doens\'t exist.', 'project-huddle' ), array( 'status' => 403 ) );
		}

		// get comment and url
		$url = get_post_meta( $post->ID, 'ph_website_url', true );

		// check api matches something
		if ( ! ph_public_api_matches( $wp->query_vars['ph_apikey'] ) ) {
			return new WP_Error( 'wrong_api_key', __( 'Wrong API Key. Please check the code you pasted on the site to make sure it\'s correct.', 'project-huddle' ), array( 'status' => 403 ) );
		}

		// double check website domain
		$referrer = isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : false;
		if ( ! $referrer ) {
			return new WP_Error( 'wrong_website_domain', __( 'Website url is wrong in settings. Please double check the url to make sure it exactly matches the site!', 'project-huddle' ), array( 'status' => 403 ) );
		}
		if ( ! ph_domain_matches( $referrer, array( $url ) ) ) {
			return new WP_Error( 'wrong_website_domain', __( 'Website url is wrong in settings. Please double check the url to make sure it exactly matches the site!', 'project-huddle' ) . ' Live site is: ' . esc_url( $_SERVER['HTTP_REFERER'] ) . '. You\'ve set: ' . esc_url( $url ) . '.', array( 'status' => 403 ) );
		}

		// installed
		update_post_meta( get_the_ID(), 'ph_installed', true );
		delete_post_meta( get_the_ID(), 'ph_installation_error' );

		// true
		return true;
	}
endif;

/**
 * Add member to project after they are created
 *
 * @param WP_User         $user    Inserted user object.
 * @param WP_REST_Request $request Request object.
 */
function ph_add_member_after_creating( $user, $request ) {
	if ( ! isset( $request['project_id'] ) || ! isset( $user->user_email ) ) {
		return;
	}

	// add member to project.
	ph_add_project_member( (int) $request['project_id'], $user, false );
}

add_action( 'ph_rest_insert_user', 'ph_add_member_after_creating', 10, 2 );

/**
 * Get all registered PH roles
 *
 * @return array
 */
function ph_get_roles() {
	return apply_filters(
		'ph_roles', array(
			'project_admin',
			'project_editor',
			'project_collaborator',
			'project_client',
		)
	);
}

/**
 * Limit media library access for specific roles
 *
 * This will show only a current user's media library uploads if they can't edit others projects
 * AKA client role
 *
 * @param $query
 *
 * @return mixed
 */
function ph_maybe_show_only_current_users_attachments( $query ) {
	// should only run this on backend media queries
	if ( ! is_admin() ) {
		return $query;
	}

	$limit            = false;
	$user_id          = (int) get_current_user_id();
	$capability_types = (array) ph_get_post_types();

	// get roles and add subscriber
	$roles   = ph_get_roles();
	$roles[] = 'subscriber';

	// only limit based on specific roles, not caps
	foreach ( $roles as $role ) {
		if ( current_user_can( $role ) ) {
			$limit = true;
			break;
		}
	}

	if ( $limit ) {
		foreach ( $capability_types as $capability_type ) {
			if ( ! current_user_can( "edit_others_{$capability_type}s" ) ) {
				if ( apply_filters( 'ph_restrict_media_gallery', true, $user_id ) ) {
					$query['author'] = $user_id;
				}
			}
		}
	}

	return $query;
}

add_filter( 'ajax_query_attachments_args', 'ph_maybe_show_only_current_users_attachments', 999999 );


/**
 * Create or showunique access token
 *
 * @param $post_id
 *
 * @return string Created or stored access token
 */
function ph_access_token( $post_id ) {
	if ( ! $token = get_post_meta( $post_id, 'access_token', true ) ) {
		// generate api key based on post id, url and time
		$token = md5( uniqid( $post_id, true ) );
		update_post_meta( $post_id, 'access_token', esc_sql( $token ) );

		return $token;
	}

	return esc_html( $token );
}

/**
 * Update permalink on website page without hitting save
 *
 * @param string  $return    Sample permalink HTML markup.
 * @param int     $post_id   Post ID.
 * @param string  $new_title New sample permalink title.
 * @param string  $new_slug  New sample permalink slug.
 * @param WP_Post $post      Post object.
 *
 * @return string
 */
function ph_projects_auto_save_permalinks( $return, $post_id, $new_title, $new_slug, $post ) {
	if ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX ) {
		return $return;
	}

	// if not one of our post types, return.
	if ( ! in_array( get_post_type( $post_id ), ph_get_post_types() ) ) {
		return $return;
	}

	// save slug.
	wp_update_post(
		array(
			'ID'        => (int) $post_id,
			'post_name' => $new_slug,
		)
	);

	return $return;
}

add_filter( 'get_sample_permalink_html', 'ph_projects_auto_save_permalinks', 10, 5 );

/**
 * Rename permalink to Magic Share Link
 * Also adds tootip to explain.
 *
 * @param $html
 *
 * @return string
 */
function ph_project_rename_permalink( $html, $post_id, $new_title, $new_slug, $post ) {
	// if not one of our post types, return.
	if ( ! in_array( get_post_type( $post_id ), ph_get_post_types() ) ) {
		return $html;
	}

	wp_enqueue_script( 'clipboard.js' );

	$html = '<div class="ph-magic-link-wrap">
        <strong class="ph-permalink-label">'
	        . __( 'Project Access Link', 'project-huddle' )
	        . '<span class="ph-tooltip-wrap">
            <i class="ion-help-circled"></i>
                <span class="ph-tooltip">
                    ' . __( 'Share this link with someone to give them access to the project.', 'project-huddle' ) . '
                </span>
            </span>'
	        . '</strong>'
	        . $html . '<button type="button" class="ph-copy-link button button-primary button-small hide-if-no-js" aria-label="Copy Access Link" data-clipboard-text="' . esc_attr( get_permalink( $post->ID ) ) . '">Copy</button><span class="clipboard-confirm hidden">' . __( 'Copied!', 'project-huddle' ) . '</span>
    </div>
    <script>
    jQuery(document).ready(function(){
    	var selector = ".ph-copy-link",
    		phClipboard = new Clipboard(selector);
    	
    	phClipboard.on("success", function(e) {
    		var confirm = jQuery(selector).next(".clipboard-confirm");
		    confirm.show();
			setTimeout(function(){
				 confirm.hide();
			}, 1500);
		});
    });
    </script>';

	return $html;
}

add_filter( 'get_sample_permalink_html', 'ph_project_rename_permalink', 10, 5 );

/**
 * Add access token to website project links
 *
 * @param $permalink
 * @param $post
 *
 * @return string
 */
function ph_add_access_token( $permalink, $post ) {
	global $access_token_override;

	// if not one of our post types, return
	if ( ! in_array( get_post_type( $post->ID ), ph_get_post_types() ) && ! in_array( get_post_type( $post->ID ), ph_get_child_post_types() ) ) {
		return $permalink;
	}

	if ( is_admin() || $access_token_override ) {
		$permalink = $permalink . '?access_token=' . ph_access_token( $post->ID );
	}

	return $permalink;
}

add_filter( 'post_type_link', 'ph_add_access_token', 10, 2 );
add_filter( 'preview_post_link', 'ph_add_access_token', 10, 2 );
