<?php
/**
 * Project functions
 *
 * Project functions for ProjectHuddle
 *
 * @package     ProjectHuddle
 * @copyright   Copyright (c) 2015, Andre Gagnon
 * @since       1.0.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Create unique unguessable slugs
 *
 * @param string $slug Slug.
 * @param int    $post_ID Post id.
 * @param string $post_status Post status.
 * @param string $post_type Post type.
 * @param int    $post_parent Parent.
 *
 * @return string
 */
function ph_cross_type_unique_slugs( $slug, $post_ID, $post_status, $post_type, $post_parent ) {
	// only do our post types.
	if ( 'ph-project' != $post_type ) {
		return $slug;
	}

	$post = get_post( $post_ID );

	// if the post hasn't been created yet.
	if ( empty( $post->post_name ) ) {
		$slug = ph_generate_unique_hash( $post_ID );
	}

	return $slug;
}

add_filter( 'wp_unique_post_slug', 'ph_cross_type_unique_slugs', 10, 5 );

/**
 * Generate unique 5 digit hash
 *
 * @param int $id     ID to use for hash.
 * @param int $length Length of hash.
 *
 * @return string
 */
function ph_generate_unique_hash( $id, $length = 6 ) {
	$hex = md5( $id . uniqid( '', true ) );

	$pack = pack( 'H*', $hex );
	$tmp  = base64_encode( $pack );

	$uid = preg_replace( '#(*UTF8)[^A-Za-z0-9]#', '', $tmp );

	$length = max( 4, min( 128, $length ) );

	while ( strlen( $uid ) < $length ) {
		$uid .= ph_gen_uuid( 22 );
	}

	return strtolower( substr( $uid, 0, $length ) );
}

/**
 * Clear mockup transients by id
 *
 * @param int $post_id Post ID.
 */
function ph_mockup_clear_resolved_transients( $post_id ) {
	if ( 'ph_comment_location' !== get_post_type( $post_id ) ) {
		return;
	}

	$project = get_post_meta( $post_id, 'mockup_id', true );

	delete_transient( 'ph_mockup_approval_status_' . $project );
}

/**
 * Query users subscribed projects
 *
 * @uses WP_Query
 *
 * @param array $args Arguments.
 *
 * @return array array of post type objects
 */
function ph_query_users_projects( $args ) {
	$defaults = array(
		'types'       => array( 'all' ),
		'numberposts' => 10,
		'paged'       => 1,
		'user_id'     => get_current_user_id(),
	);

	$args = wp_parse_args( $args, $defaults );

	foreach ( $args['types'] as $key => $type ) {
		if ( 'mockup' === $type ) {
			$args['types'][ $key ] = 'ph-project';
		}
		if ( 'website' === $type ) {
			$args['types'][ $key ] = 'ph-website';
		}
		if ( 'all' === $type ) {
			unset( $args['types'][ $key ] );
			$args[] = 'ph-website';
			$args[] = 'ph-mockup';
		}
	}

	$args['post_type'] = $args['types'];
	unset( $args['types'] );

	// only project members.
	if ( ! isset( $args['meta_query'] ) ) {
		$args['meta_query'] = array();
	}

	$args['meta_query'][] = array(
		'key'     => 'project_members',
		'value'   => $args['user_id'],
		'compare' => 'LIKE',
	);

	// allow filters.
	$args['suppress_filters'] = false;

	// force status.
	if ( empty( $args['post_status'] ) ) {
		$args['post_status'] = 'publish';
	}

	// number of posts.
	if ( ! empty( $args['numberposts'] ) && empty( $args['posts_per_page'] ) ) {
		$args['posts_per_page'] = $args['numberposts'];
	}

	// include post ids.
	if ( ! empty( $args['include'] ) ) {
		$incposts               = wp_parse_id_list( $args['include'] );
		$args['posts_per_page'] = count( $incposts );  // only the number of posts included.
		$args['post__in']       = $incposts;
	} elseif ( ! empty( $args['exclude'] ) ) {
		$args['post__not_in'] = wp_parse_id_list( $args['exclude'] );
	}

	$args['ignore_sticky_posts'] = true;
	$args['no_found_rows']       = true;

	wp_reset_postdata();
	$query = new WP_Query( apply_filters( __FUNCTION__ . '_args', $args ) );
	return $query;
}

/**
 * Get users projects
 *
 * @param array $args Arguments.
 * @return array Array of Post Objects
 */
function ph_get_users_projects( $args ) {
	$query = ph_query_users_projects( $args );
	$posts = $query->posts;
	wp_reset_postdata();
	return $posts;
}

/**
 * Query project images.
 *
 * @param array $args Arguments.
 * @return bool|array
 */
function ph_query_mockup_images( $args = array() ) {
	return ph_query_project_subcollection( $args );
}

/**
 * Get project images as array
 *
 * @param array $args Arguments.
 * @return array Array of Post Objects
 */
function ph_get_mockup_images( $args ) {
	$query = ph_query_mockup_images( $args );
	$posts = $query->posts;
	wp_reset_postdata();
	return $posts;
}

/**
 * Query project images.
 *
 * @param array $args Arguments.
 * @return bool|WP_Query
 */
function ph_query_website_pages( $args = array() ) {
	return ph_query_project_subcollection( $args );
}

/**
 * Get project images as array
 *
 * @param array $args Arguments.
 * @return array Array of Post Objects
 */
function ph_get_website_pages( $args ) {
	$query = ph_query_website_pages( $args );
	$posts = $query->posts;
	wp_reset_postdata();
	return $posts;
}

/**
 * Query project images.
 *
 * @param array  $args Arguments.
 * @return bool|WP_Query
 */
function ph_query_project_subcollection( $args = array() ) {
	global $post;

	$defaults = array(
		'id' => is_a( $post, 'WP_Post' ) ? $post->ID : 0,
	);

	$args = wp_parse_args( $args, $defaults );

	if ( ! $args['id'] ) {
		return false;
	}

	$type = 'all';

	if ( ! isset( $args['post_type'] ) ) {
		switch ( get_post_type( $args['id'] ) ) {
			case 'ph-project':
				$type = 'project_image';
				break;
			case 'ph-website':
				$type = 'ph-webpage';
				break;
			case 'project_image':
				$type = 'ph_comment_location';
				break;
			case 'ph-webpage':
				$type = 'phw_comment_loc';
				break;
			default:
				$type = 'all';
				break;
		}
	}

	if ( ! isset( $args['meta_query'] ) ) {
		$args['meta_query'] = array();
	}

	$args['meta_query'][] = array(
		'key'   => 'parent_id',
		'value' => $args['id'],
	);
	unset( $args['id'] );

	$args['post_type'] = isset( $args['post_type'] ) ? $args['post_type'] : array( $type );

	wp_reset_postdata();
	$query = new WP_Query( apply_filters( __FUNCTION__ . '_args', $args ) );
	return $query;
}

/**
 * Wrapper for get_post_meta in case of key changes.
 */
function ph_get_data( $post_id, $key = '', $single = false ) {
	return apply_filters( "ph_get_{$key}_data", get_post_meta( $post_id, $key, $single ), $post_id, $key, $single );
}

/**
 * Wrapper for update_post_meta in case of key changes.
 */
function ph_update_data( $post_id, $key = '', $data ) {
	$data = apply_filters( "ph_update_{$key}_data", $data, $post_id, $key );

	return update_post_meta( $post_id, $key, $data );
}
