<?php
/**
 * Runs needed database upgrades
 *
 * @package ProjectHuddle
 */

if ( ! class_exists( 'PH_Background_Updater', false ) ) {
	require_once PH_PLUGIN_DIR . 'includes/class-ph-background-updater.php';
}

/**
 * Upgrading Notice
 */
function ph_upgrading_notice() {
	if ( version_compare( get_site_option( 'ph_db_version' ), PH_VERSION, '<' ) ) {
		$updater = new PH_Background_Updater();

		if ( $updater->is_updating() ) {
			if ( ! get_site_option( 'dismissed-ph-updating-' . sanitize_html_class( PH_VERSION ), false ) ) {
				echo '<div class="notice notice-info is-dismissible ph-notice" data-notice="ph-updating-' . sanitize_html_class( PH_VERSION ) . '">
					<p><strong>' . esc_html__( 'ProjectHuddle Database Update', 'project-huddle' ) . '</strong> &#8211; ' . esc_html__( 'Your database is being updated in the background.', 'project-huddle' ) . '</p>
				</div>';
				ph_dismiss_js();
			} ?>
			<?php
		}
	}
}

add_action( 'admin_notices', 'ph_upgrading_notice' );

/**
 * Run plugin updates
 *
 * @return void
 */
function ph_plugin_update() {
	$background_updater = new PH_Background_Updater();
	$update_queued      = false;

	// only run update if it hasn't completed
	if ( ! version_compare( get_site_option( 'ph_db_version' ), PH_VERSION, '<' ) ) {
		return;
	}

	// update already started
	if ( PH_VERSION === get_site_option( 'ph_update_started' ) ) {
		return;
	}

	// flag as already started
	update_site_option( 'ph_update_started', PH_VERSION );

	// update metadata.
	if ( ! get_site_option( 'ph_update_metadata_2_6_0', false ) ) {
		ph_log( 'ph_update_metadata_2_6_0 Update Queued' );
		$background_updater->push_to_queue( 'ph_update_metadata_2_6_0' );
		$update_queued = true;
	}

	// fix missing pages update.
	if ( ! get_site_option( 'ph_fix_missing_pages_2_6_0_5', false ) ) {
		ph_log( 'ph_fix_missing_pages_2_6_0_5 Update Queued' );
		$background_updater->push_to_queue( 'ph_fix_missing_pages_2_6_0_5' );
		$update_queued = true;
	}

	// fix homepage link update.
	if ( ! get_site_option( 'ph_fix_homepage_link_2_6_0_7', false ) ) {
		ph_log( 'ph_fix_homepage_link_2_6_0_7 Update Queued' );
		$background_updater->push_to_queue( 'ph_fix_homepage_link_2_6_0_7' );
		$update_queued = true;
	}

	// 3.0.0 update.
	$update_3_queued = false;
	if ( version_compare( get_site_option( 'ph_db_version' ), '3.0.0', '<' ) ) {
		$update_3_queued = true;

		ph_log( 'Update Data for 3.0.0 Queued' );
		$background_updater->push_to_queue( 'ph_update_metakeys_3_0_0' );

		ph_log( 'Update Data for 3.0.0 Update Queued' );
		$background_updater->push_to_queue( 'ph_update_data_3_0_0' );

		if ( is_multisite() ) {
			ph_log( 'Update data for 3.0.5 Multisite Queued' );
			$background_updater->push_to_queue( 'ph_update_data_3_0_0_multisite' );
		}

		ph_log( 'Update metakeys for 3.0.5 Multisite Queued' );
		$background_updater->push_to_queue( 'ph_project_access_key' );
		$update_queued = true;
	}

	// 3.0.5 Update for multisite
	if ( ! $update_3_queued && version_compare( get_site_option( 'ph_db_version' ), '3.0.5', '<' ) ) {
		ph_log( 'Update metakeys for 3.0.5 Queued (suppliment)' );
		$background_updater->push_to_queue( 'ph_project_access_key' );

		ph_log( 'Update data for 3.0.5 Multisite Queued' );
		$background_updater->push_to_queue( 'ph_update_data_3_0_0_multisite (suppliment)' );

		$update_queued = true;
	}

	if ( $update_queued ) {
		ph_log( 'Updater dispatched.' );
		$background_updater->save()->dispatch();
	}
}

add_action( 'init', 'ph_plugin_update' );

/**
 * Fix homepage link
 *
 * @return void
 */
function ph_fix_homepage_link_2_6_0_7() {
	// phpcs:ignore
	$threads = get_posts(
		array(
			'post_type'      => 'phw_comment_loc',
			'posts_per_page' => - 1,
			'status'         => 'publish',
			'date_query'     => array(
				'after'  => date( '2017-10-6', strtotime( '-10 days' ) ),
				'before' => date( '2017-10-27' ),
			),
			array(
				'key'     => 'website_id',
				'compare' => 'EXISTS',
			),
		)
	);

	// loop through each thread.
	foreach ( $threads as $thread ) {
		if ( ! $thread->parent_id || ! $thread->page_url ) {
			return;
		}

		$page_url = get_post_meta( $thread->parent_id, 'page_url', true );
		if ( ! $page_url ) {
			return;
		}

		// normalize for now.
		$thread_url = str_replace( 'https', 'http', $thread->page_url );
		$page_url   = str_replace( 'https', 'http', $page_url );

		if ( $thread_url !== $page_url ) {
			// find existing page.
			$items = PHW()->page->get_items(
				array(
					'page_url' => $thread->page_url,
				)
			);

			// if page already exists and website project is published.
			if ( isset( $items[0] ) && get_post_status( $items[0]['parent_id'] ) === 'publish' ) {
				update_post_meta( $thread->ID, 'parent_id', $items[0]['id'] );
			} else {
				// create page.
				$page = PHW()->page->create_item(
					array(
						'title'     => $thread->page_url,
						'page_url'  => $thread->page_url,
						'parent_id' => $thread->website_id,
					)
				);

				// update thread parent.
				update_post_meta( $thread->ID, 'parent_id', $page['id'] );
			}
		}
	}
	update_site_option( 'ph_fix_homepage_link_2_6_0_7', true );
}

/**
 * Fix missing pages
 *
 * @return void
 */
function ph_fix_missing_pages_2_6_0_5() {
	// phpcs:ignore
	$threads = get_posts(
		array(
			'post_type'      => 'phw_comment_loc',
			// phpcs:ignore
			'posts_per_page' => - 1,
			// phpcs:ignore
			'meta_query'     => array(
				array(
					'key'   => 'parent_id',
					'value' => 0,
				),
			),
		)
	);

	foreach ( $threads as $thread ) {
		$page_url = get_post_meta( $thread->ID, 'page_url', true );

		// if we have a page url.
		if ( $page_url ) {
			$page_query = new WP_Query(
				array(
					'post_type'  => 'ph-webpage',
					// phpcs:ignore
					'meta_query' => array(
						array(
							'key'   => 'page_url',
							'value' => $page_url,
						),
					),
					'fields'     => 'ids',
				)
			);

			$pages = $page_query->posts;

			// if no matching pages, create one.
			if ( ! $pages ) {
				// find website with same hostname.
				$parsed  = wp_parse_url( $page_url );
				$origin  = $parsed['scheme'] . '://' . $parsed['host'];
				$origin  .= isset( $parsed['port'] ) ? $parsed['port'] : '';
				$website = PHW()->website->get_items(
					array(
						'website_url' => $origin,
					)
				);

				// create page.
				$page = PHW()->page->create_item(
					array(
						'title'     => $page_url,
						'page_url'  => $page_url,
						'parent_id' => $website[0]['id'],
					)
				);

				update_post_meta( $thread->ID, 'parent_id', $page['id'] );
			}

			if ( ! empty( $pages ) && is_array( $pages ) ) {
				update_post_meta( $thread->ID, 'parent_id', $pages[0] );
			}
		}
	}

	update_site_option( 'ph_fix_missing_pages_2_6_0_5', true );
}

/**
 * Update metadat
 *
 * @return void
 */
function ph_update_metadata_2_6_0() {
	$sites = new WP_Query(
		array(
			'post_type'      => 'ph-website',
			'posts_per_page' => - 1,
			'fields'         => 'ids',
			// phpcs:ignore
			'meta_query'     => array(
				'relation' => 'OR',
				array(
					'key'   => 'website_url',
					'value' => 0,
				),
				array(
					'key'     => 'website_url',
					'compare' => 'NOT EXISTS',
				),
			),
		)
	);

	$site_ids = (array) $sites->posts;

	foreach ( $site_ids as $id ) {
		// remove prefix.
		$website_url = get_post_meta( $id, 'ph_website_url', true );
		update_post_meta( $id, 'website_url', $website_url );

		$pages = get_post_meta( $id, 'ph_webpages', true );

		// we're going to use parents instead.
		if ( ! empty( $pages ) ) {
			foreach ( $pages as $page_id => $page_url ) {

				// store generic parent id.
				update_post_meta( $page_id, 'parent_id', $id );

				$locations = (array) get_post_meta( $page_id, 'comment_locations', true );

				// update locations parent.
				foreach ( $locations as $location ) {
					// use parent instead.
					update_post_meta( $location, 'parent_id', $page_id );
				}

				// use page path for just path of page.
				update_post_meta( $page_id, 'page_path', $page_url );

				// have absolute url.
				update_post_meta( $page_id, 'page_url', trailingslashit( $website_url ) . ltrim( $page_url, '/' ) );
			}
		}
	}
	update_site_option( 'ph_update_metadata_2_6_0', true );
	ph_log( 'ph_update_metadata_2_6_0 Update Completed' );
}

/**
 * Rename meta keys
 * Usage: update_meta_key( 'old_key', 'new_key');
 *
 * @param string $old_key Old key.
 * @param string $new_key New key.
 */
function ph_update_meta_key( $old_key = null, $new_key = null ) {
	global $wpdb;

	// multisite
	$query = 'UPDATE ' . $wpdb->base_prefix . "postmeta SET meta_key = '" . $new_key . "' WHERE meta_key = '" . $old_key . "'";
	// phpcs:ignore
	$results = $wpdb->get_results( $query, ARRAY_A );

	// regular
	$query = 'UPDATE ' . $wpdb->prefix . "postmeta SET meta_key = '" . $new_key . "' WHERE meta_key = '" . $old_key . "'";
	// phpcs:ignore
	$results = $wpdb->get_results( $query, ARRAY_A );

	return $results;
}

/**
 * Store license data in database if not stored yet.
 *
 * @return void
 */
function ph_store_license_data() {
	if ( ! get_site_option( 'ph_license_data', false ) ) {

		// store license key data in database
		// data to send in our API request using the id.
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => get_site_option( 'ph_license_key' ),
			'item_id'    => PH_SL_ITEM_ID,
			'url'        => home_url(),
		);

		// Call the custom API.
		$response = wp_remote_post(
			PH_SL_STORE_URL, array(
				'timeout'   => 15,
				'sslverify' => false,
				'body'      => $api_params,
			)
		);

		// make sure the response came back okay.
		if ( is_wp_error( $response ) ) {
			return;
		}
		// decode the license data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// store all data.
		update_site_option( 'ph_license_data', $license_data );
	}
}

function ph_update_metakeys_3_0_0() {
	foreach (
		array(
			'ph_image_alignment'                 => 'alignment',
			'ph_image_size'                      => 'size',
			'ph_image_background_color'          => 'background_color',
			'ph_image_background_image'          => 'background_image',
			'ph_image_background_image_position' => 'background_image_position',
//			'ph_project_access'                  => 'project_access', // we want to skip websites here
			'ph_project_comments'                => 'project_comments',
			'ph_project_sharing'                 => 'project_sharing',
			'ph_retina'                          => 'retina',
			'ph_project_download'                => 'project_download',
			'ph_zoom'                            => 'zoom',
			'ph_project_approval'                => 'project_approval',
			'ph_project_unapproval'              => 'project_unapproval',
		) as $old => $new
	) {
		ph_update_meta_key( $old, $new );
	}

	ph_log( '3.0.0 Meta Keys Updated' );
}

/**
 * Only update the project access key for mockups
 */
function ph_project_access_key() {
	// handle multisite
	if ( is_multisite() ) {
		$sites = get_sites( array(
			'number' => 100,
			'fields' => 'ids'
		) );

		if ( ! empty( $sites ) ) {
			foreach ( $sites as $site_id ) {
				switch_to_blog( $site_id );
				update_mockup_access_key();
				restore_current_blog();
			}
		}
	} else {
		update_mockup_access_key();
	}
}

function update_mockup_access_key() {
	$projects = new WP_Query(
		array(
			'post_type'      => array( 'ph-project' ),
			'posts_per_page' => - 1,
			'fields'         => 'ids'
		)
	);

	if ( ! empty( $projects->posts ) ) {
		foreach ( $projects->posts as $project_id ) {
			// if project access is set, update to new key
			if ( $project_access = get_post_meta( $project_id, 'ph_project_access', true ) ) {
				ph_log( 'Updating ph_project_access for ' . $project_id );
				update_post_meta( $project_id, 'project_access', $project_access );
			} else {
				ph_log( 'ph_project_access not set for ' . $project_id );
			}
		}
	}
}

function ph_update_data_3_0_0() {
	$mockups = new WP_Query(
		array(
			'post_type'      => 'ph-project',
			'posts_per_page' => - 1,
			'fields'         => 'ids',
		)
	);

	$mockup_ids = (array) $mockups->posts;

	if ( ! empty( $mockup_ids ) ) {
		foreach ( $mockup_ids as $id ) {
			// project members.
			$emails  = (array) get_post_meta( (int) $id, 'ph_project_emails_enable', true );
			$emails  = array_unique( $emails );
			$members = array();

			// Store user ids in ids array.
			foreach ( $emails as $key => $email ) {
				if ( ! $email ) {
					continue;
				}

				// Get user by email.
				$user = get_user_by( 'email', $email );

				// users only.
				if ( $user && is_a( $user, 'WP_User' ) ) {
					// store user in array.
					$members[] = $user;
				}
			}

			// update in new post meta.
			if ( ! empty( $members ) ) {
				$user_ids = array();
				foreach ( $members as $member ) {
					if ( is_a( $member, 'WP_User' ) ) {
						$user_ids[] = $member->ID;
					}
				}

				$ids = (array) get_post_meta( $id, 'project_members', true );
				$ids = array_unique( array_merge( $user_ids, $ids ) );
				update_post_meta( $id, 'project_members', $ids );
			}

			// images.
			$image_ids  = get_post_meta( $id, 'ph_project_images', true );
			$menu_order = 0;

			if ( ! empty( $image_ids ) ) {
				foreach ( $image_ids as $image_id ) {
					wp_update_post(
						array(
							'ID'         => $image_id,
							'menu_order' => $menu_order ++, // store order.
							'meta_input' => array(
								'parent_id' => $id,
							),
						)
					);

					// get approval data.
					$approval = (array) get_post_meta( (int) $image_id, 'ph_approved', true );

					// get last approval.
					if ( is_array( $approval ) ) {
						end( $approval );         // move the internal pointer to the end of the array.
						$key = key( $approval );  // fetches the key of the element pointed to by the internal pointer.
					}

					// store new data format.
					if ( isset( $key ) && isset( $approval[ $key ] ) && isset( $approval[ $key ]['approval'] ) ) {
						$value  = (bool) $approval[ $key ]['approval'];
						$status = $value ? ph_approval_completed_status() : ph_get_default_approval_status();
						wp_set_object_terms( $image_id, $status, 'ph_approval', true );
					}

					// get the array of comment location ids for the image.
					$comment_locations = get_post_meta( $image_id, 'comment_locations', true );

					if ( ! empty( $comment_locations ) ) {
						foreach ( $comment_locations as $comment_id ) {
							// store image id in comment.
							update_post_meta( $comment_id, 'parent_id', $image_id );
						}
					}
				}
			}
		}
	} else {
		ph_log( 'No mockup posts to update for 3.0.0' );
	}

	ph_log( '3.0.0 Data Updated' );
}

function ph_update_data_3_0_0_multisite() {
    // handle multisite
	if ( is_multisite() ) {
		$sites = get_sites( array(
			'number' => 100,
			'fields' => 'ids'
		) );

		if ( ! empty( $sites ) ) {
			foreach ( $sites as $site_id ) {
				switch_to_blog( $site_id );
				ph_log( 'Updating Data for 3.0.0' );
				ph_update_data_3_0_0();
				restore_current_blog();
			}
		}
	}
}

/**
 * General update stuff on every update.
 *
 * @return void
 */
function ph_general_update() {
	$roles = new PH_Roles();
	$roles->add_roles();
	$roles->add_caps();

	// store license data.
	ph_store_license_data();

	/* Restore original Post Data */
	wp_reset_postdata();

	// flush rewrite rules.
	flush_rewrite_rules();

	// update option.
	update_site_option( 'ph_db_version', PH_VERSION );

	ph_log( 'Permalinks and roles flushed as part of ' . PH_VERSION . ' update.' );
}
